#========================================================================================
# package      DE_EPAGES::CyberMUT::API::Cartridge
# base         DE_EPAGES::Payment::API::PaymentInstaller
# state        public
#----------------------------------------------------------------------------------------
# description  This is the main cartridge class for install/patch/uninstall.
#========================================================================================
package DE_EPAGES::CyberMUT::API::Cartridge;
use base DE_EPAGES::Payment::API::PaymentInstaller;

use strict;
use DE_EPAGES::Cartridge::API::Table::Cartridge qw (DeleteCartridge ExistsCartridgeByPackage);
use DE_EPAGES::Core::API::CartridgeConfig qw (CartridgeConfig);
use DE_EPAGES::Core::API::File qw (DeleteDirectory DeleteFile ExistsDirectory ExistsFile RunOnGlob);
use DE_EPAGES::Core::API::Log qw (GetLog);
use DE_EPAGES::Core::API::PerlTools qw (UsePackage);
use DE_EPAGES::Database::API::Config qw (StoreType);
use DE_EPAGES::Database::API::Connection qw (GetCurrentDBHandle);
use DE_EPAGES::Object::API::Language qw (GetPKeyLanguageByCode);
use DE_EPAGES::Object::API::Factory qw (LoadClassByAlias LoadObjectByPath LoadRootObject);
use DE_EPAGES::Object::API::XML::ObjectDeleteHandler qw (DeleteObjects);
use DE_EPAGES::Trigger::API::XML::HookDeleteHandler qw (DeleteHooks);

#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer = DE_EPAGES::CyberMUT::API::Cartridge->new(%options);
#----------------------------------------------------------------------------------------
# description  Creates a new installer object of DE_EPAGES::CyberMUT.
#----------------------------------------------------------------------------------------
# input        %options | options for un/install
#               <ul>
#                 <li>IsRecursive - recursive un/installation of required or dependent
#                     cartridges
#               </ul>
#               | hash
# return       $Installer | cartridge installer | object
#========================================================================================
sub new {
    my ($class, %options) = @_;

    my $self = __PACKAGE__->SUPER::new(
        %options,
        'CartridgeDirectory' => 'DE_EPAGES/CyberMUT',
        'Version'            => '7.63.0',
        'Patches'            => ['6.15.1','6.15.2','6.15.3','6.15.4','6.16.0','6.16.1','6.16.2','6.16.3','6.17.0','6.17.1','6.17.2','6.17.3','6.17.4','6.17.5','6.17.6','6.17.7','6.17.8','6.17.9','6.17.9.1','6.17.9.2','6.17.9.3','6.17.10','6.17.11','6.17.12','6.17.12.1','6.17.12.2','6.17.13','6.17.13.1','6.17.13.2','6.17.14','6.17.14.1','6.17.15','6.17.15.1','6.17.16','6.17.17','6.17.17.1','6.17.18','6.17.19','6.17.20','6.17.20.1','6.17.21','6.17.21.1','6.17.22','6.17.23','6.17.24','6.17.24.1','6.17.25','6.17.26','6.17.26.1','6.17.27','6.17.27.1','6.17.28','6.17.28.1','6.17.29','6.17.29.1','6.17.30','6.17.31','6.17.32','6.17.33','6.17.33.1','6.17.34','6.17.34.1','6.17.35','6.17.36','6.17.36.1','6.17.37','6.17.38','6.17.38.1','6.17.39','6.17.39.1','6.17.40','6.17.40.1','6.17.41','6.17.41.1','6.17.42','6.17.43','6.17.43.1','6.17.44','6.17.45','6.17.45.1','6.17.46','6.17.47','6.17.47.1','6.17.48','6.17.48.1','6.17.49','6.17.49.1','6.17.50','6.17.51','6.17.51.1','6.17.52','6.17.53','7.0.0','7.1.0','7.2.0','7.3.0','7.4.0','7.5.0','7.6.0','7.7.0','7.8.0','7.9.0','7.10.0','7.11.0','7.12.0','7.13.0','7.14.0','7.15.0','7.16.0','7.17.0','7.18.0','7.19.0','7.20.0','7.21.0','7.22.0','7.23.0','7.24.0','7.25.0','7.26.0','7.27.0','7.28.0','7.29.0','7.30.0','7.31.0','7.32.0','7.33.0','7.34.0','7.35.0','7.36.0','7.37.0','7.38.0','7.39.0','7.40.0','7.41.0','7.42.0','7.43.0','7.44.0','7.45.0','7.46.0','7.47.0','7.48.0','7.49.0','7.50.0','7.51.0','7.52.0','7.53.0','7.54.0','7.55.0','7.56.0','7.57.0','7.58.0','7.59.0','7.60.0','7.61.0','7.62.0'],
    );
    return bless $self, $class;
}

#========================================================================================
# function     install
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->install;
#----------------------------------------------------------------------------------------
# description  Installs the cartridge.
#========================================================================================
sub install {
    my $self = shift;

    $self->startInstall;

    #--- if old CyberMUT cartridge is installed the old cartridge must be removed, but the data must be kept!
    my $existsOldCartridge = ExistsCartridgeByPackage('FR_EUROINFORMATION::CyberMUT');
    my $OldCartridge;
    if ($existsOldCartridge) {
        UsePackage('FR_EUROINFORMATION::CyberMUT::API::Cartridge');
        $OldCartridge = FR_EUROINFORMATION::CyberMUT::API::Cartridge->new;
        $self->_convertOldCartridge($OldCartridge);
    }

    $self->SUPER::install;
    $self->finishInstall;

    if ($existsOldCartridge) {
        $self->_removeOldCartridge($OldCartridge);
    }
    return;
}

#========================================================================================
sub _convertOldCartridge {
    my $self = shift;
    my ($OldCartridge) = @_;

    my $xmlDir = $OldCartridge->xmlDirectory;

    #--- remove all permissions, actions, page types, forms, hooks, nav elements, searches
    GetLog()->info('remove database entries from old cybermut cartridge');
    RunOnGlob($xmlDir, "Hooks*.xml", sub { DeleteHooks($_) }, {reverse=>1, force=>$self->force });
    $self->call( sub { DeleteObjects("$xmlDir/Permissions.xml") });
    RunOnGlob($xmlDir, "PageTypes*.xml", sub { DeleteObjects($_) }, {reverse=>1, force=>$self->force });
    RunOnGlob($xmlDir, "Actions*.xml", sub { DeleteObjects($_) }, {reverse=>1, force=>$self->force });
    RunOnGlob($xmlDir, "Forms*.xml", sub { DeleteObjects($_) }, {reverse=>1, force=>$self->force });
    # need not be deleted because new cartridge contains the same nav element:
    # RunOnGlob($xmlDir, "NavElements*.xml", sub { DeleteObjects($_) }, {reverse=>1, force=>$self->force });
    RunOnGlob($xmlDir, "Search*.xml", sub { DeleteObjects($_) }, {reverse=>1, force=>$self->force });

    # this solves the forms problem, but I don't know why
    my $ClassForm = LoadClassByAlias('Form');
    $ClassForm->getObjects('ByFormClassIDName', [LoadClassByAlias('Shop')->id, 'SearchCyberMUTTransactions']);

    #--- TestMode => IsInTestMode
    my $PmClass = LoadClassByAlias('PaymentMethodCyberMUT');
    GetLog()->info('set IsInTestMode attribute for objects of class PaymentMethodCyberMUT');
    foreach my $PaymentMethod (@{$PmClass->objects}) {
        $PaymentMethod->set({'IsInTestMode' => $PaymentMethod->get('TestMode') eq '100'  ?  1 : 0});
    }

    #--- remove/rename attributes of class PaymentMethodCyberMUT
    GetLog()->info('remove/rename attributes of class PaymentMethodCyberMUT');
    $PmClass->directAttribute('AuthMode')->delete;
    $PmClass->directAttribute('TestMode')->delete;
    $PmClass->directAttribute('URLHomePage')->delete;
    $PmClass->directAttribute('Passerelle')->set({'Alias' => 'Gateway'});

    #--- remove name and description of attributes of class PaymentMethodCyberMUT
    my $langidFR = GetPKeyLanguageByCode('fr');
    my $langidEN = GetPKeyLanguageByCode('en');
    foreach my $AttributeAlias (qw(numTPE codeSite cleMAC Gateway)) {
        my $Attribute = $PmClass->directAttribute($AttributeAlias);
        $Attribute->set({'Name' => undef, 'Description' => undef}, $langidFR);
        $Attribute->set({'Name' => undef, 'Description' => undef}, $langidEN);
    }

    my $dbi = GetCurrentDBHandle();

    if ($dbi->driverName() eq 'Sybase') {
        #--- drop stored procedure
        GetLog()->info('drop procedure i_LineItemPaymentCyberMUT');
        $dbi->dropProcedure('i_LineItemPaymentCyberMUT') if $dbi->existsProcedure('i_LineItemPaymentCyberMUT');

        #--- copy transaction data
        if ($dbi->existsTable('lineitempaymentcybermut')) {
            GetLog()->info("move data from 'lineitempaymentcybermut' to 'lineitempaymentexternal'");
            $dbi->do(<<END_SQL);
            INSERT INTO lineitempaymentexternal (lineitempaymentexternalid, currencyid, amount, transid,        returncode, message,            transtime,       authcode,                          issecure)
                                          SELECT lineitempaymentcybermutid, currencyid, amount, orderreference, returncode, extendedreturncode, transactiontime, convert(varchar, authorisationno), 0
                FROM lineitempaymentcybermut
END_SQL
        }
    }
    elsif ($dbi->driverName() eq 'mysql') {
        #--- copy transaction data
        if ($dbi->existsTable('lineitempaymentcybermut')) {
            GetLog()->info("move data from 'lineitempaymentcybermut' to 'lineitempaymentexternal'");
            $dbi->do(<<END_SQL);
            INSERT INTO lineitempaymentexternal (lineitempaymentexternalid, currencyid, amount, transid,        returncode, message,            transtime,       authcode,        issecure)
                                          SELECT lineitempaymentcybermutid, currencyid, amount, orderreference, returncode, extendedreturncode, transactiontime, authorisationno, 0
                FROM lineitempaymentcybermut
END_SQL
        }
    }

    #---remove French name of class LineItemPaymentCyberMUT
    my $LineItemClass = LoadClassByAlias('LineItemPaymentCyberMUT');
    $LineItemClass->set({'Name' => undef, 'Description' => undef}, $langidFR);

    #--- remove attributes of class LineItemPaymentCyberMUT
    foreach my $AttributeAlias (qw(
        ShopID
        CurrencyID
        LanguageID
        Amount
        OrderReference
        ReturnCode
        ExtendedReturnCode
        TransactionTime
        SiteCode
        Version
        AdditionalText
        AuthorisationNo
        AuthMode
    )) {
        $LineItemClass->directAttribute($AttributeAlias)->delete;
    }

    #--- remove database table
    GetLog()->info("drop table 'lineitempaymentcybermut'");
    $dbi->dropTable('lineitempaymentcybermut');

    #--- remove system attributes
    my $SystemClass = LoadClassByAlias('System');
    foreach my $AttributeAlias (qw(
        CyberMUT_UrlPaiementTest
        CyberMUT_UrlPaiementProd
        CyberMUT_UrlTdbTest
        CyberMUT_UrlTdbProd
        CIC_UrlPaiementTest
        CIC_UrlPaiementProd
        CIC_UrlTdbTest
        CIC_UrlTdbProd
        CyberMUT_authMode
        CyberMUT_Passerelle
        CyberMUT_VersionKit
        CyberMUT_logging
    )) {
        $SystemClass->attribute($AttributeAlias)->delete;
    }

    #--- remove shop attribute
    LoadClassByAlias('Shop')->attribute('CyberMUTLogos')->delete;

    #--- remove name of payment logo
    my $Logo = LoadObjectByPath('/PaymentLogos/CYB_CB');
    $Logo->set({'Name' => undef}, $langidEN);
    $Logo->set({'Name' => undef}, $langidFR);

    #--- remove name of payment type
    my $PaymentType = LoadObjectByPath('/PaymentTypes/CyberMUT');
    $PaymentType->set({'Name' => undef}, $langidFR);

    #--- remove data files
    GetLog()->info("remove cybermut data files");

    # Data/Public/cybermut_clearsearchform.js and Data/Public/SF/PaymentMethods/ico_m_cyb_cb.gif
    # before 6.10.3 the location was $ENV{'EPAGES_WEBROOT'}/$Store/cybermut_clearsearchform.js,
    # from 6.10.3 on it is $ENV{'EPAGES_WEBROOT'}/StoreTypes/$Version/$StoreType/cybermut_clearsearchform.js
    # both locations will be looked up for the files to remove
    my $System    = LoadRootObject();
    my $StoreName = $System->get('StoreName');
    my $StoreType = StoreType($StoreName);
    my $Version   = $System->get('EpagesVersion');
    foreach my $File (
        "$ENV{'EPAGES_WEBROOT'}/$StoreName/cybermut_clearsearchform.js",
        "$ENV{'EPAGES_WEBROOT'}/StoreTypes/$Version/$StoreType/cybermut_clearsearchform.js",
        "$ENV{'EPAGES_WEBROOT'}/$StoreName/SF/PaymentMethods/ico_m_cyb_cb.gif",
        "$ENV{'EPAGES_WEBROOT'}/StoreTypes/$Version/$StoreType/SF/PaymentMethods/ico_m_cyb_cb.gif"
    ) {
        DeleteFile($File) if (ExistsFile($File));
    }

    # Data/WebRoot/Doc/Help/en/CyberMUT/*, Data/WebRoot/Doc/Help/fr/CyberMUT/*
    foreach my $dir ("$ENV{'EPAGES_WEBROOT'}/Doc/Help/en/CyberMUT", "$ENV{'EPAGES_WEBROOT'}/Doc/Help/fr/CyberMUT") {
        DeleteDirectory($dir, {'recursive' => 1}) if (ExistsDirectory($dir));
    }

    return;
}

#========================================================================================
sub _removeOldCartridge {
    my $self = shift;
    my ($OldCartridge) = @_;

    # remove cartridge from database
    DeleteCartridge($OldCartridge->id);

    # remove cartridge from Store.conf
    my $CartridgeConfig = CartridgeConfig(LoadRootObject()->get('StoreName'));
    $CartridgeConfig->delete('Cartridges', 'FR_EUROINFORMATION::CyberMUT');
    $CartridgeConfig->rewrite;

    return;
}

#========================================================================================
# function     uninstall
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->uninstall;
#----------------------------------------------------------------------------------------
# description  Uninstalls the cartridge.
#========================================================================================
sub uninstall {
    my $self = shift;

    $self->startUninstall;
    $self->SUPER::uninstall;
    $self->finishUninstall;
    return;
}

1;
