#========================================================================================
# package      DE_EPAGES::GoogleSmartShopping::API::GoogleApiClient
# state        public
# description  Wrapper for GoogleApiClient requests with token refresh if expired
#========================================================================================
package DE_EPAGES::GoogleSmartShopping::API::GoogleApiClient;

use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::EPagesJ::API::Object::RestClient;
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::GoogleSmartShopping::API::Tools qw ( LogGoogleSmartShopping );
use HTTP::Request;
use HTTP::Status qw ( is_server_error is_client_error );
use LWP::UserAgent;
use Encode qw ( encode );
use JSON::MaybeXS qw ( decode_json );
use DE_EPAGES::GoogleSmartShopping::API::Constants qw (
    EPAGESJ_BASE_URI
);

use strict;

use constant POST_METHOD => 'POST';
use constant PUT_METHOD => 'PUT';
use constant GET_METHOD => 'GET';
use constant DELETE_METHOD => 'DELETE';
use constant HTTP_STATUS_UNAUTHORIZED => 401;
use constant UNAUTHORIZED_ERROR_DESCRIPTION => 'Invalid Credentials';
use constant REQUEST_FAILED_CODE => 'REQUEST_FAILED';

sub new {
    my $class = shift;
    my ($hSettings)  = @_;

    my $self  = bless {
        Shop => $hSettings->{'Shop'},
    }, $class;

    return $self;
}

sub doGet {
    my $self = shift;
    my ($Url, $Headers, $hQueryParams) = @_;

    $self->loadGoogleSettings();

    my $DecodedResponse;
    eval {
        $DecodedResponse =  $self->_doRequest(GET_METHOD, $Url, $Headers, $hQueryParams);
    };
    if (ExistsError()) {
        my $Error = $self->_GetError();
        if($self->_isTokenExpired($Error)) {
            LogGoogleSmartShopping( 'Authentication Error', $Error->message());
            $self->_doRefreshAccessToken();
            return $DecodedResponse =  $self->_doRequest(GET_METHOD, $Url, $Headers, $hQueryParams);
        }
        Error($Error);
    }
    return $DecodedResponse;
}

sub doPost{
    my $self = shift;
    my ($Url, $Headers, $RequestContent) = @_;

    $self->loadGoogleSettings();

    my $DecodedResponse;
    eval {
        $DecodedResponse =  $self->_doRequest(POST_METHOD, $Url, $Headers, $RequestContent);
    };
    if (ExistsError()) {
        my $Error = $self->_GetError();
        if($self->_isTokenExpired($Error)) {
            LogGoogleSmartShopping( 'Authentication Error', $Error->message());
            $self->_doRefreshAccessToken();
            return $DecodedResponse =  $self->_doRequest(POST_METHOD, $Url, $Headers, $RequestContent);
        }
        Error($Error);
    }
    return $DecodedResponse;
}

sub _doRequest {
    my $self = shift;
    my ($HttpMethod, $Url, $Headers, $hData)  = @_;

    if ($self->_isRequestToEpagesj($Url)) {
        return $self->_doEpagesjRequest($HttpMethod, $Url, $Headers, $hData);
    }
    return $self->_doExternalRequest($HttpMethod, $Url, $Headers, $hData);
}

sub _doExternalRequest {
    my $self = shift;
    my ($HttpMethod, $Url, $Headers, $hData)  = @_;

    my $Request = HTTP::Request->new($HttpMethod, $Url, $Headers, $hData);
    my $AccessToken = $self->{'GoogleSmartShoppingConfig'}->get('AccessToken');
    $Request->header(Authorization => "Bearer $AccessToken");
    my $UserAgent = LWP::UserAgent->new();

    my $Response = $UserAgent->request($Request);

    my $hResponse = decode_json($Response->decoded_content());
    if ($hResponse->{'error'}) {
        Error('EXTERNAL_REQUEST_ERROR', {
            error => $hResponse->{'error'},
            error_description => $hResponse->{'error_description'}
        });
    }
    return $hResponse;
}

sub _doEpagesjRequest {
    my $self = shift;
    my ($HttpMethod, $EpjEndpoint, $hRequestHeader, $hData)  = @_;

    $hRequestHeader->{'X-AccessToken'} = $self->{'GoogleSmartShoppingConfig'}->get('AccessToken');

    if ($hRequestHeader->{'AdminToken'}) {
        $hRequestHeader->{'X-AccessToken'} = $hRequestHeader->{'AdminToken'};
        # We should not delete the AdminToken afterwards
        # In case an unauthorized response comes, the AccessToken could be used after it is refreshed
    }

    my $RestClient = $self->_createEpagesjRestClient($self->{'Shop'});

    my $Url = $RestClient->{'BaseURL'}.$EpjEndpoint;
    my $RequestUrl = URI::URL->new($Url);

    my $Response;
    if ($HttpMethod eq POST_METHOD) {
        my $JsonRequestPayload = $RestClient->encodeRequest($hData);
        $Response = $RestClient->client()->post($RequestUrl, $hRequestHeader, $JsonRequestPayload);
    } elsif($HttpMethod eq PUT_METHOD) {
        my $JsonRequestPayload = $RestClient->encodeRequest($hData);
        $Response = $RestClient->client()->put($RequestUrl, $hRequestHeader, $JsonRequestPayload);
    } elsif($HttpMethod eq DELETE_METHOD) {
        $RequestUrl->query_form($hData);
        $Response = $RestClient->client()->delete($RequestUrl, $hRequestHeader);
    } else {
        $RequestUrl->query_form($hData);
        $Response = $RestClient->client()->get($RequestUrl, $hRequestHeader);
    }

    return $self->_decodeFromResponse($RestClient, $Response);
}

sub _isRequestToEpagesj {
    my $self = shift;
    my ($Url)  = @_;

    if (index($Url, EPAGESJ_BASE_URI) != -1) {
        return 1;
    }
    return 0;
}

#========================================================================================
# function     _doRefreshAccessToken
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hResponse = $self->doRefreshAccessToken();
#----------------------------------------------------------------------------------------
# description  It refreshes accessToken using a refreshToken.
#               https://developers.google.com/identity/protocols/oauth2/web-server#offline
#----------------------------------------------------------------------------------------
# return       $hResponse | hash with response | hashref
#========================================================================================
sub _doRefreshAccessToken {
    my $self = shift;

    my $AccessToken = $self->GetAccessToken($self->{'GoogleSmartShoppingConfig'}->get('RefreshToken'));

    $self->{'GoogleSmartShoppingConfig'}->set({ 'AccessToken' => $AccessToken });

    return;
}

#========================================================================================
# function     GetAccessToken
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetAccessToken( $RefreshToken );
#----------------------------------------------------------------------------------------
# description  It gets an accessToken using a refreshToken.
#----------------------------------------------------------------------------------------
# input        $RefreshToken | RefreshToken used to get an AccessToken | string
# return       $AccessToken | returns the AccessToken | string
#========================================================================================
sub GetAccessToken {
    my $self = shift;
    my ($RefreshToken) = @_;

    my $Url = $self->{oauth_token_url};
    my $Header = [ 'Content-Type' => 'application/x-www-form-urlencoded' ];
    my $RequestContent = sprintf(
        'client_id=%s&client_secret=%s&refresh_token=%s&grant_type=%s',
        $self->{'client_id'},
        $self->{'client_secret'},
        $RefreshToken,
        'refresh_token'
    );

    my $request = HTTP::Request->new('POST', $Url, $Header, encode("UTF-8", $RequestContent));
    my $userAgent = LWP::UserAgent->new();
    my $response = $userAgent->request($request);

    my $ErrorMessagePrefix = $self->{'GoogleSmartShopping_AdminRefreshToken'} eq $RefreshToken ? "Admin" : "";

    if ( is_server_error($response->code) ) {
        Error('Internal Server Error');
    }

    my $hResponse = decode_json($response->decoded_content());

    if (is_client_error($response->code) &&
        isValid($hResponse->{'error_description'}) &&
        $hResponse->{'error_description'} eq 'Token has been expired or revoked.'
    ){
        LogGoogleSmartShopping($ErrorMessagePrefix.'RefreshToken expired or has been revoked', {'Response' => $hResponse});
        Error($ErrorMessagePrefix.'RefreshToken expired or has been revoked');
    }

    if (defined $hResponse && isValid($hResponse->{'access_token'})) {
        LogGoogleSmartShopping($ErrorMessagePrefix.'AccessToken refreshed.', {});
        return $hResponse->{'access_token'};
    }

    LogGoogleSmartShopping( $ErrorMessagePrefix.'AccessToken not refreshed.', {'Response' => $hResponse});
    Error($ErrorMessagePrefix.'AccessToken not refreshed.');
}

# -----------------------------------------------------------
sub _createEpagesjRestClient {
    my $self = shift;
    my ($Shop) = @_;

    my $System = LoadRootObject();
    my $hArgs = {};

    $hArgs->{BaseURL} = $System->get('OAuthServiceBaseUrl') // $System->get('EPagesJBaseURL');
    $hArgs->{ShopType} = $Shop->get('ShopTypeRef')->alias();
    $hArgs->{Shop} = $Shop;
    $hArgs->{BusinessUnit} = $System->get('ProviderID');

    return DE_EPAGES::EPagesJ::API::Object::RestClient->create( $System, $hArgs );
}

# -----------------------------------------------------------
sub _isTokenExpired {
    my $self = shift;
    my ($Error) = @_;

    my $isTokenExpired = 0;

    if (defined($Error)) {
        if (defined $Error->vars->{'Status'} && $Error->vars->{'Status'} eq HTTP_STATUS_UNAUTHORIZED) {
            my $hResponse = decode_json($Error->vars->{'Response'});
            $isTokenExpired = (
                defined $hResponse->{'errors'} &&
                    isValid($hResponse->{'message'}) &&
                    index($hResponse->{'message'}, 'Expected OAuth 2 access token') != -1
            );
        } elsif (defined $Error->vars->{'error_description'} && $Error->vars->{'error_description'} eq UNAUTHORIZED_ERROR_DESCRIPTION) {
            $isTokenExpired = 1;
        }
    }
    return $isTokenExpired;
}

# -----------------------------------------------------------
sub _GetError {
    my $self = shift;

    return GetError();
}
# -----------------------------------------------------------
sub _decodeFromResponse {
    my $self = shift;
    my ($Client, $Response) = @_;

    my $ResponseContent = $Response->decoded_content();
    # Empty content responses fail to decode (i.e. "No Content - 204")
    return $ResponseContent unless (isValid($ResponseContent));

    return $Client->decodeResponse( $ResponseContent );
}

sub loadGoogleSettings {
    my $self = shift;

    if (isValid($self->{'client_id'})) {
        return;
    }

    my $hSystemSettings = LoadRootObject()->get([qw(
        GoogleSmartShopping_AppClientID
        GoogleSmartShopping_AppClientSecret
        GoogleSmartShopping_AuthTokenURL
        GoogleSmartShopping_AppRedirectURL
        GoogleSmartShopping_AdminRefreshToken
        GoogleSmartShopping_MerchantCenterMultiClientAccountID
    )]);

    $self->{'client_id'} = $hSystemSettings->{'GoogleSmartShopping_AppClientID'};
    $self->{'client_secret'} = $hSystemSettings->{'GoogleSmartShopping_AppClientSecret'};
    $self->{'oauth_token_url'} = $hSystemSettings->{'GoogleSmartShopping_AuthTokenURL'};
    $self->{'oauth_redirect_url'} = $hSystemSettings->{'GoogleSmartShopping_AppRedirectURL'};
    $self->{'GoogleSmartShopping_AdminRefreshToken'} = $hSystemSettings->{'GoogleSmartShopping_AdminRefreshToken'};
    $self->{'GoogleSmartShopping_MerchantCenterMultiClientAccountID'} = $hSystemSettings->{'GoogleSmartShopping_MerchantCenterMultiClientAccountID'};

    $self->{'GoogleSmartShoppingConfig'} = $self->{'Shop'}->existsChild('GoogleSmartShoppingConfig') ?
        $self->{'Shop'}->get('GoogleSmartShoppingConfig') : undef;

    return;
}

1;