#========================================================================================
# package      DE_EPAGES::GoogleSmartShopping::API::Object::GoogleSmartShoppingConfig
# state        public
# base         DE_EPAGES::Shop::API::Object::ShopObject
# description  object configuration interface for GoogleSmartShoppingConfig.
#========================================================================================
package DE_EPAGES::GoogleSmartShopping::API::Object::GoogleSmartShoppingConfig;
use base DE_EPAGES::Shop::API::Object::ShopObject;

use strict;

use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::GoogleSmartShopping::API::Constants qw ( MERCHANT_CENTER_REQUIREMENTS_CHECK_LIST );
use DE_EPAGES::Trigger::API::Trigger qw (TriggerHook);

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $GoogleSmartShoppingConfig = DE_EPAGES::GoogleSmartShopping::API::Object::GoogleSmartShoppingConfig->insert($hInfo);
# example      my $Class = LoadClassByAlias('GoogleSmartShoppingConfig');
#               my $GoogleSmartShoppingConfig = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook 'OBJ_InsertGoogleSmartShoppingConfig' is triggered.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
# return       $GoogleSmartShoppingConfig | new object | object
# hook         OBJ_InsertGoogleSmartShoppingConfig | hook parameter keys :
#               <ul>
#                 <li>Object                      - this object                             - object
#                 <li>GoogleSmartShoppingConfigID - Google Smart Shopping Config identifier - int
#               </ul>
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;

    my $self = $class->SUPER::insert($hInfo);

    TriggerHook(
        'OBJ_InsertGoogleSmartShoppingConfig',
        {'Object' => $self, 'GoogleSmartShoppingConfigID' => $self->id }
    );

    return $self;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $GoogleSmartShoppingConfig->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeleteGoogleSmartShoppingConfig' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeleteGoogleSmartShoppingConfig | hook parameter keys :
#               <ul>
#                 <li>Object                      - this object                             - object
#                 <li>GoogleSmartShoppingConfigID - Google Smart Shopping Config identifier - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    TriggerHook(
        'OBJ_DeleteGoogleSmartShoppingConfig',
        { 'Object' => $self, 'GoogleSmartShoppingConfigID' => $self->id(), }
    );

    $self->SUPER::delete;
    return;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $GoogleSmartShoppingConfig->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdateGoogleSmartShoppingConfig'
#               and 'OBJ_AfterUpdateGoogleSmartShoppingConfig'.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
# input        $LanguageID | (optional; required for localized attributes) language id | int
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdateGoogleSmartShoppingConfig | hook parameter keys :
#               <ul>
#                 <li>Object                      - this object                             - object
#                 <li>GoogleSmartShoppingConfigID - Google Smart Shopping Config identifier - int
#                 <li>Values                      - new values for object                   - ref.hash
#                 <li>LanguageID                  - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdateGoogleSmartShoppingConfig | hook parameter keys :
#               <ul>
#                 <li>Object                      - this object                             - object
#                 <li>GoogleSmartShoppingConfigID - Google Smart Shopping Config identifier - int
#                 <li>Values                      - updated values                          - ref.hash
#                 <li>LanguageID                  - values updated of this language         - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    TriggerHook('OBJ_BeforeUpdateGoogleSmartShoppingConfig', {
            'Object'=>$self,
            'Values'=> $hValues,
            'GoogleSmartShoppingConfigID' => $self->id(),
            'LanguageID' => $LanguageID,
    });

    $self->SUPER::set($hValues, $LanguageID);

    TriggerHook('OBJ_AfterUpdateGoogleSmartShoppingConfig', {
            'Object'=>$self,
            'Values'=> $hValues,
            'GoogleSmartShoppingConfigID' => $self->id(),
            'LanguageID' => $LanguageID,
    });

    return;
}

#========================================================================================
# function     areGoogleMerchantCenterRequirementsMet
# state        public
#----------------------------------------------------------------------------------------
# syntax       $GoogleSmartShoppingConfig->areGoogleMerchantCenterRequirementsMet();
#----------------------------------------------------------------------------------------
# description  This function returns boolean value stating if all requirements for the
#               generation or assignment of a Merchant Center account have been met.
#----------------------------------------------------------------------------------------
# return       $areRequirementsMet | 0/1 merchant center requirements all true | boolean
#========================================================================================
sub areGoogleMerchantCenterRequirementsMet {
    my $self = shift;

    my $hRequirements = $self->getGoogleMerchantCenterRequirementsStatus();
    foreach my $Requirement (keys %{$hRequirements}) {
        return 0 unless ($hRequirements->{$Requirement});
    }
    return 1;
}

#========================================================================================
# function     getGoogleMerchantCenterRequirementsStatus
# state        public
#----------------------------------------------------------------------------------------
# syntax       $GoogleSmartShoppingConfig->getGoogleMerchantCenterRequirementsStatus();
#----------------------------------------------------------------------------------------
# description  This function returns merchant center requirements status.
#----------------------------------------------------------------------------------------
# return       $hRequirementsStatus | hash with check list requirement values | hasref
#========================================================================================
sub getGoogleMerchantCenterRequirementsStatus {
    my $self = shift;

    return $self->get([ MERCHANT_CENTER_REQUIREMENTS_CHECK_LIST ]);
}

#========================================================================================
# function     isMerchantCenterAccountClaimed
# state        public
#----------------------------------------------------------------------------------------
# syntax       $GoogleSmartShoppingConfig->isMerchantCenterAccountClaimed();
#----------------------------------------------------------------------------------------
# description  This function informs if merchant center account has been claimed.
#----------------------------------------------------------------------------------------
# return       $IsClaimed | boolean value (0/1) | boolean
#========================================================================================
sub isMerchantCenterAccountClaimed {
    my $self = shift;

    my $hMerchantCenterAccount = $self->get([ qw(MerchantCenterAccountID MerchantCenterAccountName ) ]);

    return isValid($hMerchantCenterAccount->{'MerchantCenterAccountID'}) &&
        isValid($hMerchantCenterAccount->{'MerchantCenterAccountName'});
}

1;