#==============================================================================
# package      DE_EPAGES::GoogleSmartShopping::UI::Shop
# state        private
#==============================================================================
package DE_EPAGES::GoogleSmartShopping::UI::Shop;
use base DE_EPAGES::Presentation::UI::Object;

use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::GoogleSmartShopping::API::GoogleApiService;
use DE_EPAGES::GoogleSmartShopping::API::GoogleSmartShoppingUrlBuilder;
use DE_EPAGES::GoogleSmartShopping::API::Tools qw ( LogGoogleSmartShopping GetGoogleApiErrorCode );
use DE_EPAGES::Shop::API::Url qw ( BuildShopUrl );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError Error );
use HTTP::Status qw (HTTP_OK HTTP_BAD_REQUEST HTTP_FORBIDDEN HTTP_INTERNAL_SERVER_ERROR);
use DE_EPAGES::GoogleSmartShopping::API::Constants qw (
    ERROR_FORM_SUBMIT
    ERROR_ACCEPT_TERMS
    ERROR_TOKEN_EXPIRED_OR_REVOKED
);

use strict;

use constant OAUTH_ONBOARDING => 0;
use constant ACCOUNT_CONNECTION_GET_EMAIL => 1;

#==============================================================================
# Event handlers
#==============================================================================
sub UnityViewGoogleSmartShopping {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();
    my $aSetupStages = $self->_reckonSetupStages($Shop);

    if (OAUTH_ONBOARDING ~~ $aSetupStages) {
        $Servlet->vars('GoogleOAuthConnect', $self->_buildConnectOAuthURL($Servlet, $Shop));
        return $self->SUPER::View($Servlet);
    }

    my $GoogleSmartShoppingConfig = $Shop->get('GoogleSmartShoppingConfig');
    my $hGoogleSmartShoppingConfig = $GoogleSmartShoppingConfig->get([
        'ChecklistContactInfo',
        'ChecklistReturnPolicy',
        'ChecklistBillingTOC',
        'ChecklistGoogleAdsPolicies',
        'MerchantCenterAccountName',
        'MerchantCenterAccountID',
    ]);

    $Servlet->vars('GoogleSmartShoppingConfig', $hGoogleSmartShoppingConfig);

    if (ACCOUNT_CONNECTION_GET_EMAIL ~~ $aSetupStages) {
        my $hEmailResult = $self->_getConnectedAccountEmail($Shop);
        if (defined $hEmailResult && isValid($hEmailResult->{'email'})) {
            $Servlet->vars('ConnectedGoogleAccountEmail', $hEmailResult->{'email'});
            if (defined $hEmailResult->{'error'} && $hEmailResult->{'error'} eq ERROR_TOKEN_EXPIRED_OR_REVOKED) {
                $Servlet->vars('TokenExpiredOrRevoked', 1);
                $Servlet->vars('GoogleOAuthConnect', $self->_buildConnectOAuthURL($Servlet, $Shop));
                return $self->SUPER::View($Servlet);
            }
        } else {
            $Servlet->vars('ConnectGoogleAccountError', 1);
            return $self->SUPER::View($Servlet);
        }

        $Servlet->vars('GoogleMerchantCenterRequirementsMet', $GoogleSmartShoppingConfig->areGoogleMerchantCenterRequirementsMet());

        my $MerchantCenterAccounts = $self->_getMerchantCenterAccounts($Shop);
        unless (defined($MerchantCenterAccounts)) {
            $Servlet->vars('MerchantCenterAccountsError', 1);
            return $self->SUPER::View($Servlet);
        }
        $Servlet->vars('MerchantCenterAccounts', $MerchantCenterAccounts);
    }

    return $self->SUPER::View($Servlet);
}

sub GoogleOAuthCallback {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();
    my $Form = $Servlet->form();
    my $hQueryParameters = $Form->queryValues();
    my $payload;
    my $Code;

    if ($hQueryParameters->{'code'}) {
        LogGoogleSmartShopping('Callback Success: ', $hQueryParameters);
        my $hResponse = $self->_exchangeAuthCodeForAccessTokens($Shop, $hQueryParameters->{'code'}->[0]);
        LogGoogleSmartShopping('OAuth code exchange response: ', $hResponse);
        if ($hResponse->{'access_token'}) {
            $self->_saveTokensToConfig($Shop, $hResponse->{'access_token'}, $hResponse->{'refresh_token'});
            $payload = "window.opener.ep.fn.callbackSuccess();";
            $Code = HTTP_OK;
        } else {
            $payload = "window.opener.ep.fn.showError('" . quotemeta($hResponse->{'error'}->[0]) . "');";
            $Code = HTTP_INTERNAL_SERVER_ERROR;
        }
    } elsif ($hQueryParameters->{'error'}) {
        LogGoogleSmartShopping('Callback Error: ', $hQueryParameters);
        $payload = "window.opener.ep.fn.showError('" . quotemeta($hQueryParameters->{'error'}->[0]) . "');";
        $Code = HTTP_FORBIDDEN;
    } else {
        LogGoogleSmartShopping('Unexpected parameters to OAuth Callback: ', $hQueryParameters);
        $payload = "window.opener.ep.fn.showError('unknown_error');";
        $Code = HTTP_BAD_REQUEST;
    }
    my $jsContent = <<"EOF";
<script type="text/javascript">
    $payload
    window.close();
</script>
EOF
    return $self->_writeHTTPResponse($Servlet, $Code, {'ContentType' => 'text/html'}, $jsContent);
}

#========================================================================================
# function     CreateMerchantCenterAccount
# state        public
#----------------------------------------------------------------------------------------
# syntax       CreateMerchantCenterAccount( $Servlet );
#----------------------------------------------------------------------------------------
# description  Create a new SubAccount on ePages MerchantCenterAccount
#----------------------------------------------------------------------------------------
# input        $Servlet | Presentation Object | object
# return       $Account | Account details for created MerchantCenterAccount | object
#========================================================================================
sub CreateMerchantCenterAccount {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();
    my $Form = $Servlet->form();
    my $hFormValues = $Form->formValues();

    my $GoogleSmartShoppingConfig = $Shop->get('GoogleSmartShoppingConfig');
    Error('Undefined configuration for GoogleSmartShopping') unless defined $GoogleSmartShoppingConfig;

    my $FormErrorReason = $self->_validateCreateSubAccountRequest($hFormValues, $GoogleSmartShoppingConfig);

    if ($FormErrorReason) {
        $Form->executeFormError({
            'Name'   => 'CreationFailed',
            'Reason' => $FormErrorReason
        });
        return;
    }

    my $GoogleApiService = DE_EPAGES::GoogleSmartShopping::API::GoogleApiService->new({Shop => $Shop});

    my $NewMerchantCenterSubAccount;
    eval {
        $NewMerchantCenterSubAccount = $GoogleApiService->createMerchantCenterSubAccount(
            $hFormValues->{'ConnectedGoogleAccountEmail'}->[0],
            $hFormValues->{'HasAdultContent'}->[0]
        );
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogGoogleSmartShopping('Account creation failed', {
            'Error' => $Error->message(),
        });

        $Form->executeFormError({
            'Name'   => 'CreationFailed',
            'Reason' => GetGoogleApiErrorCode($Error->message())
        });
        return;
    }

    $Servlet->vars('ResponseContent', $NewMerchantCenterSubAccount);
    return;
}

#----------------------------------------------------------------------------------------
sub _validateCreateSubAccountRequest {
    my $self = shift;
    my ($hFormValues, $GoogleSmartShoppingConfig) = @_;

    return ERROR_FORM_SUBMIT unless (isValid($hFormValues->{'ConnectedGoogleAccountEmail'}) && isValid($hFormValues->{'ConnectedGoogleAccountEmail'}->[0]));
    return ERROR_FORM_SUBMIT unless (isValid($hFormValues->{'HasAdultContent'}) && isValid($hFormValues->{'HasAdultContent'}->[0]));
    return ERROR_ACCEPT_TERMS unless (isValid($hFormValues->{'newMerchantAccountAcceptTerms'}) && $hFormValues->{'newMerchantAccountAcceptTerms'}->[0]);
    return ERROR_ACCEPT_TERMS unless ($GoogleSmartShoppingConfig->areGoogleMerchantCenterRequirementsMet());

    return;
}

#----------------------------------------------------------------------------------------
sub _reckonSetupStages {
    my $self = shift;
    my ($Shop) = @_;

    return [ OAUTH_ONBOARDING ] unless ($Shop->existsChild('GoogleSmartShoppingConfig'));
    my $GoogleSmartShoppingConfig = $Shop->get('GoogleSmartShoppingConfig');
    return [ OAUTH_ONBOARDING ] unless ($GoogleSmartShoppingConfig->get('AccessToken'));
    return [ OAUTH_ONBOARDING ] unless ($GoogleSmartShoppingConfig->get('RefreshToken'));

    my $aSecondStageMultiStep = [ ACCOUNT_CONNECTION_GET_EMAIL ];

    return $aSecondStageMultiStep;
}

#----------------------------------------------------------------------------------------
sub _buildConnectOAuthURL {
    my $self = shift;
    my ( $Servlet, $Shop ) = @_;

    my $hParams = {
        'ChangeAction' => 'GoogleOAuthCallback',
        'ObjectID'   => $Shop->id(),
    };
    my $hOptions = {
        'Type' => 'admin',
        'AddToken' => 1,
        'UseSSL' => 1,
    };
    my $CallbackUrl = BuildShopUrl($Shop, $hParams, $hOptions, $Servlet);
    LogGoogleSmartShopping('Callback URL is:', $CallbackUrl);
    my $GoogleUrlBuilder = DE_EPAGES::GoogleSmartShopping::API::GoogleSmartShoppingUrlBuilder->new();

    return $GoogleUrlBuilder->buildConnectOAuthUrl($CallbackUrl);
}

#----------------------------------------------------------------------------------------
sub _exchangeAuthCodeForAccessTokens {
    my $self = shift;
    my ($Shop, $OAuthCode) = @_;

    my $GoogleService = DE_EPAGES::GoogleSmartShopping::API::GoogleApiService->new({Shop => $Shop});
    return $GoogleService->getOAuthTokens($OAuthCode);
}

#----------------------------------------------------------------------------------------
sub _saveTokensToConfig {
    my $self = shift;
    my ($Shop, $AccessToken, $RefreshToken) = @_;

    unless($Shop->existsChild('GoogleSmartShoppingConfig')) {
        $Shop->insertChild('GoogleSmartShoppingConfig', { 'Alias' => 'GoogleSmartShoppingConfig' });
    }
    my $GoogleSmartShoppingConfig = $Shop->get('GoogleSmartShoppingConfig');
    $GoogleSmartShoppingConfig->set({ 'AccessToken' => $AccessToken });

    if (defined $RefreshToken) {
        $GoogleSmartShoppingConfig->set({ 'RefreshToken' => $RefreshToken });
    }

}

#----------------------------------------------------------------------------------------
sub _getConnectedAccountEmail {
    my $self = shift;
    my ($Shop) = @_;

    my $service = DE_EPAGES::GoogleSmartShopping::API::GoogleApiService->new({Shop => $Shop});
    my $hUserInfo = $service->getAndSaveUserInfo();
    if (!defined $hUserInfo->{'email'}) {
        LogGoogleSmartShopping("Google OpenID userinfo lookup failed: ", $hUserInfo);
    }

    return $hUserInfo;
}

sub DisconnectPersonalAccount {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();
    my $GoogleSmartShoppingConfig = $Shop->get('GoogleSmartShoppingConfig');
    return unless defined $GoogleSmartShoppingConfig;
    my $AccessToken = $GoogleSmartShoppingConfig->get('AccessToken');

    $self->_deleteGoogleSmartShoppingConfig($GoogleSmartShoppingConfig);

    LogGoogleSmartShopping("GoogleSmartShoppingConfig deleted for Shop ", $Shop->id() );

    LogGoogleSmartShopping("Revoking access token ", $AccessToken);
    my $service = DE_EPAGES::GoogleSmartShopping::API::GoogleApiService->new({Shop => $Shop});
    my $hRevokeTokenResult = $service->revokeAccessToken($AccessToken);
    if (defined $hRevokeTokenResult->{'error'}) {
        LogGoogleSmartShopping("Google  revoke token failed: ", $hRevokeTokenResult);
        $Servlet->form->executeFormError({
            'Name' => $hRevokeTokenResult->{'error'},
            'Reason' => $hRevokeTokenResult->{'errorMessage'}
        });
    }
}

sub _deleteGoogleSmartShoppingConfig {
    my $self = shift;
    my ($GoogleSmartShoppingConfig) = @_;

    $GoogleSmartShoppingConfig->delete();

}

#----------------------------------------------------------------------------------------
sub _writeHTTPResponse {
    my $self = shift;
    my ($Servlet, $Code, $hParams, $Content) = @_;

    $Servlet->response->code($Code);
    foreach my $key (keys %{$hParams}) {
        $Servlet->params($key, $hParams->{$key});
    }
    return $Servlet->writeContentOutput($Content);
}

sub _getMerchantCenterAccounts {
    my $self = shift;
    my ($Shop) = @_;

    my $service = DE_EPAGES::GoogleSmartShopping::API::GoogleApiService->new({Shop => $Shop});
    return $service->getMerchantCenterAccounts();
}

1;
