#========================================================================================
# package      DE_EPAGES::GoogleSmartShopping::t::unit::GoogleApiClient
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::GoogleSmartShopping::API::GoogleApiClient
#========================================================================================
package DE_EPAGES::GoogleSmartShopping::t::unit::GoogleApiClient;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Test::Mock::Mockify qw ( WasCalled );
use Test::Mockify::Matcher qw ( Object HashRef );
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::MockifyServlet;
use DE_EPAGES::Test::Mock::EpagesObject;

use JSON::Any;
use Test::Exception;
use Test::Mockify::Sut;
use Test::MockObject::Extends;
use Test::More;

use DE_EPAGES::GoogleSmartShopping::API::Constants qw (
    EPAGESJ_BASE_URI
);

use strict;

use constant POST_METHOD => 'POST';
use constant GET_METHOD => 'GET';
use constant ACCESS_TOKEN => 'FAKE_ACCESS_TOKEN';
use constant EXPIRED_ACCESS_TOKEN => 'FAKE_EXPIRED_ACCESS_TOKEN';
use constant NEW_ACCESS_TOKEN => 'FAKE_NEW_ACCESS_TOKEN';
use constant REFRESH_TOKEN => 'FAKE_REFRESH_TOKEN';
use constant EPAGESJ_SUFFIX_URL_PATH => '/epjsuffix/path';
use constant DECODED_JSON_RESPONSE_AS_HASH => { 'resfield1' => 'resvalue1', 'resfield2' => 'resvalue2' };
use constant HEADER_WITH_EXPIRED_TOKEN => { 'AccessToken' => EXPIRED_ACCESS_TOKEN };
use constant REQUEST_FAILED_CODE => 'REQUEST_FAILED';
use constant MOLLIE_ERROR_STRING => "MollieErrorResponse [status=401 | title=Unauthorized Request | detail=Missing authentication, or failed to authenticate | field=null]";
use constant HTTP_STATUS_UNAUTHORIZED => 401;
use constant AUTH_TOKEN_URL => 'https://oauth2.googleapis.com/token';

use constant EXPECTED_ACCESS_TOKEN_ERROR_HASH => {
    'status'    => HTTP_STATUS_UNAUTHORIZED,
    'content'   => {
        'status' => HTTP_STATUS_UNAUTHORIZED,
        'title'  => 'Unauthorized Request',
        'detail' => 'Missing authentication, or failed to authenticate',
        'field'  => 'null'
    }
};

use constant EXPECTED_ACCESS_TOKEN_ERROR_RESPONSE => '{
    "code": 401,
    "errors": [
        {
            "domain": "global",
            "location": "Authorization",
            "locationType": "header",
            "message": "Invalid Credentials",
            "reason": "authError"
        }
    ],
    "message": "Request had invalid authentication credentials. Expected OAuth 2 access token, login cookie or other valid authentication credential. See https://developers.google.com/identity/sign-in/web/devconsole-project.",
    "status": "UNAUTHENTICATED"
}';


use constant TOKENS_RESPONSE_HASH => {
    'access_token' => NEW_ACCESS_TOKEN,
    'refresh_token' => REFRESH_TOKEN,
    'expires_in' => 3600,
    'token_type' => 'bearer',
    'scope' => 'payments.read payments.write refunds.read refunds.write profiles.read profiles.write'
};

use constant USER_ACCOUNT_RESPONSE_HASH => {
    'accountIdentifiers' => [
        {
            merchantId => 444950401 
        }
    ],
    'kind' => 'content#accountsAuthInfoResponse',
};

sub test {
    my $self = shift;

    $self->test_DoGet_WithEpagesjRequest();
    $self->test_DoGet_WithEpagesjRequest_ExpiredToken();
    done_testing();
}

#----------------------------------------------------------------------------------------
# Tests
#----------------------------------------------------------------------------------------
sub test_DoGet_WithEpagesjRequest {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    note("-------- doGet --------");

    my $MockedGoogleConfig = $self->_createGoogleConfig(
        {
            'RefreshToken' => REFRESH_TOKEN,
            'AccessToken' => ACCESS_TOKEN,
        }
    );

    my $SUT = $self->_createSUT({'Shop' => $self->_createShop($MockedGoogleConfig)});

    my $Url = EPAGESJ_BASE_URI . 'accounts/user-account-info';
    my $Response = $SUT->doGet($Url);

    ok(!WasCalled($SUT, '_isTokenExpired'), "$SubTestName - _isTokenExpired was NOT called.");
    ok(!WasCalled($SUT, '_doRefreshAccessToken'), "$SubTestName - _doRefreshAccessToken was NOT called.");

    is_deeply($Response, USER_ACCOUNT_RESPONSE_HASH, "$SubTestName - ACCESS_TOKEN returns correct USER_ACCOUNT_RESPONSE_HASH.");

}


sub test_DoGet_WithEpagesjRequest_ExpiredToken {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    note("-------- doGet ExpiredToken--------");

    my $MockedGoogleConfig = $self->_createGoogleConfig(
        {
            'RefreshToken' => REFRESH_TOKEN,
            'AccessToken' => EXPIRED_ACCESS_TOKEN,
        }
    );

    my $SUT = $self->_createSUT({'Shop' => $self->_createShop($MockedGoogleConfig)});

    my $Url = EPAGESJ_BASE_URI . 'accounts/user-account-info';
    my $Response = $SUT->doGet($Url);

    ok(WasCalled($SUT, '_isTokenExpired'), "$SubTestName - _isTokenExpired was called.");
    ok(WasCalled($SUT, '_doRefreshAccessToken'), "$SubTestName - _doRefreshAccessToken was called.");

    is_deeply($Response, USER_ACCOUNT_RESPONSE_HASH, "$SubTestName - EXPIRED_ACCESS_TOKEN is renovated and returns correct USER_ACCOUNT_RESPONSE_HASH.");

}

#----------------------------------------------------------------------------------------
# Helpers
#----------------------------------------------------------------------------------------

sub _createGoogleConfig {
    my $self = shift;
    my ($hParams) = @_;

    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::GoogleSmartShopping::API::Object::GoogleSmartShoppingConfig', $hParams
    );

    return $Mockify->getMockObject();
}

sub _createShop {
    my $self = shift;
    my ($MockedGoogleConfig) = @_;

    my $hAttributes = {
        'Alias' => 'MyShop',
        'ShopTypeRef' => $self->_createShopTypeRef(),
        'GoogleSmartShoppingConfig' => $MockedGoogleConfig
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Shop::API::Object::Shop', $hAttributes
    );

    $Mockify->addMockWithReturnValueAndParameterCheck('existsChild', 1, [{'string'=>'GoogleSmartShoppingConfig'}]);

    return $Mockify->getMockObject();
}

sub _createSUT {
    my $self = shift;
    my ($RequestMock, $MockedTokensRequest) = @_;

    $MockedTokensRequest = {};

    my $MockifySut = Test::Mockify::Sut->new(
        'DE_EPAGES::GoogleSmartShopping::API::GoogleApiClient', [ $RequestMock ]
    );

    $MockifySut->mockImported('DE_EPAGES::Object::API::Factory', 'LoadRootObject')
        ->whenAny()
        ->thenReturn($self->_createSystem());

    $MockifySut->mockImported('DE_EPAGES::GoogleSmartShopping::API::Tools', 'LogGoogleSmartShopping')
        ->whenAny()
        ->thenReturnUndef();

    $MockifySut->mockStatic('DE_EPAGES::EPagesJ::API::Object::RestClient::create')
        ->whenAny()
        ->thenReturn($self->_createRestClient());

    my $Error = DE_EPAGES::Core::API::Error->new(
        Status => HTTP_STATUS_UNAUTHORIZED,
        Vars => {
            'Status' => HTTP_STATUS_UNAUTHORIZED,
            'Response' => EXPECTED_ACCESS_TOKEN_ERROR_RESPONSE
        }
    );

    $MockifySut->spy('_isTokenExpired')
        ->when(Object());

    $MockifySut->spy('_doRefreshAccessToken')
        ->when();

    $MockifySut->mock('_GetError')->whenAny()->thenReturn($Error);

    # mocks for _doRefreshAccessToken
    my $UserAgent = Test::Mockify->new('LWP::UserAgent');
    $UserAgent->mock('request')->when(Object())->thenReturn($self->_createTokenResponse());

    $MockifySut->mockStatic('LWP::UserAgent::new')->when()->thenReturn($UserAgent->getMockObject());

    return $MockifySut->getMockObject();
}

sub _createTokenResponse {
    my $self = shift;

    my $MockifyResponse = Test::Mockify->new('HTTP::Response');
    $MockifyResponse->mock('code')->when()->thenReturn('200');
    $MockifyResponse->mock('decoded_content')->when()->thenReturn('{"access_token": "FAKE_ACCESS_TOKEN"}');
    return $MockifyResponse->getMockObject();

}

sub _createSystem {
    my $self = shift;

    my $hAttributes = {
        'EPagesJBaseURL' => 'https://shop.example.com:8088/rs',
        'OAuthServiceBaseUrl' => undef,
        'ProviderID' => 'epages',
        'GoogleSmartShopping_AuthURL' => 'epages',
        'GoogleSmartShopping_AuthTokenURL' => AUTH_TOKEN_URL,

        'GoogleSmartShopping_AppClientID' => 'CLIENT_ID',
        'GoogleSmartShopping_AppClientSecret' => 'CLIENT_SECRET',
        'GoogleSmartShopping_AppRedirectURL' => 'CLIENT_REDIRECT_URI',
        'GoogleSmartShopping_AdminRefreshToken' => REFRESH_TOKEN,
        'GoogleSmartShopping_MerchantCenterMultiClientAccountID' => 12345,

    };
    my $MockifySystem = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Object::API::Object::Site', $hAttributes
    );

    return $MockifySystem->getMockObject();
}

sub _createRestClient {
    my $self = shift;

    my $aParameters = [];
    my $MockifyRestClient = DE_EPAGES::Test::Mock::Mockify->new(
            'DE_EPAGES::EPagesJ::API::Object::RestClient', $aParameters
    );
    $MockifyRestClient->addMockWithReturnValue('client', $self->_createHttpClient());

    return $MockifyRestClient->getMockObject();
}

sub _createHttpClient {
    my $self = shift;

    my $Mockify = Test::Mockify->new('DE_EPAGES::EPagesJ::API::Object::HTTPClient', []);

    $Mockify->mock('get')
            ->when(Object(), HashRef({'X-AccessToken' => ACCESS_TOKEN}))
            ->thenReturn($self->_createHttpResponse(USER_ACCOUNT_RESPONSE_HASH));

    $Mockify->mock('get')
            ->when(Object(), HashRef({'X-AccessToken' => EXPIRED_ACCESS_TOKEN}))
            ->thenThrowError('UNAUTHORIZED');

    return $Mockify->getMockObject();
}


sub _createHttpResponse {
    my $self = shift;
    my ($ResponseObj) = @_;

    my $Json = JSON::Any->new();

    my $MockifyHttpResponse = DE_EPAGES::Test::Mock::Mockify->new('HTTP::Response');
    $MockifyHttpResponse->addMockWithReturnValue('decoded_content', $Json->objToJson($ResponseObj));

    return $MockifyHttpResponse->getMockObject();
}

sub _createShopTypeRef {
    my $self = shift;

    my $hAttributes = {
        'Alias' => 'UnityDemo'
    };
    my $MockifySystem = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Shop::API::Object::ShopTypeRef', $hAttributes
    );

    return $MockifySystem->getMockObject();
}


#========================================================================================
# Main
#========================================================================================

__PACKAGE__->runScript();
