#========================================================================================
# package      DE_EPAGES::GoogleSmartShopping::t::unit::UI_GoogleSmartShoppingConfig
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for the GoogleSmartShoppingConfig UI/AJAX handler
#========================================================================================
package DE_EPAGES::GoogleSmartShopping::t::unit::UI_GoogleSmartShoppingConfig;
use base DE_EPAGES::Core::API::Test;

use strict;

use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::MockifyServlet;

use Test::Mockify;
use Test::Mockify::Matcher qw ( HashRef String );
use Test::Mockify::Sut;
use Test::Mockify::Verify qw ( GetParametersFromMockifyCall );
use Test::More;

use HTTP::Status qw (HTTP_OK HTTP_NOT_FOUND);

use constant SHOPPING_CHECKLIST_NONE_CONFIRMED => {
    'ChecklistContactInfo'          => 0,
    'ChecklistBillingTOC'           => 0,
    'ChecklistReturnPolicy'         => 0,
    'ChecklistGoogleAdsPolicies'    => 0,
};

use constant SHOPPING_CHECKLIST_CONTACT_CONFIRMED => {
    'ChecklistContactInfo'          => 1,
    'ChecklistBillingTOC'           => 0,
    'ChecklistReturnPolicy'         => 0,
    'ChecklistGoogleAdsPolicies'    => 0,
};

use constant ALL_CHECKLIST_CONFIRMED => {
    'ChecklistContactInfo'          => 1,
    'ChecklistBillingTOC'           => 1,
    'ChecklistReturnPolicy'         => 1,
    'ChecklistGoogleAdsPolicies'    => 1,
};

# the AJAX call sends 'true' or 'false' to the backend
use constant SHOPPING_CHECKLIST_UPDATE_DATA => {
    'ChecklistContactInfo' => [ 'true' ],
};

use constant SELECTED_MERCHANT_CENTER_ACCOUNT_ID => 'merchant-center-account-id-1';

use constant SELECTED_MERCHANT_CENTER_ACCOUNT_NAME => 'merchant center account name 1';

use constant SELECTED_MERCHANT_CENTER_SUB_ACCOUNT_ID => 'merchant-center-sub-account-id-2';

use constant SELECTED_MERCHANT_CENTER_SUB_ACCOUNT_NAME => 'merchant center sub-account name 2';

use constant SELECTED_MERCHANT_CENTER_SUB_ACCOUNT_ID_NOT_EXISTENT => 'merchant-center-sub-account-id-0';

use constant SELECTED_MERCHANT_CENTER_SUB_ACCOUNT_NAME_NOT_EXISTENT => 'merchant center sub-account name 0';

use constant MERCHANT_CENTER_MULTI_CLIENT_ACCOUNT_ID => 'aggregator-account-id-1';

use constant USER_ACCOUNT_INFO => {
    'accountIdentifiers' =>
        [
            {
                'merchantId' => SELECTED_MERCHANT_CENTER_ACCOUNT_ID
            },
            {
                'aggregatorId' => MERCHANT_CENTER_MULTI_CLIENT_ACCOUNT_ID,
                'merchantId'   => SELECTED_MERCHANT_CENTER_SUB_ACCOUNT_ID
            },
            {
                'merchantId' => 'merchant-center-account-id-3'
            }
        ]
};


sub test {
    my $self = shift;

    $self->test_ChecklistHandlerReturnsCorrectInfoUnchanged();
    $self->test_ChecklistHandlerReturnsCorrectInfoWithUpdate();
    $self->test_ChecklistHandlerNoShoppingConfigError();
    $self->test_MerchantCenterAccountConnectsSuccessfully();
    $self->test_MerchantCenterSubAccountConnectsSuccessfully();
    $self->test_MerchantCenterAccountConnectsError();

    done_testing();
}

sub test_ChecklistHandlerReturnsCorrectInfoUnchanged {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    # mockery/cruft
    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig('some-access-token', 'some-refresh-token', SHOPPING_CHECKLIST_NONE_CONFIRMED);
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hPostData = {};
    my $ServletForm = $self->_createServletForm($hPostData);
    my $ServletResponse = $self->_createServletResponse();
    my $Servlet = $self->_createServlet($Shop, $ServletForm, $ServletResponse);
    my $SUT = $self->_createSUT();

    # run it
    $SUT->HandleChecklistAJAX($Servlet);
    my $Code = GetParametersFromMockifyCall($ServletResponse, 'code')->[0];
    my $aParams = GetParametersFromMockifyCall($Servlet, 'params');
    my $hContent = GetParametersFromMockifyCall($Servlet, 'writeContentOutput')->[0];

    # actual tests
    is(
        $Code,
        HTTP_OK,
        $SubTestName . ' - Ensure Ensure HTTP response code was HTTP_OK'
    );
    is_deeply(
        $aParams,
        [ 'ContentType' => 'application/json' ],
        $SubTestName . ' - Ensure ContentType of application/json'
    );
    is_deeply(
        $hContent,
        SHOPPING_CHECKLIST_NONE_CONFIRMED,
        $SubTestName . ' - Ensure $Servlet->writeContentOutput() was called with the unchanged Shopping Checklist'
    );
}

sub test_ChecklistHandlerReturnsCorrectInfoWithUpdate {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    # mockery/cruft
    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig('some-access-token', 'some-refresh-token', SHOPPING_CHECKLIST_NONE_CONFIRMED);
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hPostData = SHOPPING_CHECKLIST_UPDATE_DATA;
    my $ServletForm = $self->_createServletForm($hPostData);
    my $ServletResponse = $self->_createServletResponse();
    my $Servlet = $self->_createServlet($Shop, $ServletForm, $ServletResponse);
    my $SUT = $self->_createSUT();

    # run it
    $SUT->HandleChecklistAJAX($Servlet);
    my $Code = GetParametersFromMockifyCall($ServletResponse, 'code')->[0];
    my $aParams = GetParametersFromMockifyCall($Servlet, 'params');
    my $hContent = GetParametersFromMockifyCall($Servlet, 'writeContentOutput')->[0];

    # actual tests
    is(
        $Code,
        HTTP_OK,
        $SubTestName . ' - Ensure HTTP response code was HTTP_OK'
    );
    is_deeply(
        $aParams,
        [ 'ContentType' => 'application/json' ],
        $SubTestName . ' - Ensure ContentType of application/json'
    );
    is_deeply(
        $hContent,
        SHOPPING_CHECKLIST_CONTACT_CONFIRMED,
        $SubTestName . ' - Ensure $Servlet->writeContentOutput() was called with the updated Shopping Checklist'
    );
}

sub test_ChecklistHandlerNoShoppingConfigError {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    # mockery/cruft
    my $GoogleSmartShoppingConfig = undef;
    my $AlreadyExistingConfig = 0;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hPostData = {  };
    my $ServletForm = $self->_createServletForm($hPostData);
    my $ServletResponse = $self->_createServletResponse();
    my $Servlet = $self->_createServlet($Shop, $ServletForm, $ServletResponse);
    my $SUT = $self->_createSUT();

    # run it
    $SUT->HandleChecklistAJAX($Servlet);
    my $Code = GetParametersFromMockifyCall($ServletResponse, 'code')->[0];

    # actual test
    is(
        $Code,
        HTTP_NOT_FOUND,
        $SubTestName . ' - Ensure HTTP response code was HTTP_NOT_FOUND'
    );
}

sub test_MerchantCenterAccountConnectsSuccessfully {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig(
        'some-access-token',
        'some-refresh-token',
        ALL_CHECKLIST_CONFIRMED
    );
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hPostData = {
        'MerchantCenterAccountList' => [
            SELECTED_MERCHANT_CENTER_ACCOUNT_ID . '|' . SELECTED_MERCHANT_CENTER_ACCOUNT_NAME
        ]
    };
    my $ServletForm = $self->_createServletForm($hPostData);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createApiService({ UserAccountInfo => USER_ACCOUNT_INFO });
    my $SUT = $self->_createSUT($ApiService);

    $SUT->ConnectMerchantCenterAccount($Servlet);

    is(
        $GoogleSmartShoppingConfig->get('MerchantCenterAccountID'),
        SELECTED_MERCHANT_CENTER_ACCOUNT_ID,
        $SubTestName . ' - Selected merchant center account ID is saved into DB'
    );
    is(
        $GoogleSmartShoppingConfig->get('MerchantCenterAccountName'),
        SELECTED_MERCHANT_CENTER_ACCOUNT_NAME,
        $SubTestName . ' - Selected merchant center account name is saved into DB'
    );
    is(
        $GoogleSmartShoppingConfig->get('MerchantCenterMultiClientAccountID'),
        undef,
        $SubTestName . ' - No multi-client account ID is being saved into DB'
    );
    is(
        $Servlet->vars('WebsiteClaimedOnMerchantCenterAccount'),
        1,
        $SubTestName . ' - Website claimed on selected Merchant Center Account'
    );
}

sub test_MerchantCenterSubAccountConnectsSuccessfully {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig(
        'some-access-token',
        'some-refresh-token',
        ALL_CHECKLIST_CONFIRMED
    );
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hPostData = {
        'MerchantCenterAccountList' => [
            SELECTED_MERCHANT_CENTER_SUB_ACCOUNT_ID . '|' . SELECTED_MERCHANT_CENTER_SUB_ACCOUNT_NAME
        ]
    };
    my $ServletForm = $self->_createServletForm($hPostData);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createApiService({ UserAccountInfo => USER_ACCOUNT_INFO });
    my $SUT = $self->_createSUT($ApiService);

    $SUT->ConnectMerchantCenterAccount($Servlet);

    is(
        $GoogleSmartShoppingConfig->get('MerchantCenterAccountID'),
        SELECTED_MERCHANT_CENTER_SUB_ACCOUNT_ID,
        $SubTestName . ' - Selected merchant center account ID is saved into DB'
    );
    is(
        $GoogleSmartShoppingConfig->get('MerchantCenterAccountName'),
        SELECTED_MERCHANT_CENTER_SUB_ACCOUNT_NAME,
        $SubTestName . ' - Selected merchant center account name is saved into DB'
    );
    is(
        $GoogleSmartShoppingConfig->get('MerchantCenterMultiClientAccountID'),
        MERCHANT_CENTER_MULTI_CLIENT_ACCOUNT_ID,
        $SubTestName . ' - Multi-client account ID is being saved into DB'
    );
    is(
        $GoogleSmartShoppingConfig->get('IsMerchantCenterSubAccount'),
        1,
        $SubTestName . ' - Merchant center account IS an epages sub-account'
    );
    is(
        $Servlet->vars('WebsiteClaimedOnMerchantCenterAccount'),
        1,
        $SubTestName . ' - Website claimed on selected Merchant Center Account'
    );
}

sub test_MerchantCenterAccountConnectsError {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig(
        'some-access-token',
        'some-refresh-token',
        ALL_CHECKLIST_CONFIRMED
    );
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hPostData = {
        'MerchantCenterAccountList' => [
            SELECTED_MERCHANT_CENTER_SUB_ACCOUNT_ID_NOT_EXISTENT . '|'
            . SELECTED_MERCHANT_CENTER_SUB_ACCOUNT_NAME_NOT_EXISTENT
        ]
    };
    my $ServletForm = $self->_createServletForm($hPostData);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createApiService({ UserAccountInfo => USER_ACCOUNT_INFO });
    my $SUT = $self->_createSUT($ApiService);

    $SUT->ConnectMerchantCenterAccount($Servlet);

    is(
        $GoogleSmartShoppingConfig->get('MerchantCenterAccountID'),
        undef,
        $SubTestName . ' - Selected merchant center account ID is NOT saved into DB'
    );
    is(
        $GoogleSmartShoppingConfig->get('MerchantCenterAccountName'),
        undef,
        $SubTestName . ' - Selected merchant center account name is NOT saved into DB'
    );
    is(
        $GoogleSmartShoppingConfig->get('MerchantCenterMultiClientAccountID'),
        undef,
        $SubTestName . ' - No multi-client account ID is being saved into DB'
    );
    is(
        $Servlet->vars('MerchantCenterAccountClaimError'),
        1,
        $SubTestName . ' - Website claiming error is being set'
    );
}

#----------------------------------------------------------------------------------------
# Mockery
#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;
    my ($ApiService) = @_;

    my $MockifySut = Test::Mockify::Sut->new('DE_EPAGES::GoogleSmartShopping::UI::GoogleSmartShoppingConfig');
    $MockifySut->mockImported('DE_EPAGES::GoogleSmartShopping::API::Tools', 'LogGoogleSmartShopping')
        ->whenAny()
        ->thenReturnUndef();

    $MockifySut->mockImported('DE_EPAGES::Object::API::Factory', 'LoadRootObject')
        ->whenAny()
        ->thenReturn($self->_createSystem());

    if (defined($ApiService)) {
        $MockifySut->mockStatic('DE_EPAGES::GoogleSmartShopping::API::GoogleApiService::new')
            ->when(String(), HashRef())
            ->thenReturn($ApiService);
    }

    return $MockifySut->getMockObject();
}

sub _createGoogleSmartShoppingConfig {
    my $self = shift;
    my ($AccessToken, $RefreshToken, $hChecklist) = @_;

    my $hAttributes = {
        AccessToken => $AccessToken,
        RefreshToken => $RefreshToken,
        %{$hChecklist},
        MerchantCenterAccountID => undef,
        MerchantCenterAccountName => undef,
        MerchantCenterMultiClientAccountID => undef,
        IsMerchantCenterSubAccount => 0
    };
    my $MockifyConfig = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::GoogleSmartShopping::API::Object::GoogleSmartShoppingConfig', $hAttributes
    );

    return $MockifyConfig->getMockObject();
}

sub _createServlet {
    my $self = shift;
    my ($Shop, $ServletForm, $HTTPResponse) = @_;

    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new($Shop);
    $MockifyServlet->addMockWithReturnValue('form', $ServletForm);
    $MockifyServlet->addMockWithReturnValue('response', $HTTPResponse);
    $MockifyServlet->addMockWithReturnValueAndParameterCheck('writeContentOutput', undef, [ 'hashref' ]);
    $MockifyServlet->addMockWithReturnValueAndParameterCheck('params', undef, [ 'string', 'string' ]);

    return $MockifyServlet->getMockObject();
}

sub _createShop {
    my $self = shift;
    my ($GoogleSmartShoppingConfig, $AlreadyExistingConfig) = @_;

    my $hAttributes = {
        'ID' => undef,
        GoogleSmartShoppingConfig => $GoogleSmartShoppingConfig
    };
    my $MockifyShop = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', $hAttributes);
    $MockifyShop->addMockWithReturnValueAndParameterCheck(
        'existsChild', $AlreadyExistingConfig, [ {'string' => 'GoogleSmartShoppingConfig'} ]
    );

    return $MockifyShop->getMockObject();
}

sub _createServletForm {
    my $self = shift;
    my ($hPostData) = @_;

    my $Mockify = Test::Mockify->new('DE_EPAGES::Presentation::API::Form');
    $Mockify->mock('formValues')
        ->when()
        ->thenReturn($hPostData);

    return $Mockify->getMockObject();
}

sub _createServletResponse {
    my $self = shift;

    my $Mockify = Test::Mockify->new('HTTP::Response');
    $Mockify->mock('code')
        ->whenAny()
        ->thenReturnUndef();

    return $Mockify->getMockObject();
}

sub _createSystem {
    my $self = shift;

    my $hAttributes = {
        GoogleSmartShopping_MerchantCenterMultiClientAccountID => MERCHANT_CENTER_MULTI_CLIENT_ACCOUNT_ID
    };
    my $MockifySystem = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Object::API::Object::Site',
        $hAttributes
    );

    return $MockifySystem->getMockObject();
}

sub _createApiService {
    my $self = shift;
    my ($hServiceResponse) = @_;

    my $Mockify = Test::Mockify->new('DE_EPAGES::GoogleSmartShopping::API::GoogleApiService');
    $Mockify->mock('getUserAccountInfo')
        ->when()
        ->thenReturn($hServiceResponse->{'UserAccountInfo'} // {});

    return $Mockify->getMockObject();
}

__PACKAGE__->runScript();

