#========================================================================================
# package      DE_EPAGES::GoogleSmartShopping::t::unit::UI_Shop
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for the GoogleSmartShopping view setup
#========================================================================================
package DE_EPAGES::GoogleSmartShopping::t::unit::UI_Shop;
use base DE_EPAGES::Core::API::Test;

use strict;

use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::MockifyServlet;

use Test::Mockify;
use Test::Mockify::Matcher qw ( HashRef Object String Number );
use Test::Mockify::Sut;
use Test::Mockify::Verify qw ( WasCalled GetParametersFromMockifyCall );
use Test::More;

use DE_EPAGES::GoogleSmartShopping::API::Constants qw (
    ERROR_FORM_SUBMIT
    ERROR_TOO_MANY_SUBACCOUNTS
);

use HTTP::Status qw (HTTP_OK HTTP_BAD_REQUEST HTTP_FORBIDDEN HTTP_INTERNAL_SERVER_ERROR);

use constant CONNECT_OAUTH_URL => 'https://www.example.com';
use constant GOOGLE_OAUTH_ERROR => 'access_denied';
use constant GOOGLE_SINGLE_QUOTE_ERROR => 'quo\'te';
use constant GOOGLE_OAUTH_TOKENS_EXCHANGE_ERROR => 'INTERNAL_SERVER_ERROR';
use constant SUCCESSFUL_GOOGLE_OAUTH_CALLBACK_RESPONSE => {
    code  => [ 'some-google-authorization-code' ],
    scope => [ 'space-separated-list-of-scopes-string']
};
use constant UNSUCCESSFUL_GOOGLE_OAUTH_CALLBACK_RESPONSE => {
    error  => [ GOOGLE_OAUTH_ERROR ]
};
use constant GOOGLE_OAUTH_CALLBACK_UNKNOWN_PARAMETER => {
    unknown => 'asdf'
};
use constant GOOGLE_OAUTH_CALLBACK_ERROR_SINGLE_QUOTE => {
    error  => [ GOOGLE_SINGLE_QUOTE_ERROR ]
};
use constant SUCCESSFUL_GOOGLE_OAUTH_TOKENS_EXCHANGE_RESPONSE => {
    access_token  => 'some-google-access-token',
    refresh_token => 'some-google-refresh-token',
    id_token      => 'some-long-encrypted-key',
    expires_in    => 3599,
    token_type    => 'Bearer',
    scope         => 'space-separated-list-of-scopes-string'
};
use constant UNSUCCESSFUL_GOOGLE_OAUTH_TOKENS_EXCHANGE_RESPONSE => {
    error => [ GOOGLE_OAUTH_TOKENS_EXCHANGE_ERROR ]
};
use constant TOKENS_EXCHANGE_SINGLE_QUOTE_ERROR_RESPONSE => {
    error => [ GOOGLE_SINGLE_QUOTE_ERROR ]
};
use constant EXAMPLE_EMAIL => 'google-account-email';
use constant SUCCESSFUL_USER_INFO_RESPONSE => {
    email => EXAMPLE_EMAIL
};
use constant SUCCESSFUL_MERCHANT_CENTER_ACCOUNTS_RESPONSE => {
    name => 'google-merchant-center-account-name',
    id   => 123456789
};
use constant FAILED_USER_INFO_RESPONSE => {
    error => 'token_expired_or_revoked'
};
use constant FAILED_USER_INFO_RESPONSE_WITH_EMAIL => {
    error => 'token_expired_or_revoked',
    email => EXAMPLE_EMAIL,
};
use constant SUCCESSFUL_CALLBACK_JS_CONTENT_RESPONSE => "<script type=\"text/javascript\">\n    window.opener.ep.fn.callbackSuccess();\n    window.close();\n</script>\n";
use constant ACCESS_DENIED_CALLBACK_JS_CONTENT_RESPONSE => "<script type=\"text/javascript\">\n    window.opener.ep.fn.showError('access_denied');\n    window.close();\n</script>\n";
use constant MISC_ERROR_CALLBACK_JS_CONTENT_RESPONSE => "<script type=\"text/javascript\">\n    window.opener.ep.fn.showError('INTERNAL_SERVER_ERROR');\n    window.close();\n</script>\n";
use constant QUOTE_ESCAPED_CALLBACK_JS_CONTENT_RESPONSE => "<script type=\"text/javascript\">\n    window.opener.ep.fn.showError('quo\\'te');\n    window.close();\n</script>\n";
use constant BAD_REQUEST_JS_CONTENT_RESPONSE => "<script type=\"text/javascript\">\n    window.opener.ep.fn.showError('unknown_error');\n    window.close();\n</script>\n";


use constant REVOKE_TOKEN_RESPONSE_SUCCESSFUL => {};
use constant REVOKE_TOKEN_RESPONSE_ERROR => {
    error => 'some-error',
    errorMessage => 'some-revoke-token-reason'
};

use constant ACCOUNT_ID => 89879879;

sub test {
    my $self = shift;

    $self->test_ViewGoogleSmartShopping_On_OnboardingStart_NoGoogleSmartShoppingConfig();
    $self->test_ViewGoogleSmartShopping_On_OnboardingStart_NoAccessToken();
    $self->test_ViewGoogleSmartShopping_On_OnboardingStart_NoRefreshToken();
    $self->test_ViewGoogleSmartShopping_On_GettingConnectedAccountEmail();
    $self->test_ViewGoogleSmartShopping_On_ConnectedAccountEmailFailsFirstTime();
    $self->test_ViewGoogleSmartShopping_On_ConnectedAccountEmailFailsTokenExpiredOrRevoked();
    $self->test_DisconnectConnectedAccount_RevokeToken_NoConfig();
    $self->test_DisconnectConnectedAccount_RevokeToken_Successful();
    $self->test_DisconnectConnectedAccount_RevokeToken_Error();
    $self->test_GoogleOAuthCallback_On_OnboardingCallbackSuccess_And_SuccessfulTokensExchange();
    $self->test_GoogleOAuthCallback_On_OnboardingCallbackError();
    $self->test_GoogleOAuthCallback_On_OnboardingCallbackSuccess_And_UnsuccessfulTokensExchange();
    $self->test_GoogleOAuthCallback_On_OnboardingCallbackErrorSingleQuote();
    $self->test_GoogleOAuthCallback_On_OnboardingCallbackSuccess_And_TokensExchangeSingleQuoteError();
    $self->test_GoogleOAuthCallback_On_OnboardingCallbackUnknownParameter();
    $self->test_ConnectedAccount_MerchantCenterRequirements_Not_Met();
    $self->test_ViewGoogleSmartShopping_On_Getting_MerchantCenterAccounts();
    $self->test_ViewGoogleSmartShopping_On_Getting_MerchantCenterAccounts_NoAccounts();
    $self->test_CreateMerchantCenterAccount_Success();
    $self->test_CreateMerchantCenterAccount_Fail_Form();
    $self->test_CreateMerchantCenterAccount_Fail_Api();

    done_testing();
}

sub test_ViewGoogleSmartShopping_On_OnboardingStart_NoGoogleSmartShoppingConfig {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = undef;
    my $AlreadyExistingConfig = 0;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $SUT = $self->_createSUT();

    $SUT->UnityViewGoogleSmartShopping($Servlet);

    is(
        $Servlet->vars('GoogleOAuthConnect'),
        CONNECT_OAUTH_URL,
        "$SubTestName - GoogleSmartShopping setup view has OAuth URL when there is no GoogleSmartShoppingConfig object"
    );
    ok( WasCalled($SUT, '_buildConnectOAuthURL'), $SubTestName . ' - Verify that OAuth connect URL was built' );
}

sub test_ViewGoogleSmartShopping_On_OnboardingStart_NoAccessToken {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig(undef, undef);
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $SUT = $self->_createSUT();

    $SUT->UnityViewGoogleSmartShopping($Servlet);

    is(
        $Servlet->vars('GoogleOAuthConnect'),
        CONNECT_OAUTH_URL,
        "$SubTestName - GoogleSmartShopping setup view has OAuth URL when GoogleSmartShoppingConfig access token is empty"
    );
    ok( WasCalled($SUT, '_buildConnectOAuthURL'), $SubTestName . ' - Verify that OAuth connect URL was built' );
}

sub test_ViewGoogleSmartShopping_On_OnboardingStart_NoRefreshToken {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig('some-access-token', undef);
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $SUT = $self->_createSUT();

    $SUT->UnityViewGoogleSmartShopping($Servlet);

    is(
        $Servlet->vars('GoogleOAuthConnect'),
        CONNECT_OAUTH_URL,
        "$SubTestName - GoogleSmartShopping setup view has OAuth URL when GoogleSmartShoppingConfig refresh token is empty"
    );
    ok( WasCalled($SUT, '_buildConnectOAuthURL'), $SubTestName . ' - Verify that OAuth connect URL was built' );
}

sub test_ViewGoogleSmartShopping_On_GettingConnectedAccountEmail {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig('some-access-token', 'some-refresh-token');
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createGoogleApiService({
        UserInfo => SUCCESSFUL_USER_INFO_RESPONSE,
        MerchantCenterAccounts => SUCCESSFUL_MERCHANT_CENTER_ACCOUNTS_RESPONSE
    });
    my $SUT = $self->_createSUT($ApiService);

    $SUT->UnityViewGoogleSmartShopping($Servlet);

    ok(
        WasCalled($ApiService, 'getAndSaveUserInfo'),
        $SubTestName . ' - Verify that Google account email is being retrieved'
    );
    ok(
        WasCalled($ApiService, 'getMerchantCenterAccounts'),
        $SubTestName . ' - Verify that getMerchantCenterAccounts was called when AccountInfo call is successful'
    );
    ok( !WasCalled($SUT, '_buildConnectOAuthURL'), $SubTestName . ' - Verify that OAuth connect URL was NOT built' );
}

sub test_ViewGoogleSmartShopping_On_Getting_MerchantCenterAccounts {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig('some-access-token', 'some-refresh-token', 0);
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createGoogleApiService({
        UserInfo => SUCCESSFUL_USER_INFO_RESPONSE,
        MerchantCenterAccounts => SUCCESSFUL_MERCHANT_CENTER_ACCOUNTS_RESPONSE
    });
    my $SUT = $self->_createSUT($ApiService);

    $SUT->UnityViewGoogleSmartShopping($Servlet);

    ok(
        WasCalled($ApiService, 'getAndSaveUserInfo'),
        $SubTestName . ' - Verify that Google account email is being retrieved'
    );
    ok(
        WasCalled($ApiService, 'getMerchantCenterAccounts'),
        $SubTestName . ' - Verify that getMerchantCenterAccounts was called when AccountInfo call is successful'
    );
    is(
        $Servlet->vars('MerchantCenterAccounts'),
        SUCCESSFUL_MERCHANT_CENTER_ACCOUNTS_RESPONSE,
        "$SubTestName - Verify that MerchantCenterAccounts correctly returned."
    );
    ok( !WasCalled($SUT, '_buildConnectOAuthURL'), $SubTestName . ' - Verify that OAuth connect URL was NOT built' );
}


sub test_ViewGoogleSmartShopping_On_Getting_MerchantCenterAccounts_NoAccounts {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig('some-access-token', 'some-refresh-token', 0);
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createGoogleApiService({
        UserInfo => SUCCESSFUL_USER_INFO_RESPONSE,
        MerchantCenterAccounts => []
    });
    my $SUT = $self->_createSUT($ApiService);

    $SUT->UnityViewGoogleSmartShopping($Servlet);

    ok(
        WasCalled($ApiService, 'getAndSaveUserInfo'),
        $SubTestName . ' - Verify that Google account email is being retrieved'
    );
    ok(
        WasCalled($ApiService, 'getMerchantCenterAccounts'),
        $SubTestName . ' - Verify that getMerchantCenterAccounts was called when AccountInfo call is successful'
    );
    is_deeply(
        $Servlet->vars('MerchantCenterAccounts'),
        [],
        "$SubTestName - Verify that MerchantCenterAccounts returns empty response when no accounts found."
    );
    ok( !WasCalled($SUT, '_buildConnectOAuthURL'), $SubTestName . ' - Verify that OAuth connect URL was NOT built' );
}

sub test_ViewGoogleSmartShopping_On_ConnectedAccountEmailFailsFirstTime {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig('some-access-token', 'some-refresh-token');
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createGoogleApiService({
        UserInfo => FAILED_USER_INFO_RESPONSE
    });
    my $SUT = $self->_createSUT($ApiService);

    $SUT->UnityViewGoogleSmartShopping($Servlet);

    ok(
        WasCalled($ApiService, 'getAndSaveUserInfo'),
        $SubTestName . ' - Verify that lookup of Google account email is attempted'
    );
}

sub test_ViewGoogleSmartShopping_On_ConnectedAccountEmailFailsTokenExpiredOrRevoked {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig('some-access-token', 'some-refresh-token', 0, EXAMPLE_EMAIL);
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createGoogleApiService({
        UserInfo => FAILED_USER_INFO_RESPONSE_WITH_EMAIL
    });
    my $SUT = $self->_createSUT($ApiService);

    $SUT->UnityViewGoogleSmartShopping($Servlet);

    ok(
        WasCalled($ApiService, 'getAndSaveUserInfo'),
        $SubTestName . ' - Verify that lookup of Google account email is attempted'
    );
    ok(
        !WasCalled($ApiService, 'getMerchantCenterAccounts'),
        $SubTestName . ' - Verify that getMerchantCenterAccounts was not called when AccountInfo fails'
    );
    ok( WasCalled($SUT, '_buildConnectOAuthURL'), $SubTestName . ' - Verify that OAuth connect URL was built' );
    is(
        $Servlet->vars('TokenExpiredOrRevoked'),
        1,
        $SubTestName . ' - Ensure servlet flag is set for expired or revoked RefreshToken.'
    );
    is(
        $Servlet->vars('ConnectedGoogleAccountEmail'),
        EXAMPLE_EMAIL,
        $SubTestName . ' - Check that saved email was retrieved.'
    );
}

sub test_GoogleOAuthCallback_On_OnboardingCallbackSuccess_And_SuccessfulTokensExchange {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig('some-access-token', 'some-refresh-token');
    my $AlreadyExistingConfig = 0;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = SUCCESSFUL_GOOGLE_OAUTH_CALLBACK_RESPONSE;
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createGoogleApiService({
        OAuthTokens => SUCCESSFUL_GOOGLE_OAUTH_TOKENS_EXCHANGE_RESPONSE
    });
    my $SUT = $self->_createSUT($ApiService);

    $SUT->GoogleOAuthCallback($Servlet);
    my $aWriteHTTPResponseParams = GetParametersFromMockifyCall($SUT, '_writeHTTPResponse');
    my $Code = $aWriteHTTPResponseParams->[1];
    my $hParams = $aWriteHTTPResponseParams->[2];
    my $Content = $aWriteHTTPResponseParams->[3];

    ok(
        WasCalled($Shop, 'existsChild'),
        $SubTestName . ' - Verify that access token is being saved into GoogleSmartShoppingConfig object '
    );
    ok(
        WasCalled($Shop, 'insertChild'),
        $SubTestName . ' - Verify that GoogleSmartShoppingConfig object gets created because did not exist before'
    );
    is(
        $Code,
        HTTP_OK,
        $SubTestName . ' - Ensure HTTP response type is OK'
    );
    is_deeply(
        $hParams,
        { 'ContentType' => 'text/html' },
        $SubTestName . ' - Ensure HTTP response with ContentType set to text/html'
    );
    is(
        $Content,
        SUCCESSFUL_CALLBACK_JS_CONTENT_RESPONSE,
        $SubTestName . ' - Ensure $Servlet->writeContentOutput() was called with the correct JS to refresh the parent window'
    );
}

sub test_GoogleOAuthCallback_On_OnboardingCallbackError {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = undef;
    my $AlreadyExistingConfig = 0;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = UNSUCCESSFUL_GOOGLE_OAUTH_CALLBACK_RESPONSE;
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $SUT = $self->_createSUT();

    $SUT->GoogleOAuthCallback($Servlet);
    my $aWriteHTTPResponseParams = GetParametersFromMockifyCall($SUT, '_writeHTTPResponse');
    my $Code = $aWriteHTTPResponseParams->[1];
    my $hParams = $aWriteHTTPResponseParams->[2];
    my $Content = $aWriteHTTPResponseParams->[3];

    is(
        $Code,
        HTTP_FORBIDDEN,
        $SubTestName . ' - Ensure HTTP response type is 403 Forbidden'
    );
    is_deeply(
        $hParams,
        { 'ContentType' => 'text/html' },
        $SubTestName . ' - Ensure HTTP response with ContentType set to text/html'
    );
    is(
        $Content,
        ACCESS_DENIED_CALLBACK_JS_CONTENT_RESPONSE,
        $SubTestName . ' - Ensure $Servlet->writeContentOutput() was called with the correct JS to send access_denied error to the parent window'
    );
}

sub test_GoogleOAuthCallback_On_OnboardingCallbackSuccess_And_UnsuccessfulTokensExchange {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig('some-access-token', 'some-refresh-token');
    my $AlreadyExistingConfig = 0;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = SUCCESSFUL_GOOGLE_OAUTH_CALLBACK_RESPONSE;
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createGoogleApiService({
        OAuthTokens => UNSUCCESSFUL_GOOGLE_OAUTH_TOKENS_EXCHANGE_RESPONSE
    });
    my $SUT = $self->_createSUT($ApiService);

    $SUT->GoogleOAuthCallback($Servlet);
    my $aWriteHTTPResponseParams = GetParametersFromMockifyCall($SUT, '_writeHTTPResponse');
    my $Code = $aWriteHTTPResponseParams->[1];
    my $hParams = $aWriteHTTPResponseParams->[2];
    my $Content = $aWriteHTTPResponseParams->[3];

    is(
        $Code,
        HTTP_INTERNAL_SERVER_ERROR,
        $SubTestName . ' - Ensure HTTP response type is 500 Internal server error'
    );
    is_deeply(
        $hParams,
        { 'ContentType' => 'text/html' },
        $SubTestName . ' - Ensure HTTP response with ContentType set to text/html'
    );
    is(
        $Content,
        MISC_ERROR_CALLBACK_JS_CONTENT_RESPONSE,
        $SubTestName . ' - Ensure $Servlet->writeContentOutput() was called with the correct JS to send INTERNAL_SERVER_ERROR to the parent window'
    );
}

sub test_GoogleOAuthCallback_On_OnboardingCallbackErrorSingleQuote {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig('some-access-token', 'some-refresh-token');
    my $AlreadyExistingConfig = 0;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = GOOGLE_OAUTH_CALLBACK_ERROR_SINGLE_QUOTE;
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $SUT = $self->_createSUT();

    $SUT->GoogleOAuthCallback($Servlet);
    my $aWriteHTTPResponseParams = GetParametersFromMockifyCall($SUT, '_writeHTTPResponse');
    my $Code = $aWriteHTTPResponseParams->[1];
    my $hParams = $aWriteHTTPResponseParams->[2];
    my $Content = $aWriteHTTPResponseParams->[3];

    is(
        $Code,
        HTTP_FORBIDDEN,
        $SubTestName . ' - Ensure HTTP response type is 403 Forbidden'
    );
    is_deeply(
        $hParams,
        { 'ContentType' => 'text/html' },
        $SubTestName . ' - Ensure HTTP response with ContentType set to text/html'
    );
    is(
        $Content,
        QUOTE_ESCAPED_CALLBACK_JS_CONTENT_RESPONSE,
        $SubTestName . ' - Ensure callback error string concatenated into JavaScript was properly escaped'
    );
}

sub test_GoogleOAuthCallback_On_OnboardingCallbackSuccess_And_TokensExchangeSingleQuoteError {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig('some-access-token', 'some-refresh-token');
    my $AlreadyExistingConfig = 0;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = SUCCESSFUL_GOOGLE_OAUTH_CALLBACK_RESPONSE;
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createGoogleApiService({
        OAuthTokens => TOKENS_EXCHANGE_SINGLE_QUOTE_ERROR_RESPONSE
    });
    my $SUT = $self->_createSUT($ApiService);

    $SUT->GoogleOAuthCallback($Servlet);
    my $aWriteHTTPResponseParams = GetParametersFromMockifyCall($SUT, '_writeHTTPResponse');
    my $Code = $aWriteHTTPResponseParams->[1];
    my $hParams = $aWriteHTTPResponseParams->[2];
    my $Content = $aWriteHTTPResponseParams->[3];

    is(
        $Code,
        HTTP_INTERNAL_SERVER_ERROR,
        $SubTestName . ' - Ensure HTTP response type is 500 Internal Server Error'
    );
    is_deeply(
        $hParams,
        { 'ContentType' => 'text/html' },
        $SubTestName . ' - Ensure HTTP response with ContentType set to text/html'
    );
    is(
        $Content,
        QUOTE_ESCAPED_CALLBACK_JS_CONTENT_RESPONSE,
        $SubTestName . ' - Ensure token exchange error string concatenated into JavaScript was properly escaped'
    );
}

sub test_GoogleOAuthCallback_On_OnboardingCallbackUnknownParameter {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig('some-access-token', 'some-refresh-token');
    my $AlreadyExistingConfig = 0;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = GOOGLE_OAUTH_CALLBACK_UNKNOWN_PARAMETER;
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $SUT = $self->_createSUT();

    $SUT->GoogleOAuthCallback($Servlet);
    my $aWriteHTTPResponseParams = GetParametersFromMockifyCall($SUT, '_writeHTTPResponse');
    my $Code = $aWriteHTTPResponseParams->[1];
    my $hParams = $aWriteHTTPResponseParams->[2];
    my $Content = $aWriteHTTPResponseParams->[3];

    is(
        $Code,
        HTTP_BAD_REQUEST,
        $SubTestName . ' - Ensure HTTP response type is 400 Bad Request for unknown parameter to callback ChangeAction'
    );
    is_deeply(
        $hParams,
        { 'ContentType' => 'text/html' },
        $SubTestName . ' - Ensure HTTP response with ContentType set to text/html'
    );
    is(
        $Content,
        BAD_REQUEST_JS_CONTENT_RESPONSE,
        $SubTestName . ' - Ensure content contains JS for window close - consistency with other cases where popup is closed'
    );
}


sub test_DisconnectConnectedAccount_RevokeToken_NoConfig {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = undef;
    my $AlreadyExistingConfig = 0;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createGoogleApiService(undef);

    # given: 'On-boarding NOT done, GoogleSmartShoppingConfig not defined '
    my $SUT = $self->_createSUT($ApiService);

    # when: 'The Merchant clicks disconnect button'
    $SUT->DisconnectPersonalAccount($Servlet);

    # then: 'Make request to google API to revoke the token, delete gss config from shop'
    ok(!WasCalled($ApiService, 'revokeAccessToken'),
        $SubTestName . ' - Verify service revokeAccessToken was NOT called');
    ok(!WasCalled($SUT, '_deleteGoogleSmartShoppingConfig'),
        $SubTestName . ' - Verify _deleteGoogleSmartShoppingConfig was NOT called');

}

sub test_DisconnectConnectedAccount_RevokeToken_Successful {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig('some-access-token', 'some-refresh-token');
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createGoogleApiService(REVOKE_TOKEN_RESPONSE_SUCCESSFUL);

    # setup
    my $SUT = $self->_createSUT($ApiService);

    # given: 'On-boarding done, connected account email successful retrieved '
    $SUT->UnityViewGoogleSmartShopping($Servlet);
    ok(WasCalled($ApiService, 'getAndSaveUserInfo'), $SubTestName . ' - Verify getAndSaveUserInfo was called');
    ok( !WasCalled($SUT, '_buildConnectOAuthURL'), $SubTestName . ' - Verify that OAuth connect URL was NOT built' );

    # when: 'The Merchant clicks disconnect button'
    $SUT->DisconnectPersonalAccount($Servlet);

    # then: 'Make request to google API to revoke the token, delete gss config from shop'
    ok(WasCalled($ApiService, 'revokeAccessToken'),
        $SubTestName . ' - Verify service revokeAccessToken was called');
    ok(WasCalled($SUT, '_deleteGoogleSmartShoppingConfig'),
        $SubTestName . ' - Verify _deleteGoogleSmartShoppingConfig was called');

}


sub test_DisconnectConnectedAccount_RevokeToken_Error {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig('some-access-token', 'some-refresh-token');
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createGoogleApiService(REVOKE_TOKEN_RESPONSE_ERROR);

    # given: 'On-boarding done, connected account email successful retrieved '
    my $SUT = $self->_createSUT($ApiService);

    # when: 'Merchant clicks Disconnect button', service call returns an error
    $SUT->DisconnectPersonalAccount($Servlet);

    # then: Delete config, call revokeAccessToken, executeFormError should be called
    ok(WasCalled($SUT, '_deleteGoogleSmartShoppingConfig'),
        $SubTestName . ' - Verify _deleteGoogleSmartShoppingConfig was called');
    ok(WasCalled($ApiService, 'revokeAccessToken'),
        $SubTestName . ' - Verify service revokeAccessToken was called');
    ok(WasCalled($Servlet->form(), 'executeFormError'),
        "$SubTestName - Verify form error was executed.");

}

sub test_ConnectedAccount_MerchantCenterRequirements_Not_Met {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig('some-access-token', 'some-refresh-token');
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createGoogleApiService({
        UserInfo => SUCCESSFUL_USER_INFO_RESPONSE
    });
    my $SUT = $self->_createSUT($ApiService);

    $SUT->UnityViewGoogleSmartShopping($Servlet);

    is(
        $Servlet->vars('GoogleSmartShoppingConfig')->{'MerchantCenterAccountName'},
        "MerchantCenterAccountName",
        "$SubTestName - MerchantCenterAccountName is assigned to the Servlet"
    );

    is(
        $Servlet->vars('GoogleSmartShoppingConfig')->{'MerchantCenterAccountID'},
        "MerchantCenterAccountID",
        "$SubTestName - MerchantCenterAccountID is assigned to the Servlet"
    );

    is(
        $Servlet->vars('GoogleSmartShoppingConfig')->{'ChecklistBillingTOC'},
        0,
        "$SubTestName - Google Merchant Center billing terms and conditions requirement is not met"
    );
    is(
        $Servlet->vars('GoogleMerchantCenterRequirementsMet'),
        0,
        "$SubTestName - Google Merchant Center requirements not met warning is shown"
    );
}

sub test_CreateMerchantCenterAccount_Success {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig('some-access-token', 'some-refresh-token', 1);
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {
        ConnectedGoogleAccountEmail => ['ConnectedGoogleAccount@Email'],
        HasAdultContent => [0],
        newMerchantAccountAcceptTerms => [1],
    };
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createGoogleApiService({
        createMerchantCenterSubAccount => {
            id => ACCOUNT_ID
        }
    });
    my $SUT = $self->_createSUT($ApiService);

    $SUT->CreateMerchantCenterAccount($Servlet);

    is_deeply(
        $Servlet->vars('ResponseContent'),
        {id => ACCOUNT_ID},
        "$SubTestName - MerchantCenter SubAccount is successfuly created"
    );
}

sub test_CreateMerchantCenterAccount_Fail_Form {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig('some-access-token', 'some-refresh-token');
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {
        ConnectedGoogleAccountEmail => ['ConnectedGoogleAccount@Email'],
        HasAdultContent => [""],
        newMerchantAccountAcceptTerms => [1],
    };
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createGoogleApiService({
        createMerchantCenterSubAccount => {
            id => ACCOUNT_ID
        }
    });
    my $SUT = $self->_createSUT($ApiService);

    $SUT->CreateMerchantCenterAccount($Servlet);

    ok(WasCalled($Servlet->form(), 'executeFormError'),
        "$SubTestName - Verify form error was executed.");

    my $aExecuteFormErrorParams = GetParametersFromMockifyCall($Servlet->form(), 'executeFormError');

    is_deeply(
        $aExecuteFormErrorParams->[0],
        {
            'Name'   => 'CreationFailed',
            'Reason' => ERROR_FORM_SUBMIT
        },
        $SubTestName . ' - Ensure Form error gets thrown'
    );

}

sub test_CreateMerchantCenterAccount_Fail_Api {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig('some-access-token', 'some-refresh-token', 1);
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {
        ConnectedGoogleAccountEmail => ['ConnectedGoogleAccount@Email'],
        HasAdultContent => [1],
        newMerchantAccountAcceptTerms => [1],
    };
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createGoogleApiService({
        createMerchantCenterSubAccount => {
            id => ACCOUNT_ID,
            throw_error => 1
        }
    });
    my $SUT = $self->_createSUT($ApiService);

    $SUT->CreateMerchantCenterAccount($Servlet);

    ok(WasCalled($Servlet->form(), 'executeFormError'),
        "$SubTestName - Verify form error was executed.");

    my $aExecuteFormErrorParams = GetParametersFromMockifyCall($Servlet->form(), 'executeFormError');

    is_deeply(
        $aExecuteFormErrorParams->[0],
        {
            'Name'   => 'CreationFailed',
            'Reason' => ERROR_TOO_MANY_SUBACCOUNTS
        },
        $SubTestName . ' - Ensure Api error too_many_accounts gets thrown'
    );

}

#----------------------------------------------------------------------------------------
# Mockery
#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;
    my ($ApiService) = @_;

    my $MockifySut = Test::Mockify::Sut->new('DE_EPAGES::GoogleSmartShopping::UI::Shop');
    $MockifySut->mock('_buildConnectOAuthURL')
        ->when(Object('DE_EPAGES::Presentation::Servlet::Permission'), Object('DE_EPAGES::Shop::API::Object::Shop'))
        ->thenReturn(CONNECT_OAUTH_URL);
    $MockifySut->mock('_deleteGoogleSmartShoppingConfig')
        ->whenAny()
        ->thenReturnUndef();
    $MockifySut->mock('_writeHTTPResponse')
        ->whenAny()
        ->thenReturnUndef();
    $MockifySut->mockImported('DE_EPAGES::GoogleSmartShopping::API::Tools', 'LogGoogleSmartShopping')
        ->whenAny()
        ->thenReturnUndef();
    if (defined($ApiService)) {
        $MockifySut->mockStatic('DE_EPAGES::GoogleSmartShopping::API::GoogleApiService::new')
            ->when(HashRef())
            ->thenReturn($ApiService);
    }

    return $MockifySut->getMockObject();
}

sub _createGoogleSmartShoppingConfig {
    my $self = shift;
    my ($AccessToken, $RefreshToken, $AreGoogleMerchantCenterRequirementsMet, $Email) = @_;

    $AreGoogleMerchantCenterRequirementsMet //= 0;

    my $hAttributes = {
        AccountEmail               => $Email // undef,
        AccessToken                => $AccessToken,
        RefreshToken               => $RefreshToken,
        ChecklistBillingTOC        => $AreGoogleMerchantCenterRequirementsMet,
        ChecklistContactInfo       => $AreGoogleMerchantCenterRequirementsMet,
        ChecklistGoogleAdsPolicies => $AreGoogleMerchantCenterRequirementsMet,
        ChecklistReturnPolicy      => $AreGoogleMerchantCenterRequirementsMet,
        MerchantCenterAccountName => "MerchantCenterAccountName", 
        MerchantCenterAccountID => "MerchantCenterAccountID",
    };
    my $MockifyConfig = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::GoogleSmartShopping::API::Object::GoogleSmartShoppingConfig', $hAttributes
    );
    $MockifyConfig->addMockWithReturnValueAndParameterCheck('set', undef, [ 'hashref' ]);

    return $MockifyConfig->getMockObject();
}

sub _createGoogleApiService {
    my $self = shift;
    my ($hServiceResponse) = @_;

    my $Mockify = Test::Mockify->new('DE_EPAGES::GoogleSmartShopping::API::GoogleApiService');
    $Mockify->mock('getOAuthTokens')
        ->when(String())
        ->thenReturn($hServiceResponse->{'OAuthTokens'} // {});
    $Mockify->mock('getAndSaveUserInfo')
        ->when()
        ->thenReturn($hServiceResponse->{'UserInfo'} // {});
    $Mockify->mock('revokeAccessToken')
        ->when(String())
        ->thenReturn($hServiceResponse // {});
    $Mockify->mock('getMerchantCenterAccounts')
        ->when()
        ->thenReturn($hServiceResponse->{'MerchantCenterAccounts'} // {});

    if ($hServiceResponse->{'createMerchantCenterSubAccount'}->{'throw_error'}) {
        $Mockify->mock('createMerchantCenterSubAccount')
            ->when(String(), Number())
            ->thenThrowError(ERROR_TOO_MANY_SUBACCOUNTS);
    } else {
        $Mockify->mock('createMerchantCenterSubAccount')
            ->when(String(), Number())
            ->thenReturn($hServiceResponse->{'createMerchantCenterSubAccount'} // {});
    }

    return $Mockify->getMockObject();
}

sub _createServlet {
    my $self = shift;
    my ($Shop, $ServletForm) = @_;

    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new($Shop);
    $MockifyServlet->addMockWithReturnValue('form', $ServletForm);

    return $MockifyServlet->getMockObject();
}

sub _createShop {
    my $self = shift;
    my ($GoogleSmartShoppingConfig, $AlreadyExistingConfig) = @_;

    my $hAttributes = {
        'ID' => undef,
        GoogleSmartShoppingConfig => $GoogleSmartShoppingConfig
    };
    my $MockifyShop = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', $hAttributes);
    $MockifyShop->addMockWithReturnValueAndParameterCheck(
        'existsChild', $AlreadyExistingConfig, [ {'string' => 'GoogleSmartShoppingConfig'} ]
    );
    $MockifyShop->addMockWithReturnValueAndParameterCheck(
        'insertChild', $GoogleSmartShoppingConfig, [ {'string' => 'GoogleSmartShoppingConfig'}, 'hashref' ]
    );

    return $MockifyShop->getMockObject();
}

sub _createServletForm {
    my $self = shift;
    my ($hQueryParameters) = @_;

    my $Mockify = Test::Mockify->new( 'DE_EPAGES::Presentation::API::Form');
    $Mockify->mock('queryValues')
        ->when()
        ->thenReturn($hQueryParameters);

    $Mockify->mock('formValues')
        ->when()
        ->thenReturn($hQueryParameters);

    $Mockify->mock('executeFormError')
        ->when(HashRef())
        ->thenReturnUndef();

    return $Mockify->getMockObject();
}

__PACKAGE__->runScript();

