#==============================================================================
# package      DE_EPAGES::Order::API::LineItemContainer
# state        public
# description   The line item container
#==============================================================================
package DE_EPAGES::Order::API::LineItemContainer;
use base Exporter;
use strict;

our @EXPORT_OK = qw( CheckSavePayment CheckSaveShipping SavePayment SaveShipping UpdateTaxModel );

use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Object::API::Factory qw ( Transaction );
use DE_EPAGES::Shop::API::Address qw ( IsTaxModelNet );
use DE_EPAGES::Shop::API::Constants qw ( TAX_MODEL_NET TAX_MODEL_GROSS );


#========================================================================================
# function     _SaveShippingOrPayment
# state        private
#----------------------------------------------------------------------------------------
# syntax       _SaveShippingOrPayment($Container, $NewMethod, $ShippingOrPayment);
# syntax       _SaveShippingOrPayment($Container, undef, $ShippingOrPayment);
# example      _SaveShippingOrPayment($Container, $ShippingMethod, 'Shipping');
# example      _SaveShippingOrPayment($Container, undef, 'Payment');
#----------------------------------------------------------------------------------------
# input        $Container | line item container | object
# input        $NewMethod | (optional) shipping method | object
# input        $ShippingOrPayment | 'Shipping' or 'Payment' | string
#========================================================================================
sub _SaveShippingOrPayment {
    my ($Container, $NewMethod, $ShippingOrPayment) = @_;

    my $LineItem = $Container->get($ShippingOrPayment);
    return unless defined $LineItem;
    my $OldMethod = $LineItem->get($ShippingOrPayment.'Method');
    $NewMethod = $OldMethod unless defined $NewMethod;

    # get all methods if we change a CustomerOrder
    my $aPossibleMethods;
    if ($Container->parent->instanceOf('CustomerOrder')) {
        $aPossibleMethods = $Container->getSite()->get($ShippingOrPayment.'Methods');
    } else {
        $aPossibleMethods = $Container->get($ShippingOrPayment.'Methods');
    }

    #  was removed at mbo or changed at basket
    if (not defined $NewMethod
        or
        not grep { $_->id == $NewMethod->id } @$aPossibleMethods) {
        # selected is not possible use default or first
        my @Defaults = grep { $_->get('IsDefault') } @$aPossibleMethods;
        $NewMethod = scalar @Defaults ? $Defaults[0] : $aPossibleMethods->[0];

        Warning($ShippingOrPayment.'MethodChanged', {
            'NewMethod' => $NewMethod,
            'OldMethod' => $OldMethod,
            'Name'      => $ShippingOrPayment .'Method',
        }) if defined $NewMethod;
    }
    if (not defined $NewMethod) {
        Warning('No'.$ShippingOrPayment.'Method', {
            'OldMethod' => $OldMethod,
            'Name'      => $ShippingOrPayment .'Method',
        });
        $LineItem->delete;
    } elsif (not defined $OldMethod or $NewMethod->id != $OldMethod->id) {
        Transaction(sub {
            $LineItem->delete;
            $NewMethod->addToContainer($Container,{});
        });
    }
    return;
}

#========================================================================================
# function     SavePayment
#----------------------------------------------------------------------------------------
# syntax       SavePayment($Container, $NewPaymentMethod);
# state        public
#----------------------------------------------------------------------------------------
# description  Set the new payment method for container.
#               Adds a Warning 'PaymentMethodChanged' if another payment method is used,
#               because the $NewPaymentMethod is not avaible (not visible, no price, ...).
#               If $NewPaymentMethod is undefined, then the current payment method is
#               re-validated. The warning 'PaymentMethodChanged' is added if the current
#               payment is no longer available.
#----------------------------------------------------------------------------------------
# input        $Container | line item container | object
# input        $NewPaymentMethod | payment method (optional) | object
#========================================================================================
sub SavePayment ($;$) {
    my ($Container, $NewPaymentMethod) = @_;

    $Container->addLineItemPayment() unless $Container->get('Payment');
    _SaveShippingOrPayment($Container, $NewPaymentMethod, 'Payment');
}

#========================================================================================
# function     SaveShipping
#----------------------------------------------------------------------------------------
# syntax       SaveShipping($Container, $NewShippingMethod);
# state        public
#----------------------------------------------------------------------------------------
# description  Set the new shipping method for container.
#               Adds a Warning 'ShippingMethodChanged' if another Shipping method is used,
#               because the $NewShippingMethod is not avaible (not visible, no price, ...).
#               If $NewShippingMethod is undefined, then the current shipping method is
#               re-validated. The warning 'ShippingMethodChanged' is added if the current
#               shipping is no longer available.
#----------------------------------------------------------------------------------------
# input        $Container | line item container | object
# input        $NewShippingMethod | shipping method (optional) | object
#========================================================================================
sub SaveShipping ($;$) {
    my ($Container, $NewShippingMethod) = @_;
    $Container->addLineItemShipping() unless $Container->get('Shipping');
    _SaveShippingOrPayment($Container, $NewShippingMethod, 'Shipping');
}

#========================================================================================
# function     UpdateTaxModel
# state        public
#----------------------------------------------------------------------------------------
# syntax       UpdateTaxModel($LineItemContainer);
#----------------------------------------------------------------------------------------
# description  Updates the tax model of the LineItemContainer based on the billing address.
#               Set tax model to net, when country of billing address in eu and vatid
#               is valid.
#               Set the container TaxModel to gross is shop is VAT released
#----------------------------------------------------------------------------------------
# input        $LineItemContainer | lineitemcontainer | object
# return       $NewTaxModel | new tax model if tax model was changed, otherwise undef | int
#========================================================================================
sub UpdateTaxModel {
    my ($LineItemContainer) = @_;
    return unless $LineItemContainer;

    my $BillingAddress = $LineItemContainer->parent->get('BillingAddress');
    return unless $BillingAddress;

    my $Shop = $LineItemContainer->getSite();

    # test if we have a VAT released shop
    # set the TaxModel to gross  in every case
    if ($Shop->get('VATreleased')) {
        $LineItemContainer->set({'TaxModel' => TAX_MODEL_GROSS});
        return ;
    }

    # normal handling for not VAT released shop
    my $TaxModelChanged;

    # we change the tax model from gross to net by billing address
    my $OldTaxModel = $LineItemContainer->get('TaxModel');
    if (IsTaxModelNet($Shop, $BillingAddress)) {
        $LineItemContainer->set({'TaxModel' => TAX_MODEL_NET}) unless $OldTaxModel == TAX_MODEL_NET;
        $TaxModelChanged = 1;
    }

    # we change the tax model from gross to net by shipping address
    my $ShippingAddress = $LineItemContainer->parent->get('ShippingAddress');
    if ((not $TaxModelChanged) and ( defined $ShippingAddress)) {
        if (IsTaxModelNet($Shop, $ShippingAddress)) {
            $LineItemContainer->set({'TaxModel' => TAX_MODEL_NET}) unless $OldTaxModel == TAX_MODEL_NET;
            $TaxModelChanged = 1;
        }
    }

    # we change the tax model from net to gross:
    # - only if we have a gross shop
    # - and $TaxModelChanged was not set above
    if ( ($Shop->get('TaxModel') == TAX_MODEL_GROSS) and
         (not $TaxModelChanged) and
         ($OldTaxModel != TAX_MODEL_GROSS) ) {

            $LineItemContainer->set({'TaxModel' => TAX_MODEL_GROSS});
            $TaxModelChanged = 1;
    }

    return $TaxModelChanged ? $LineItemContainer->get('TaxModel') : undef;
}

#========================================================================================
# function     CheckSavePayment
#----------------------------------------------------------------------------------------
# syntax       CheckSavePayment($Container);
# state        public
#----------------------------------------------------------------------------------------
# description  Checks if the default payment method of a lineitemcontainer is in the
#               list of valid maymentss. If not it will be replaced by a valid one.
#----------------------------------------------------------------------------------------
# input        $Container | line item container | object
#========================================================================================
sub CheckSavePayment ($) {
    my ($Container) = @_;
    my $aPaymentMethods = $Container->get('PaymentMethods');
    my $LineItemPayment = $Container->get('Payment');
    my $foundValidPayment = 0;
    foreach my $PaymentMethod (@$aPaymentMethods) {
        if (defined $LineItemPayment and ($PaymentMethod->id == $LineItemPayment->get('PaymentMethod')->id)) {
            $foundValidPayment = 1;
            last;
        }
    }
    if (!$foundValidPayment && scalar(@$aPaymentMethods)>0) {
        SavePayment($Container,$aPaymentMethods->[0]);
    }
}

#========================================================================================
# function     CheckSaveShipping
#----------------------------------------------------------------------------------------
# syntax       CheckSaveShipping($Container);
# state        public
#----------------------------------------------------------------------------------------
# description  Checks if the default shipping method of a lineitemcontainer is in the
#               list of valid shippings. If not it will be replaced by a valid one.
#----------------------------------------------------------------------------------------
# input        $Container | line item container | object
#========================================================================================
sub CheckSaveShipping ($) {
    my ($Container) = @_;
    my $aShippingMethods = $Container->get('ShippingMethods');
    my $LineItemShipping = $Container->get('Shipping');
    my $foundValidShipping = 0;
    foreach my $ShippingMethod (@$aShippingMethods) {
        if (defined $LineItemShipping and ($ShippingMethod->id == $LineItemShipping->get('ShippingMethod')->id)) {
            $foundValidShipping = 1;
            last;
        }
    }
    if (!$foundValidShipping && scalar(@$aShippingMethods)>0) {
        SaveShipping($Container,$aShippingMethods->[0]);
    }
}

1;
