#========================================================================================
# package      DE_EPAGES::Order::API::SalesVolume
# state        public
#----------------------------------------------------------------------------------------
# description  This package provides the shop functions of order.
#========================================================================================
package DE_EPAGES::Order::API::SalesVolume;
use base Exporter;
use strict;

our @EXPORT_OK = qw (
    GetSalesVolume
    GetSalesVolumeByTaxModel
    GetDeliveryThresholdSalesVolume
);

use DE_EPAGES::Order::DAL::SalesVolume qw (DAL_GetSalesVolume DAL_GetSalesVolumeByTaxModel DAL_GetSalesVolumeDeliveryThresholdEU);
use DE_EPAGES::Core::API::TestParameter qw(TestObject TestString TestInteger );

#========================================================================================
# function     GetSalesVolume
#----------------------------------------------------------------------------------------
# syntax       $SalesVolume = GetSalesVolume($ShopID,$FromDate,$ToDate,$CurrencyID);
#----------------------------------------------------------------------------------------
# description  Return the sales volume for orders by a time range
#----------------------------------------------------------------------------------------
# input        $ShopID | Shop ID | integer
# input        $DateFrom | start date | datetime
# input        $DateTo | end date | datetime
# input        $CurrencyID | currency id | string
# return       $SalesVolume | sales volume before tax | string
#========================================================================================
sub GetSalesVolume {
    my ($Shop,$DateFrom,$DateTo,$CurrencyID) = @_;
    TestObject('Shop', $Shop);
    TestObject('DateFrom', $DateFrom);
    TestObject('DateTo', $DateTo);
    TestString('CurrencyID', $CurrencyID);

    return DAL_GetSalesVolume($Shop->id,$DateFrom,$DateTo,$CurrencyID);
}

#========================================================================================
# function     GetSalesVolumeByTaxModel
#----------------------------------------------------------------------------------------
# syntax       $SalesVolume = GetSalesVolumeByTaxModel($ShopID,$FromDate,$ToDate,$CurrencyID,$TaxModel);
#----------------------------------------------------------------------------------------
# description  Return the sales volume (gross or net) for orders by a time range
#----------------------------------------------------------------------------------------
# input        $Shop | Shop ID | integer
# input        $DateFrom | start date | datetime
# input        $DateTo | end date | datetime
# input        $CurrencyID | currency id | string
# input        $TaxModel | tax model | int
# input        $CustomerID | customer id | string
# return       $SalesVolume | sales volume before tax | string
#========================================================================================
sub GetSalesVolumeByTaxModel {
    my ($Shop,$DateFrom,$DateTo,$CurrencyID,$TaxModel,$CustomerID) = @_;
    TestObject('Shop', $Shop);
    TestObject('DateFrom', $DateFrom);
    TestObject('DateTo', $DateTo);
    TestString('CurrencyID', $CurrencyID);
    TestInteger('TaxModel', $TaxModel, 0, 1);
    TestString('CustomerID', $CustomerID) if defined $CustomerID;

    return DAL_GetSalesVolumeByTaxModel($Shop->id,$DateFrom,$DateTo,$CurrencyID,$TaxModel,$CustomerID);
}

#========================================================================================
# function     GetDeliveryThresholdSalesVolume
#----------------------------------------------------------------------------------------
# syntax       $SalesVolume = GetDeliveryThresholdSalesVolume(
#                              $ShopID, $StartDate , $EndDate);
#----------------------------------------------------------------------------------------
# description  Returns the sales volume for the given duration
#               for all EU countries except the Shop country
#----------------------------------------------------------------------------------------
# input        $ShopID | Shop ID | integer
# input        $StartDate | start date | datetime
# input        $EndDate | end date | datetime
# return       $SalesVolume | sales volume before tax | float
#========================================================================================
sub GetDeliveryThresholdSalesVolume {
    my ($Shop, $StartDate, $EndDate) = @_;
    TestObject('Shop', $Shop);

    return DAL_GetSalesVolumeDeliveryThresholdEU($Shop->id, $StartDate, $EndDate);
}

1;
