use strict;
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::Error qw( GetError ExistsError );
use DE_EPAGES::Core::API::Log qw( LogError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( RunOnStore );
use DE_EPAGES::Object::API::Factory qw ( LoadObject );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);
use DE_EPAGES::Order::API::Cartridge;
use DE_EPAGES::Shop::API::System qw( GetUnityShopIDs );
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );

#========================================================================================
# function         InitializeDeliveryThresholdReachedNotification
# state            public
#----------------------------------------------------------------------------------------
# syntax           InitializeDeliveryThresholdReachedNotification();
#----------------------------------------------------------------------------------------
# description      Adds and activates the notification for delivery thresholds
#                       to all existing Unity shops.
#========================================================================================
sub InitializeDeliveryThresholdReachedNotification {

    print "Reading all Unity shops\n";
    my $aShopIDs = GetUnityShopIDs();

    my $Cartridge = DE_EPAGES::Order::API::Cartridge->new;

    foreach my $ShopID (@$aShopIDs) {
        eval {
            my $Shop = LoadObject($ShopID);
            # import shop notifications
            ImportObjects($Cartridge->xmlDirectory().'/ShopNotificationDeliveryThreshold.xml', $Shop->pathString)
                unless $Shop->folder('ShopNotifications')->existsChild('DeliveryThresholdReached');
        };
        warn('Patch failed in shop ' . $ShopID . " with error \"" . GetError() . "\".")
            if ExistsError();
    }

    eval { SynchronizeCache(); };
    LogError(GetError()) if ExistsError();

    return;
}

sub Main {

    local $| = 1;

    my $Password;
    my $StoreName;
    my $Help;

    GetOptions(
        'help'        => \$Help,
        'passwd=s'    => \$Password,
        'storename=s' => \$StoreName,
    );

    usage() if $Help;

    unless ( defined $StoreName ) {
        print STDERR "Missing parameter -storename.\n\n";
        usage();
    }
    my $time1 = time;

    RunOnStore(
        Store      => $StoreName,
        DBPassword => $Password,
        Sub        => sub { InitializeDeliveryThresholdReachedNotification(); },
    );
    my $time2 = time;
    printf "need %.1f seconds\n", $time2 - $time1;
}

sub usage {
    print <<END_USAGE;
Description:
    Adds and activates the notification for delivery thresholds
    to all existing Unity shops.

Usage:
    perl $0 [options] [flags] datafiles

options:
    -passwd     database user password                          ()
    -storename  name of store                                   ()
flags:
    -help       show the command line options

Example:
    perl $0 -storename Store
END_USAGE
    exit 2;
}

RunScript( 'Sub' => \&Main );