#========================================================================================
# package      DE_EPAGES::PriceList::Hooks::ProductService
# state        public
# description  extends PoductService with elements
#========================================================================================
package DE_EPAGES::PriceList::Hooks::ProductService;

use strict;
use DE_EPAGES::PriceList::API::Constants qw( UNITY_PRICELIST_PREFIX );
use DE_EPAGES::Object::API::Factory qw ( LoadObject );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );


#========================================================================================
# function     OnProductServiceGetProductData
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnProductServiceGetProductData( $hParams );
#----------------------------------------------------------------------------------------
# description  Hook function for the API_ProductService_GetProductData hook.
#               Extends the ProductService result (hResult) with bulk price informations
#               for the given product.
#----------------------------------------------------------------------------------------
# input        $hParams |<ul>
#                   <li>Product - product - object</li>
#                   <li>hResult - ProductService result - ref.hash</li>
#                   <li>ProductInfo - some usefull product attribute values from product - ref.hash</li>
#                   <li>ProductData - additional elements from shop needed for formatting etc.  - ref.hash</li>
#                   </ul> | ref.hash
#========================================================================================
sub OnProductServiceGetProductData {
my ($hParams) = @_;

    my $Product               = $hParams->{'Product'};
    my $hProductServiceResult = $hParams->{'hResult'};
    my $hProductInfo          = $hParams->{'ProductInfo'};
    my $hProductData          = $hParams->{'ProductData'};
    my $SessionUserID         = $hParams->{'SessionUserID'};
    my $Shop                  = $Product->getSite();
    my $CurrencyID            = $hProductData->{'CurrencyID'};
    my $UserID                = $hParams->{'UserID'};

    if (defined $UserID && $UserID > 0) {
        $SessionUserID = $UserID;
    }

    # do this only for unity
    return unless $Shop->get('IsUnityRegistered');

    # get bulk prices for product with the defined pricelist alias
    my $PriceListAlias = sprintf('%s%s', UNITY_PRICELIST_PREFIX, $CurrencyID);

    my $aPrices = [];
    my $hDiscounts = {};
    # use session information for getting bulk prices
    if (defined $SessionUserID) {
        my $aSessionUserCustomers = LoadObject($SessionUserID)->get('Customers');
        if (scalar @$aSessionUserCustomers) {
            my $Customer = $aSessionUserCustomers->[0];
            if( (defined $Customer) and (defined $Customer->get('CustomerGroup')) ){
                my $CustomerPriceListAlias = $Customer->get('CustomerGroup')->alias;
                if ($Shop->folder('PriceLists')->existsChild($CustomerPriceListAlias)){
                    my $CustomerPriceList = $Shop->folder('PriceLists')->child($CustomerPriceListAlias);
                    # get prices and fetch also discounts for this prices
                    $aPrices = $CustomerPriceList->getScalePricesByProductID($Product->id, $Shop->get('TaxModel'), $Product->getWithParent('TaxClass'));
                    $hDiscounts->{$_->{'Quantity'}} = $_->{'Discount'} foreach @{ $CustomerPriceList->getQuantityDiscountsByProductID($Product->id) };
                }
            }
        }
    }

    # we have no customer specific prices. get standard bulk prices
    unless (scalar @$aPrices) {
        return unless $Shop->folder('PriceLists')->existsChild($PriceListAlias);
        my $PriceList = $Shop->folder('PriceLists')->child($PriceListAlias);
        $aPrices = $PriceList->getScalePricesByProductID($Product->id, $Shop->get('TaxModel'), $Product->getWithParent('TaxClass'));
    }

    # do nothing, we have no bulk prices or customer specific prices
    return unless scalar @$aPrices;

    my $Formatter  = $hProductData->{'Formatter'};
    my $TaxModel   = $hProductData->{'TaxModel'};

    # test if we have quantities for discounts not existing with this quantities in $aPrices and add as new entries
    # the reason is the inheritance of prices and discounts
    # IMPORTANT: for each discount quantity a entry in prices must exists!!!

    my $aPriceQtys = [];
    #use @$aPriceQtys as helper array for having a fast access and easy tests if an entry exists
    push @$aPriceQtys, $_->{'Quantity'} foreach @$aPrices;

    # if missing and needed, add entry with qty 1 to the prices with the listprice because we can have a discount entry for qty=1 without a bulkprice
    my $QtyOne = 1;
    if (ElementOf($QtyOne, keys %$hDiscounts) and not ElementOf($QtyOne, @$aPriceQtys)){
        push @$aPrices, { 'Quantity' => 1, 'Price' => $hProductServiceResult->{'priceInfo'}->{'price'}->{'amount'}  };
        push @$aPriceQtys,1;
    }
    # now we can find the discount entry who have no entries in prices and add these entries to the prices (inheritance)
    # because we loop over all prices and have to recalculate the prices with discounts later on

    # here we find the price to use with the discount entry
    foreach my $DiscountQuantity (keys %$hDiscounts) {
        unless( ElementOf($DiscountQuantity,@$aPriceQtys)) {
            # go backwards through the quantities and try to find which quantity and price we have to use for the entry we have to add
            foreach my $Q (reverse (0..$DiscountQuantity)){
                if (ElementOf($Q, @$aPriceQtys)){
                    # add new entry for price with the discount quantity and the inherited price
                    my $Price;
                    foreach my $hBulkPrice (@$aPrices) {
                        $Price = $hBulkPrice->{'Price'} if $Q == $hBulkPrice->{'Quantity'};
                    }
                    push @$aPrices, { 'Quantity' => $DiscountQuantity, 'Price' => $Price };
                    last;
                }
            }
        }
    }

    #build structure for bulk prices and add values
    my $BulkPrices = [];
    foreach my $hBulkPrice (@$aPrices) {

        # here we have to find out if we have a discount or an inherited discount
        my $hPriceSavings = undef;
        if(scalar keys %$hDiscounts) {
            # try to find the matching quantity discount entry
            # attention: We always inherit discounts from lower to larger quantities
            # if we have no own entry for the quantity!!
            my $QuantityToUse;
            foreach my $Qty (reverse(0 .. $hBulkPrice->{'Quantity'})) {
                if ( ElementOf($Qty , keys %$hDiscounts ) ) {
                    $QuantityToUse = $Qty;
                    last;
                }
            }
            # calculate the price with the discount if a discount is defined
            if (exists $hDiscounts->{$QuantityToUse}) {
                my %Percent = (
                    'percentage' => $hDiscounts->{$QuantityToUse},
                    'formatted'  => $Formatter->percentage($hDiscounts->{$QuantityToUse})->{'formatted'},
                );
                $hPriceSavings->{'percent'} = \%Percent;
                $hPriceSavings->{'regularBulkPrice'} = $Formatter->price($hBulkPrice->{'Price'}, $CurrencyID, $TaxModel);

                my $Price = $hBulkPrice->{'Price'} - (($hBulkPrice->{'Price'} * $hDiscounts->{$QuantityToUse}) / 100) ;
                $hBulkPrice->{'Price'} = $hProductData->{'SiteLocale'}->roundMoney($Price,$CurrencyID);
            }
        }

        my $hPriceInfo = $Formatter->price( $hBulkPrice->{'Price'}, $CurrencyID, $TaxModel );
        my $hQuantity = $Formatter->quantity( $hBulkPrice->{'Quantity'}, $hProductInfo->{'OrderUnit'} );
        my $hBasePrice = undef;

        # if the product has a reference price we have to add the basePrice info to the bulk prices
        if(defined $hProductServiceResult->{'priceInfo'}->{'basePrice'}) {
            my $ReferencePrice = $Product->getReferencePrice($CurrencyID, $hBulkPrice->{'Price'});
            my $RefPriceFormatted = sprintf '%d %s = %s',
                                    $hProductInfo->{'RefAmount'},
                                    $hProductServiceResult->{'priceInfo'}->{'basePrice'}->{'refQuantity'}->{'unit'},
                                    $hProductData->{'SiteLocale'}->formatMoney($ReferencePrice, $CurrencyID);

            $hBasePrice->{'formatted'} = $RefPriceFormatted;
            $hBasePrice->{'refPrice'} = $Formatter->price( $ReferencePrice, $CurrencyID, $TaxModel );
            $hBasePrice->{'quantity'} = $hProductServiceResult->{'priceInfo'}->{'basePrice'}->{'quantity'};
            $hBasePrice->{'refQuantity'} = $hProductServiceResult->{'priceInfo'}->{'basePrice'}->{'refQuantity'};
        }

        push @$BulkPrices, {
                            'price'         => $hPriceInfo,
                            'quantity'      => $hQuantity,
                            'basePrice'     => $hBasePrice,
                            'priceSavings'  => $hPriceSavings,
                           };
    }

    push @{$hProductServiceResult->{'bulkPriceInfo'}}, @$BulkPrices;
}

1;
