#========================================================================================
# package      DE_EPAGES::Product::UI::JSON
# state        public
#----------------------------------------------------------------------------------------
# description  Common user interface to show, save and delete objects.
#========================================================================================
package DE_EPAGES::Product::UI::JSON;
use base DE_EPAGES::Presentation::UI::JSON;

use strict;
use JSON;
use feature 'switch';
use DE_EPAGES::Content::API::ShortUrl qw ( GetShortUrlByClass );
use DE_EPAGES::Core::API::Constants qw ( IMAGE_FILE_TYPES );
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::File qw ( DeleteFile ExistsFile fsplit CleanPath WriteFile );
use DE_EPAGES::Core::API::Image qw ( TestImage );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Core::API::TestParameter qw ( TestInteger );
use DE_EPAGES::Core::API::Unicode qw ( Unicode2Filename );
use DE_EPAGES::Core::API::Warning qw( Warning );
use DE_EPAGES::Customer::API::Shop qw ( GetCustomerForUser );
use DE_EPAGES::Object::API::Factory qw ( ExistsObject LoadClassByAlias );
use DE_EPAGES::Presentation::API::Template qw ( Translate );
use DE_EPAGES::Presentation::API::Object::TrayBuilder;
use DE_EPAGES::Product::API::Constants qw (
    MBOUI_PRODUCT_DETAIL_PAGE
    PRODUCT_IMAGE_MAX_COUNT
    PRODUCT_IMAGE_SIZE_NAMES
    PRODUCT_IMAGES_SIZES
    VIMEO_API_URL
);
use DE_EPAGES::Product::API::LineItemContainer qw ( UpdateLineItem );
use DE_EPAGES::Product::API::Product qw (
    CropProductImages
    DeleteProductImage
    ScaleImageSizes
    SmartSearchProducts
    GetScaledImages
    GetUnityPrettyUrl
);
use DE_EPAGES::Search::API::Object::PagerSearch qw ( ReplaceWildcards );
use DE_EPAGES::Search::API::SearchDefinition qw ( GetSearchDefinition );
use DE_EPAGES::Search::API::Table::SearchSort qw ( ExistsSearchSortBySearchDefinitionID );
use DE_EPAGES::SEO::API::SEO qw ( PerformSEOCheckAndUpdateObject );
use DE_EPAGES::ShopMailType::API::ShopMailType qw ( GetShopMailType );
use POSIX;
use Readonly;
use Image::Magick;
use List::Util qw (max);
use MIME::Base64;
use LWP::UserAgent;
use HTTP::Status qw( HTTP_NOT_FOUND );

sub ErrorDefinition {
    return {
        'UNKNOWN_ATTRIBUTE_NAME' => 'The attribute #Name is unknown.',
    };
}

sub JSONAppointmentPrice {
    my $self = shift;
    my ($Servlet) = @_;
    my $User = $Servlet->user;
    my $Product = $Servlet->object;

    my $hValues = $Servlet->form->form($Product, 'JSONAppointmentPrice');

    my $Shop = $Product->getSite();
    my $Customer = defined $User ? GetCustomerForUser($User, $Shop) : undef;
    my $DurationUnit;
    if( defined $hValues->{'DurationUnitID'} ) {
        $DurationUnit = $Shop->loadObject($hValues->{'DurationUnitID'}, 'UnitOfMeasurement', {'AllowSystemSite' => 1});
    }
    my $Price = $Product->appointmentPrice(undef, {
        %$hValues,
        'DurationUnit' => $DurationUnit,
        'CurrencyID' => $Servlet->currencyID,
        'TaxModel'   => defined $Customer ? $Customer->get('TaxModel') : $Shop->get('TaxModel'),
        'User'       => $User
    });
    my $Prepayment = $Product->appointmentPrepayment(undef, {
        'CurrencyID' => $Servlet->currencyID,
        'TaxModel'   => defined $Customer ? $Customer->get('TaxModel') : $Shop->get('TaxModel'),
        'User'       => $User,
        'Quantity'   => $hValues->{'Quantity'},
        'DurationUnit' => $DurationUnit,
        'Duration'   => $hValues->{'Duration'},
    });

    return $self->writeContentOutput({
        'AppointmentPrice'=> $Price,
        'FormattedAppointmentPrice' => $Shop->siteLocale( $Servlet->locale )->format($Price, 'money', $Servlet->currencyID),
        'AppointmentPrepayment' => $Prepayment,
        'FormattedAppointmentPrepayment' => defined $Prepayment ? $Shop->siteLocale( $Servlet->locale )->format($Prepayment, 'money', $Servlet->currencyID) : undef,
    });
}

sub JSONSmartSearchProductAlias {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;
    my $Form = $Servlet->form();

    # filter criteria
    my $ExcludeObjectID = $Form->value('Exclude') // 0;
    my $LoadSuperProducts = $Form->value('OmitSuperProducts') ? 0 : 1;
    my $IncludeProductsWithoutPrice = $Form->value('ExcludeProductsWithoutPrice') ? 0 : 1;

    # Search over all strings
    my $aProducts = SmartSearchProducts($Shop, $Form->value('SearchString') );

    my $InputTaxModel = $Shop->get('InputTaxModel');
    my $CurrencyID    = $Shop->get('CurrencyID');

    my @JSONResults = map {
        {
            'alias'     => $_->alias,
            'name'      => $_->get('Name')
        }
    } grep {
        $_->id() != $ExcludeObjectID &&
        ( $IncludeProductsWithoutPrice || defined $_->getListPrice($CurrencyID, $InputTaxModel) ) &&
        ( $LoadSuperProducts || ! $_->isMaster )
    }
    @$aProducts;



    return $self->writeContentOutput({
        'NumResults'    => scalar(@JSONResults),
        'Results'       => \@JSONResults
    });
}

sub JSONCreateNewResource {
    my $self = shift;
    my ($Servlet) = @_;

    my $AppointmentFolder = $Servlet->object;
    my $Shop = $AppointmentFolder->getSite();
    my $Form = $Servlet->form;
    my $hValues = $Form->form($AppointmentFolder, 'CreateNewResource');

    my $ResourceObject;
    if(defined $hValues->{'NewResourceObjectID'} and ExistsObject($hValues->{'NewResourceObjectID'})) {
        #return DE_EPAGES::Product::UI::Product->Save($Servlet);
        $ResourceObject = $Shop->loadObject($hValues->{'NewResourceObjectID'}, 'Product');
        delete $hValues->{$_} foreach (qw ( NewResourceObjectID ));
        $ResourceObject->set($hValues);
        # create calendar child
        $ResourceObject->businessHours;
    } else {
        delete $hValues->{$_} foreach (qw ( NewResourceObjectID ));
        $hValues->{'IsVisible'} = 1 if not defined $hValues->{'IsVisible'};
        $Servlet->form->executeFormError({
            'Reason' => 'NoAlias',
            'Name'   => 'Alias',
            'Form'   => 'CreateNewResource',
        }) unless defined $hValues->{'Alias'};

        # TODO: auto-gen shortURL like with categories
        #$Values{'URI'} = GetShortUrlByClass($ProductFolder, $Shop->get('ProductClass'), $Name) if defined $Name and $Name ne '';
        $ResourceObject = eval{ $self->_createResourceObject($Servlet, $AppointmentFolder, $hValues) };
        if (ExistsError()) {
            my $Error = GetError();
            Error($Error) unless ($Error->code eq 'OBJECT_EXISTS');
            $Servlet->form->executeFormError({
                'Reason'=> $Error->code,
                'Form'  => 'CreateNewResource',
                'Name'  => 'Alias',
                'Message' => $Error->message
            });
        }

        # create calendar child
        $ResourceObject->businessHours;

        $self->writeContentOutput({
            'NewResourceObjectID'=> $ResourceObject->id,
        });
    }

    # update multi-lang name and shorturl
    $self->_setResourceLangAttributes($Shop, $AppointmentFolder, $ResourceObject, $hValues);

    return;
}

#========================================================================================
# function     JSONSearchProducts
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->JSONSearchProducts($Servlet);
# example      <button onclick="dosth(); return false;">dosth</button>
#               <script type="text/javascript">
#                 window.dosth = function() {
#                   var result = new epages.io.Json().loadSync('?', {
#                                 'ObjectID'          : #Shop.ProductFolder.ID, // Product folder
#                                 'ViewAction'        : 'JSONSearchProducts',
#                                 'WildSearchString'  : 'Jack',
#                                 'ErrorAction'       : 'JSONViewResponse',
#                                 'IsAppointment'     : 0,
#                                 'OrderBy'           : 'NameOrAlias',
#                                 'OrderDesc'         : 0,
#                                 'PagerPage'         : 2
#                               });
#                   if(result && result.error && result.error.data){
#                     console.warn("todo: handle error", result.error.data);
#                   } else {
#                     console.debug("result", result, result.data);
#                   }
#                 };
#               </script>
#----------------------------------------------------------------------------------------
# description  JSON handler for product search
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet object | object
#========================================================================================
sub JSONSearchProducts {
    my $self = shift;
    my ($Servlet) = @_;
    my $Folder = $Servlet->object;
    my $SearchForm = $Servlet->form->form($Folder, 'JSONSearchProducts');

    my $ProductClass = LoadClassByAlias('Product');
    my $SearchName = 'MBO-AllStrings';
    my $SearchDefinition = GetSearchDefinition($ProductClass, $SearchName);
    my $LocalizedAttributeName = $ProductClass->attribute('Name');
    my $Shop = $Folder->getSite();
    my %Options = (
        'Servlet' => $Servlet,
        'Object'  => $Folder,
        'SearchDefinition' => $SearchDefinition,
        'Parameters' => {
            'Shop.ID'           => $Shop->id,
            'SearchLanguageID'  => $Servlet->languageID,
            'NameAttributeID'   => $LocalizedAttributeName->id,
            'Search'            => $SearchName,
            'CurrencyID'        => $Shop->get('CurrencyID')
        },
        'UriParameters'  => {
            'Search' => $SearchName,
        },
    );
    #delete $SearchForm->{'StockWarn'} if exists $SearchForm->{'StockWarn'} and $SearchForm->{'StockWarn'} ==  0;

    if(defined $SearchForm->{'PageSize'}) {
        $Options{'PageSize'} = int($SearchForm->{'PageSize'});
    } else {
        $Options{'PageSize'} = 10; ## no critic qw(ProhibitMagicNumbers)
    }

    if(defined $SearchForm->{'OrderBy'}) {
        $Options{'OrderBy'} = $SearchForm->{'OrderBy'};
    }

    if(defined $SearchForm->{'OrderDesc'}) {
        $Options{'OrderDesc'} = int($SearchForm->{'OrderDesc'});
    }

    # undef = both, 0 = only products, 1 = only appoitments
    if(defined $SearchForm->{'IsAppointment'}) {
        $Options{'Parameters'}{'IsAppointment'} = $SearchForm->{'IsAppointment'};
    }

    if(defined $SearchForm->{'IsNotSubProduct'}) {
        $Options{'Parameters'}{'IsNotSubProduct'} = $SearchForm->{'IsNotSubProduct'};
    }

    # EPG-9002: limit the search string
    Readonly::Scalar my $MaxSearchStringLength => 80;
    my $WCSearchString = substr $SearchForm->{'WildSearchString'}, 0, $MaxSearchStringLength;

    # replace * with %
    $Options{'Parameters'}{'SearchString'} = ReplaceWildcards( $WCSearchString );

    my $Search = DE_EPAGES::Search::API::Object::PagerSearch->new(\%Options);
    my $Page = $Search->page;

    my @JSONResults;
    my $hPager = $Page->{'Pager'};

    if($hPager->{'CountOfItems'} > 0) {
        my $PageNumber;
        if(defined $SearchForm->{'PagerPage'}
           && $SearchForm->{'PagerPage'} <= $hPager->{'LastPage'}
           && $SearchForm->{'PagerPage'}  > 0
           ) {
            $PageNumber = int($SearchForm->{'PagerPage'});
        } else {
            $PageNumber = int($hPager->{'Page'});
        }
        my $aFoundObjects = $Search->items($PageNumber ? $PageNumber : 0);

        foreach my $FoundObject ( @$aFoundObjects ) {
            my $hObjectAttributes = $FoundObject->get([ 'ObjectID', 'Alias', 'IsAppointment', 'NameOrAlias', 'SuperProduct' ]);
            my $ProductAttrHash = {
                'ObjectID'      => $hObjectAttributes->{'ObjectID'},
                'Alias'         => $hObjectAttributes->{'Alias'},
                'IsAppointment' => $hObjectAttributes->{'IsAppointment'},
                'NameOrAlias'   => $hObjectAttributes->{'NameOrAlias'}
            };

            if(defined $hObjectAttributes->{'SuperProduct'}) { # is sub product
                # load super product
                my $SuperProduct = $hObjectAttributes->{'SuperProduct'};
                my @AttributeAliases = map { $_->{'Attribute'}->alias } @{$SuperProduct->get('VariationTypeValues')};
                $ProductAttrHash->{'Title'} = join (',', map { $FoundObject->getWithParent($_)->get('ValueOrAlias') } @AttributeAliases);
            }
            push(@JSONResults, $ProductAttrHash);

        }
    }
    $self->writeContentOutput({
        SearchResults => \@JSONResults,
        CountOfItems => $hPager->{'CountOfItems'},
        CountOfPages => $hPager->{'CountOfPages'},
    });
}

sub JSONInsertProducts {
    my $self = shift;
    my ($Servlet) = @_;

    my $Form = $Servlet->form;
    my $LineItemContainer = $Servlet->object;
    my $Shop = $LineItemContainer->getSite;

    my $aProducts = $Shop->loadObjects([$Form->value('ProductIDs')], 'Product');

    my $NewQuantity = defined $Form->value('NewQuantity') ? $Form->value('NewQuantity') : 1 ;
    eval { TestInteger('NewQuantity', $NewQuantity) };
    if(!ExistsError()) {
        foreach my $Product (@$aProducts) {
            eval {
                my $DefaultSubProduct = $Product->get('DefaultSubProduct');
                my $LineItem;
                if (defined $DefaultSubProduct) {
                    $LineItem = $DefaultSubProduct->addToContainer($LineItemContainer, {'Quantity' => $NewQuantity, 'IsEditOrder' => 1});
                } else {
                    $LineItem = $Product->addToContainer($LineItemContainer, {'Quantity' => $NewQuantity, 'IsEditOrder' => 1});
                }
                UpdateLineItem($LineItem, $LineItemContainer);
            };
            if (ExistsError()) {
                my $Error = GetError();
                if (ElementOf($Error->code, qw (ProductNoPrice ProductCanNotAddToBasket ProductCanNotChangedAtBasket))) {
                    $Form->mapFormError({'Name'  => 'AddToBasket'}, $Error);
                    $Form->executeFormError;
                } else {
                    Error($Error);
                }
            }
            $LineItemContainer->recalculate;
        }
    }

}

sub JSONInsertBundleProducts {
    my $self = shift;
    my ( $Servlet ) = @_;

    my $Product = $Servlet->object;
    my $Shop = $Product->getSite;
    my $Form = $Servlet->form;

    my $aProducts = $Shop->loadObjects([$Form->value('ProductIDs')], 'Product');
    my $ProductTaxClassID = $Product->get('TaxClass')->id;
    my $NewQuantity = defined $Form->value('NewQuantity') ? $Form->value('NewQuantity') : 1 ;
    eval { TestInteger('NewQuantity', $NewQuantity) };
    if(!ExistsError()) {
        foreach my $BundleProduct (@$aProducts) {
            next if $BundleProduct->id == $Product->id;
            next if $BundleProduct->get('IsBundleProduct');
            next if $BundleProduct->get('IsAppointment');
            $BundleProduct = $BundleProduct->get('DefaultSubProduct') if defined $BundleProduct->get('DefaultSubProduct');

            if ($BundleProduct->getWithParent('TaxClass')->id != $ProductTaxClassID) {
                $Form->addFormError( {
                        'Form'      => 'SaveBundleProductMaps',
                        'Reason'    => 'DifferentTaxClasses',
                        'Name'      => 'TaxClassID',
                        'Value'     => undef,
                });
                next;
            }

            # prevent adding customizable products
            if ($BundleProduct->get('IsCustomizable')) {
                $Form->addFormError( {
                        'Form'      => 'SaveBundleProductMaps',
                        'Reason'    => 'TargetIsCustomizable',
                        'Name'      => 'NewProduct',
                        'Value'     => undef,
                });
                next;
            }

            $BundleProduct->addToBundle($Product, {'BundledQuantity' => $NewQuantity});
        }
    }

    # Set ecoparticipation to bundle.
    $Product->updateBundleEcoParticipation;

    # Set deposit to bundle.
    $Product->updateBundleDeposit;

    # Renumber BundleProductMaps.
    $Product->renumberBundleProductMaps;

    $Servlet->form->executeFormError;

    return;
}

sub JSONSearchProductsToInsert {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;
    my $hParam = $Servlet->form->form( $Shop, 'JSONSearchProductsToInsert' );

    # Get required values.
    my $LocaleID   = $Servlet->locale();
    my $LanguageID = $Servlet->languageID;
    my $CurrencyID = $Shop->get('CurrencyID');

    # Load search definition.
    my $ProductClass     = LoadClassByAlias('Product');
    my $SearchName       = 'MBO-AllProducts';
    my $SearchDefinition = GetSearchDefinition( $ProductClass, $SearchName );

    # Set default search options.
    my $LocalizedAttributeName = $ProductClass->attribute('Name');
    my %Options;

    # Calculate number entries requested
    my $Pages = $hParam->{'pages'};
    my $EntriesPerPage = $hParam->{'pageEntries'};
    my $PageSize = $EntriesPerPage * $Pages;

    %Options = (
        'Servlet'          => $Servlet,
        'Object'           => $Shop,
        'SearchDefinition' => $SearchDefinition,
        'Page'             => max( ceil($hParam->{'currentPage'} / $Pages), 1 ),  # workaround for GUI caching
        'PageSize'         => $PageSize,
        'Parameters'       => {
            'Shop.ID'          => $Shop->id,
            'SearchLanguageID' => $Servlet->languageID,
            'NameAttributeID'  => $LocalizedAttributeName->id,
            'CurrencyID'       => $CurrencyID,
            'Search'           => $SearchName,
        },
        'UriParameters' => { 'Search' => $SearchName, },
    );

    my $ParentProductID = $hParam->{'ParentObjectID'};

    # Get only subproducts of a product.
    my @PathElements;
    if ( defined $ParentProductID ) {
        my $ParentProduct = $Shop->loadObject( $ParentProductID, 'Product' );
        if ( $ParentProduct->existsChild('SubProducts') ) {
            $Options{'Parameters'}{'Folder.ID'} = $ParentProduct->child('SubProducts')->id;
            foreach my $PathElement ( @{ $ParentProduct->get('PathFromSite') } ) {
                push @PathElements, {
                    'ObjectID'    => $PathElement->id,
                    'NameOrAlias' => $PathElement->get('NameOrAlias')
                };
            }
        }
        else {
            return $self->writeContentOutput( {} );
        }
    }
    else {
        my $ProductFolder = $Shop->child('Products');
        push @PathElements, {
            'ObjectID'    => $ProductFolder->id,
            'NameOrAlias' => Translate( 'Products', undef, 'DE_EPAGES::Product' ) // ''
        };
    }

    # Set sorting.
    my $OrderBy = $hParam->{'orderBy'};
    $Options{'OrderBy'} = 'NameOrAlias';
    if ( defined $OrderBy && ExistsSearchSortBySearchDefinitionID( $SearchDefinition->id, $OrderBy ) ) {
        $Options{'OrderBy'} = $OrderBy;
    }
    my $OrderDesc = $hParam->{'orderDesc'};
    $Options{'OrderDesc'} = $OrderDesc ? 1 : 0;

    # undef = both, 0 = only products, 1 = only appoitments
    $Options{'Parameters'}{'IsAppointment'} = $hParam->{'IsAppointment'}
    if defined $hParam->{'IsAppointment'};

    if ( defined $hParam->{'WildSearchString'} ) {

        # EPG-9002: limit the search string
        Readonly::Scalar my $MaxSearchStringLength => 80;
        my $WCSearchString = substr( $hParam->{'WildSearchString'}, 0, $MaxSearchStringLength );

        # replace * with %
        $Options{'Parameters'}{'SearchString'} = ReplaceWildcards($WCSearchString);
    }

    # OmitSuperProducts: used for bundleproducts, ebay!
    my $NoSuperProducts = $hParam->{'OmitSuperProducts'} ? 1 : 0;
    my $IsNotSubProduct = $NoSuperProducts ? 0 : 1;

    # Get also superproducts if it is a valid search.
    if ( length( $Options{'Parameters'}{'SearchString'} ) > 0 ) {
        $IsNotSubProduct = 0;
    }

    if ( not defined $ParentProductID ) {
        $Options{'Parameters'}{'IsNotSubProduct'} = 1 if ($IsNotSubProduct);
        $Options{'Parameters'}{'NoSuperProducts'} = 1 if ($NoSuperProducts);
    }

    # add exclusion id
    if ($hParam->{'Exclude'}) {
        $Options{'Parameters'}{'ExcludeID'} = $hParam->{'Exclude'};
    }

    # Trigger search.
    my $Search = DE_EPAGES::Search::API::Object::PagerSearch->new( \%Options );
    my $Page   = $Search->page;
    my $hPager = $Page->{'Pager'};

    my $LastPage = ceil( $hPager->{'CountOfItems'} / $EntriesPerPage );

    my $hJSONResults = {};

    # if there is a search result.
    if ( $hPager->{'CountOfItems'} > 0 ) {

        my $CurrentPage = $hParam->{'currentPage'} // 1;

        if ( $CurrentPage > $LastPage or $CurrentPage < 0 ) {
            $CurrentPage = 1;
        }
        my $PagerSearchPage = ceil( $CurrentPage / $Pages );
        my $aFoundObjects = $Search->items($PagerSearchPage);
        my $Formatter     = $Shop->siteLocale($LocaleID);
        my $InputTaxModel = $Shop->get('InputTaxModel');

        my $Counter = 1;
        my $Pager   = ( $PagerSearchPage - 1 ) * $Pages + 1;

        # build up table content.
        foreach my $FoundObject (@$aFoundObjects) {
            my $hObjectAttributes = $FoundObject->get( [ 'ObjectID', 'Alias', 'SuperProduct', 'IsMaster', 'PathFromSite' ] );
            my $ListPrice = $FoundObject->getListPrice( $CurrencyID, $InputTaxModel );
            my $Name = $FoundObject->getWithParent( 'Name', $LanguageID );
            my $SuperProduct = $hObjectAttributes->{'SuperProduct'};
            my $StockLevel = $FoundObject->get('StockLevel');

            my @Paths;
            my $PathLength = scalar @{ $hObjectAttributes->{'PathFromSite'} } - 3;
            if ( $PathLength >= 2 ) {
                for ( my $i = 0; $i <= $PathLength; $i++ ) {
                    push @Paths, {
                        'ObjectID'    => $hObjectAttributes->{'PathFromSite'}->[$i]->id,
                        'NameOrAlias' => $hObjectAttributes->{'PathFromSite'}->[$i]->get('NameOrAlias')
                    };
                }
            }

            my $VariationAttrString;
            # Check if $FoundObject is a SubProduct
            if ( defined $SuperProduct ) {
                $VariationAttrString = $FoundObject->localizedVariationString($LanguageID);
            }

            my $Price = length $ListPrice > 0 ? $Formatter->format( $ListPrice, 'money', $CurrencyID ) : undef;

            my $hRowData = {
                'ObjectID'      => int( $hObjectAttributes->{'ObjectID'} ),
                'Alias'         => $hObjectAttributes->{'Alias'},
                'NameOrAlias'   => $Name,
                'VariationAttr' => $VariationAttrString,
                'ListPrice'     => $Price,
                'StockLevel'    => $StockLevel,
            };

            # If price doesn't exist, set flag for gui handling
            if ( ( not defined $Price ) && $hParam->{'DisableProductsWithoutPrice'} ) {
                $hRowData->{'Disabled'} = 'NotSelectableNoPrice';
            }

            # Master don't have link button to subproduct in search mode
            if ( $hObjectAttributes->{'IsMaster'} && !$hParam->{'WildSearchString'}) {
                $hRowData->{'hasSubTable'} = 1;
            }

            push @{ $hJSONResults->{$Pager} }, $hRowData;

            if ( $Counter % $EntriesPerPage == 0 ) {
                $Pager++;
            }
            $Counter++;
        }
    }

    my $Builder = DE_EPAGES::Presentation::API::Object::TrayBuilder->new({LanguageID => $LanguageID, TranslationPackage => 'DE_EPAGES::Product' });
    # $Id, [$IsSelectable, $IsSortable, $IsLinkable, $IsRightAligned], $TranslationKey, $TranslationPackage
    $Builder->addColumnHeader('ObjectID', [1, 0, 0, 0], undef)
            ->addColumnHeader('Alias', [0, 1, 0, 0], 'SKU')
            ->addColumnHeader('NameOrAlias', [0, 1, 1, 0], 'Name', 'DE_EPAGES::Dictionary')
            ->addColumnHeader('VariationAttr', [0, 0, 0, 0], undef)
            ->addColumnHeader('ListPrice', [0, 1, 0, 1], 'ListPrice')
            ->addColumnHeader('StockLevel', [0, 0, 0, 1], 'StockLevel','DE_EPAGES::Dictionary');

    my $hResponse = {
        'totalNumber'  => $hPager->{'CountOfItems'},
        'lastPage'     => $LastPage,
        'tableContent' => $hJSONResults,
        'tableHead'    => $Builder->getHeaderList(),
        'orderBy'      => $Options{'OrderBy'},
        'orderDesc'    => $Options{'OrderDesc'},
        'path'         => \@PathElements
    };

    # write result data on the current servlet.
    $self->writeContentOutput($hResponse);
}

sub _setResourceLangAttributes {
    my $self = shift;
    my ($Shop, $ProductFolder, $Product, $hValues) = @_;

    my @Names = @{$hValues->{'Name'}};
    foreach my $hName (@Names) {
        $Product->set({'Name' => $hName->{'Name'}}, $hName->{'LanguageID'});
        if(defined $hName->{'Name'} and $hName->{'Name'} ne ''){
            my $ShortUrl = GetShortUrlByClass(
                $ProductFolder,
                $Shop->get('ProductClass'),
                $hName->{'Name'},
                $hName->{'LanguageID'}
            );
            $Product->set({'URI' => $ShortUrl}, $hName->{'LanguageID'});
        }
    }
}

#========================================================================================
# function JSONSaveNewFile
# state public
#----------------------------------------------------------------------------------------
# syntax $Handler->JSONSaveNewFile($Servlet);
#----------------------------------------------------------------------------------------
# description Called for product image slideshow images upon image upload.
#----------------------------------------------------------------------------------------
# input $Servlet | servlet object | object
#========================================================================================
sub JSONSaveNewFile {
    my $self = shift;
    my ($Servlet) = @_;

    my $Product = $Servlet->object;
    my $hPublicFiles = $Product->get('PublicFiles');
    my @Sizes = keys %{PRODUCT_IMAGES_SIZES()};
    my $RegSizes = join '|', @Sizes;
    my $ImagesCount = 0;

    foreach my $hFile (@$hPublicFiles){
        my (undef, $FName, $Extension) = fsplit($hFile->{'FileName'});
        if ($FName =~ /_($RegSizes)$/) {
            $FName =~ s/_($RegSizes)$//s;
            my ($hParent) = grep { $_->{'FileName'} eq $FName.$Extension } @$hPublicFiles;
            if (!$hParent) {
                $ImagesCount++;
            }
        }
        else{
            $ImagesCount++;
        }
    }
    if($ImagesCount >= PRODUCT_IMAGE_MAX_COUNT){
        $Servlet->form->executeFormError({
            'Reason'=>  'PRODUCT_IMAGE_MAX_COUNT',
            'Form'  => 'SaveFiles',
            'Name'  => 'NewFile',
        });
    }

    my $NewFileName = $Servlet->form->value('NewFile');
    my (undef, $FName, $Extension) = fsplit($NewFileName);
    if ($Extension && ElementOf(substr(lc($Extension),1), IMAGE_FILE_TYPES())) {
        # it's an image
        $self->scaleImages($Servlet);
        # EPG-11636 : enable for product slideshow per default

        my $SlideShowString = $Product->get('ImagesSlideShowString');
        my $NewFileRel = Unicode2Filename($FName.$Extension);
        if(!$SlideShowString) {
            my $DefaultImage = $Product->get('ImageLarge');
            $SlideShowString = defined $DefaultImage ? $DefaultImage . ';' . $NewFileRel : $NewFileRel;
            $Product->set( { 'ImagesSlideShowString' => $SlideShowString } );
        } elsif($SlideShowString !~ /$NewFileRel/) {
            $SlideShowString .= ';'.$NewFileRel;
            $Product->set( { 'ImagesSlideShowString' => $SlideShowString } );
        }
        $self->writeContentOutput({
            'uploadedFile' => $NewFileRel,
        });
        #invalidate pagecache if product is visible
        $Product->getSite()->invalidatePageCache if $Product->get('IsVisible');
    } else {
        #only images allowed!
        $Servlet->form->executeFormError({
            'Reason'=>  'UNSUPPORTED_IMAGE_FORMAT',
            'Form'  => 'SaveFiles',
            'Name'  => 'NewFile',
        });
    }
    return;
}

sub JSONCreateAppointmentFolder {
    my $self = shift;
    my ($Servlet) = @_;
    my $AppointmentFolder = $Servlet->object->folder('Appointments', {'Position'=>'170', 'Class'=>LoadClassByAlias('AppointmentFolder') });
    return $self->writeContentOutput({
        'AppointmentFolderID'=> $AppointmentFolder->id,
    });
}

# Return the description attribute of each supplied ObjectID
sub JSONGetProductDescriptions {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object->getSite;
    my @ObjectIDList = split /,/, $Servlet->params('ObjectIDList');
    my $hResults;

    foreach my $ObjectID (@ObjectIDList) {
        my $Product = $Shop->loadObject($ObjectID, 'Product');
        my $hAttributes = $Product->get(['Description', 'SuperProduct' ]);
        if(!$hAttributes->{'Description'} && $hAttributes->{'SuperProduct'}) { # is a subproduct and has no own description
            $hAttributes->{'Description'} = ($hAttributes->{'SuperProduct'})->get('Description');
        }
        $hResults->{$ObjectID} = $hAttributes->{'Description'};
    }

    return $self->writeContentOutput($hResults);
}


sub scaleImages {
    my $self = shift;
    my ($Servlet) = @_;

    my $Product = $Servlet->object;
    my $LargeFile = $self->saveFile($Servlet, $Product, 'NewFile');

    my $ShopPath = $Product->get('Site')->get('PublicPath');
    my $ProductPath = $Product->get('FileAlias');

    $Servlet->form->executeFormError;

    my $LargeFileFullPath = $Product->get('PublicPath').'/'.$LargeFile;

    my $hImageFiles;
    eval {
        # test if it's a valid image
        TestImage($LargeFileFullPath);

        # get the constants from the template for scaling images
        my $hParams = $Servlet->params();
        $hImageFiles = ScaleImageSizes($LargeFileFullPath, $hParams);
    };
    if (ExistsError()) {
        my $Error = GetError();

        Error($Error) unless $Error->code ~~ [qw(IMAGE_MAGICK UNSUPPORTED_IMAGE_FORMAT INVALID_IMAGE_FILE IMAGE_EXCEEDS_MEGAPIXELS)];
        DeleteFile($LargeFileFullPath);
        CleanPath($ShopPath, $ProductPath);
        $Servlet->form->executeFormError({
            'Reason'=> $Error->code,
            'Form'  => 'SaveFiles',
            'Name'  => 'NewFile',
        });

    }
    return $hImageFiles;
}

sub _createResourceObject {
    my $self = shift;
    my ($Servlet, $ProductFolder, $hValues) = @_;

    my $Shop = $ProductFolder->getSite();
    $hValues->{'Parent'} = $ProductFolder;
    my $ListPrice = $hValues->{'ListPrice'};
    delete $hValues->{'ListPrice'};

    my $Class = $Shop->get('ProductClass');
    my $Product;
    eval {
        $Product = $Class->insertObject($hValues);
        $Product->setListPrice( $Shop->get('CurrencyID'), $Shop->get('InputTaxModel'), $ListPrice ) if defined $ListPrice;
    };
    if (ExistsError()) {
        my $Error = GetError();
        $Servlet->form->executeFormError({
            'Reason'=> 'FeatureMaxValueReached',
            'Form'  => 'NewProduct',
            'Name'  => 'NewName',
        }) if $Error->code eq 'FeatureLimitExceeded';
        Error($Error);
    }

    # activate mailtypes (EPG-11524)
    my $CountAppointments = scalar @{$ProductFolder->findChildren};
    if ($CountAppointments == 1) {
        foreach my $MailTypeAlias (qw (AppointmentCreated AppointmentAccepted AppointmentCancelled)) {
            my $MailType = GetShopMailType( $Shop, $MailTypeAlias );
            unless ($MailTypeAlias eq 'AppointmentAccepted') {
                my $BCC = $MailType->get('BCC');
                $MailType->set({ 'BCC' =>  $Shop->get('MailFrom') }) unless defined $BCC;
            }
            $MailType->set({'IsActive' => 1});
        }
    }
    return $Product;
}

#========================================================================================
# function JSONSaveMBOUI
# state public
#----------------------------------------------------------------------------------------
# syntax $Handler->JSONSaveMBOUI($Servlet);
#----------------------------------------------------------------------------------------
# description Called for product detail page for saving each section's position and visibility.
#----------------------------------------------------------------------------------------
# input $Servlet | servlet object | object
#========================================================================================
sub JSONSaveMBOUI {
    my $self = shift;
    my ($Servlet) = @_;

    my $User = $Servlet->object;
    my $hValues = $Servlet->form->form($User, 'SaveMBOUI');
    my $AttributeName = $hValues->{'AttributeName'};
    my $Key           = $hValues->{'Key'};

    Error('UNKNOWN_ATTRIBUTE_NAME', { 'Name' => $AttributeName })
        if $AttributeName !~ /MBOUI_/;

    my $aItems = $User->get($AttributeName) // MBOUI_PRODUCT_DETAIL_PAGE;
    my ($hSelectedItem) = grep { $hValues->{'SectionAlias'} eq $_->{'alias'} } @$aItems;
    return unless defined $hSelectedItem;

    $hSelectedItem->{$Key} = $hValues->{'Value'};

    # Reposition items and order them by position.
    if( $Key eq 'Position' ){
        my $TmpPos = my $NewPos = $hSelectedItem->{'Position'};
        my $PosScope = $NewPos >= $#$aItems;
        foreach my $hItem (@$aItems){
            if( $hItem->{'alias'} ne $hSelectedItem->{'alias'} && $hItem->{'Position'} == $TmpPos ){
                $hItem->{'Position'} = $PosScope ? --$TmpPos : ++$TmpPos;
            }
        }
        @$aItems = sort { $a->{'Position'} <=> $b->{'Position'} } @$aItems;
    }

    $User->set({ $AttributeName => $aItems });
}

#========================================================================================
# function JSONSetProductImageForSameVariations
# state public
#----------------------------------------------------------------------------------------
# syntax $Handler->JSONSetProductImageForSameVariations($Servlet);
#----------------------------------------------------------------------------------------
# description Set the product images for variations with the same attribute to the main
#              product images.
#----------------------------------------------------------------------------------------
# input $Servlet | servlet object | object
#========================================================================================
sub JSONSetProductImageForSameVariations {
    my $self = shift;
    my ($Servlet) = @_;
    my $Product = $Servlet->object;

    my $AttributeName = $Servlet->form->value('Attribute');
    my $ValueID = $Servlet->form->value('ValueID');

    my %ImageFiles = map {$_ => undef} values %{PRODUCT_IMAGE_SIZE_NAMES()};

    foreach my $SubProduct (@{$Product->subProducts}) {
        if($SubProduct->get($AttributeName)->id == $ValueID) {
            $SubProduct->set(\%ImageFiles);
        }
    }

    # EPG-10737 upload for all views
    $Product->getSite()->invalidatePageCache;

    # trigger refresh of SEO check
    PerformSEOCheckAndUpdateObject($Product);

    return;
}

#========================================================================================
# function JSONUploadImageForSameVariations
# state public
#----------------------------------------------------------------------------------------
# syntax $Handler->JSONUploadImageForSameVariations($Servlet);
#----------------------------------------------------------------------------------------
# description Called to upload an image, generate all product images and set them for
#              all variations with the same attribute.
#----------------------------------------------------------------------------------------
# input $Servlet | servlet object | object
#========================================================================================
sub JSONUploadImageForSameVariations {
    my $self = shift;
    my ($Servlet) = @_;
    my $hImageFiles = $self->scaleImages($Servlet);
    my $Product = $Servlet->object;

    my $AttributeName = $Servlet->form->value('Attribute');
    my $ValueID = $Servlet->form->value('ValueID');

    foreach my $SubProduct (@{$Product->subProducts}) {
        $SubProduct->set($hImageFiles) if($SubProduct->get($AttributeName)->id == $ValueID);
    }

    # Fix: If image already exists "ScaleImageSizes" does not return the filenames.
    my $hServletResponseContent = $Servlet->vars('ResponseContent');
    if(not defined $hImageFiles->{'ImageSmall'}) {
        my $SmallImagePath = $hServletResponseContent->{'uploadedFile'};
        my (undef, $FileName, $Extension) = fsplit($SmallImagePath);
        $hImageFiles->{'ImageSmall'} = $FileName.'_s'.$Extension;
    }

    $hServletResponseContent->{'uploadedFile'} = $Product->get('WebPath').'/'.$hImageFiles->{'ImageSmall'};
    $Servlet->vars('ResponseContent', $hServletResponseContent);

    # EPG-10737 upload for all views
    $Product->getSite()->invalidatePageCache;

    # trigger refresh of SEO check
    PerformSEOCheckAndUpdateObject($Product);

    return;
}


#========================================================================================
# function JSONHandleProductImagesUnity
# state public
#----------------------------------------------------------------------------------------
# syntax $Handler->JSONHandleProductImagesUnity($Servlet);
#----------------------------------------------------------------------------------------
# description Called to upload, delete, rearange an image for a product.
#----------------------------------------------------------------------------------------
# input $Servlet | servlet object | object
#========================================================================================
sub JSONHandleProductImagesUnity {
    my $self = shift;
    my ($Servlet) = @_;

    my $hImagesData = $Servlet->form->value('data');
    my $Operation   = $Servlet->form->value('Operation');

    # for sub product uploads we must not set the slideshowstring
    my $NoSlideShow = $Servlet->form->value('NoSlideShow') // 0;
    my $NoDeleteFromStorage = $Servlet->form->value('NoDeleteFromStorage') // 0;

    # get data for operations: delete and sorting later
    # (operation : save has an own extraction handling for image date)

    my $ahResults;
    my $hUploadedImages;

    #switch through operations
    given ($Operation) {

        # operation: save: we have only image data without order informations. we add the new image(s) to the end
        # of the slideshow
        when ('save') {
            # save product slideshow
            ($ahResults,$hUploadedImages) = _SaveProductImagesUnity($Servlet, $hImagesData, $NoSlideShow);
        }

        # operation: delete: use $aImageDelete for deleting
        when ('delete') {
            $ahResults = _DeleteProductImagesUnity($Servlet, $hImagesData) unless $NoDeleteFromStorage;
        }

        when ('order') {
            #do nothing here because we order images later see: _SortSlideShowImages
        }
        default {
            die "no valid operation '$Operation'\n";
        }
    }

    # sort images by setting the ImagesSlideShowString and image attributes for the product
    _SortSlideShowImages($Servlet->object(), $hImagesData, $NoSlideShow, $ahResults) unless $Operation eq 'save';

    # collect output and return
    my $hJsonOutput = {
        'error' => ((defined $ahResults) and scalar(@$ahResults)) ? JSON::true : JSON::false,
        'failed' => $ahResults,
        'imgData' => $hUploadedImages, # real image names (yes we change filenames if we have special chars in it)
    };

    return $self->writeContentOutput($hJsonOutput);

}

sub JSONSaveCropData {
    my $self = shift;
    my ($Servlet) = @_;

    my $ahCropDataFromJSON = from_json($Servlet->form->value('cropData'), { utf8  => 1 });
    my $hJsonOutput        = {'success' => JSON::true};
    my $Product            = $Servlet->object();

    eval {
        CropProductImages($Product, $ahCropDataFromJSON, {'IsVariation' => 0});
    };

    if (ExistsError()) {
        my $Error = GetError();
        $hJsonOutput->{'success'} = JSON::false;
        $hJsonOutput->{'error'}   = $Error->code;
    }

    # return output
    return $self->writeContentOutput($hJsonOutput);

}

#========================================================================================
# function     _SaveProductImagesUnity
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ahResults = _SaveProductImagesUnity($Servlet, $hImagesData, $NoSlideShow);
#----------------------------------------------------------------------------------------
# description  Save uploaded images, scale images to different sized and add new images
#               to ImagesSlideShowString
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet object | object
# input        $hImagesData | image data to save | hash
# input        $NoSlideShow (optional) | rewrite the slideshow string for the given product | boolean
# return       $ahResults | save operation results | array of hashes
# return       $hUploadedImages | uploaded images with real file names | hash
#========================================================================================
sub _SaveProductImagesUnity {
    my ($Servlet, $hImagesData, $NoSlideShow) = @_;

    my $Product    = $Servlet->object;
    my $Shop       = $Product->getSite;
    my $PublicPath = $Product->get('PublicPath');

    my $hUploadedImages;
    my $ahResults;
    my @SlideShowImages;

    # extract data from JSON
    my $ahImages = _ExtractSaveImagesDataFromJSON($hImagesData);

    foreach my $Image (@$ahImages) {

        my ($ImageID, $hImageData) = each(%$Image);
        # do we have an image to save?
        next unless exists $hImageData->{'dataUrl'};

        # extract image content from base64 encoded string (dataUrl)
        my (undef, $FileName, $Extension) = fsplit($hImageData->{'name'});
        my $RealFileName = Unicode2Filename($FileName) . $Extension;
        my $FilePath = $PublicPath . '/' . $RealFileName ;

        my $SlideShowStringInDb = $Product->get('ImagesSlideShowString');
        my $ImageMediumInDb = $Product->get('ImageMedium');
        my $ImageName= $hImageData->{'name'};

        # remove file extension
        $ImageName=~ s{\.[^.]+$}{};

        # error if file exists on disk and if file reference exists on product
        if (ExistsFile($FilePath) && (index($SlideShowStringInDb, $ImageName) != -1 or index($ImageMediumInDb, $ImageName) != -1 )) {
            push @$ahResults, {
                'id'     => $ImageID,
                'source' => $hImageData->{'name'},
                'reason' => 'FILE_EXISTS_ERROR'
            };
            next;
        }

        # decode...
        $hImageData->{'dataUrl'} =~ s/^(?:.*?base64,)(.*)$/$1/;
        $hImageData->{'dataUrl'} = decode_base64($hImageData->{'dataUrl'});

        # write file to storage
        eval {
            WriteFile($FilePath, \$hImageData->{'dataUrl'}, {'binary' => 1, 'create_directory' => 1});

        };
        if (ExistsError()) {
            push @$ahResults, {
                'id'     => $ImageID,
                'source' => $hImageData->{'name'},
                'reason' => 'FILE_WRITE_ERROR'
            };
        }
        else {

            # save here the real filename because we convert scecial characters
            # so we can inform the UI about the real used filenames in storage
            $hUploadedImages->{$ImageID} = {'name' => $RealFileName };

            # try to scale images
            my $hScaledImages;
            eval {
                $hScaledImages = GetScaledImages($FilePath, $Shop);
            };
            if (ExistsError() or not defined $hScaledImages) {
                push @$ahResults, {
                    'id'     => $ImageID,
                    'source' => $hImageData->{'name'},
                    'reason' => 'FILE_SCALE_ERROR'
                };

                # delete file on error
                eval { DeleteFile($FilePath); };
                if (ExistsError()) {
                    Warning("Cannot delete file $FilePath");
                }
            }
            else {
                $Product->set($hScaledImages) unless $Product->get('ImageLarge');
                push @SlideShowImages, $hScaledImages->{'ImageLarge'};
            }
        }
    }

    # there is no need to rewrite the slideshow string
    # we use this if we upload images to the server
    # and build the slideshow afterwards (subproducts)
    return ($ahResults,$hUploadedImages) if $NoSlideShow;

    # set the new slide show string if needed
    my $ImagesSlideShowString = $Product->get('ImagesSlideShowString');
    unshift @SlideShowImages, $ImagesSlideShowString if $ImagesSlideShowString ne "";

    $ImagesSlideShowString = scalar(@SlideShowImages) ? join(';', @SlideShowImages) : undef;
    $Product->set({'ImagesSlideShowString' => $ImagesSlideShowString});

    return ($ahResults,$hUploadedImages);
}

#========================================================================================
# function     _ExtractSaveImagesDataFromJSON
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ahImages = _ExtractSaveImagesDataFromJSON($hImagesData);
#----------------------------------------------------------------------------------------
# description  extracts image relevant data from JSON string
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet object | object
# input        $hImagesData | JSON string | hash with JSON string
# return       $ahImages | sorted images for saving | array of hashes
#========================================================================================
sub _ExtractSaveImagesDataFromJSON {
    my ($hImagesData) = @_;

    # try to extract images from jsonstring because from_json is really slow here if we have large images
    # do this only for new images or sorting

    # store base64 encoded image data here
    my $aB64ImagesString = [];

    # find loop elements in image data and store also other elements
    $hImagesData =~ /({"imgData":\[.*?)(.*)(\],.*)/;
    my $Prefix = $1;
    my $Postfix = $3;
    my $ImgData = $2;

    # extract images and add these to $aB64ImagesString
    # other matches are also stored for parsing by from_json
    my $ToParse = "";
    while($ImgData =~ /({".*?")(data:.*?)("})/g){
        push @$aB64ImagesString,$2; # we store base 64 encoded image here
        $ToParse .= "$1$3,"; # all other elements in string, now we can do from_json really fast
    }
    ($ImgData, $hImagesData) = undef; # free memory
    chop($ToParse); # remove , at the end

    # ImageData without image informations (faster if parsed by from_json)

    $hImagesData  = $Prefix. $ToParse .$Postfix;
    my $JSONData     = from_json($hImagesData, { utf8  => 1 });

    # readd image data to hImageData
    my $idx = 0;
    foreach my $hImageData (@{$JSONData->{'imgData'}}){
        $hImageData->{'dataUrl'} = $aB64ImagesString->[$idx];
        $aB64ImagesString->[$idx] = undef;
        $idx ++;
    }

    #rearange image array for having data in a correctly sorted array
    my $ahImages;
    my $aOrderData   = $JSONData->{'order'};
    foreach my $ImageID (@$aOrderData) {
        foreach my $hImageData (@{$JSONData->{'imgData'}}) {
            push @$ahImages, {$ImageID => $hImageData} if $hImageData->{'id'} eq $ImageID;
        }
    }

    return $ahImages;
}

#========================================================================================
# function     _DeleteProductImagesUnity
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ahResults = _DeleteProductImagesUnity($Servlet, $hImagesData);
#----------------------------------------------------------------------------------------
# description  deletes an already uploaded image from the storage
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet object | object
# input        $hImagesData | JSON image data | hash
# return       $ahResults | delete operation results | array of hashes
#========================================================================================
sub _DeleteProductImagesUnity {
    my ($Servlet, $hImagesData) = @_;

    my $JSONData     = from_json($hImagesData, { utf8  => 1 });
    my $aImageDelete = $JSONData->{'imgDelete'};

    my $Product = $Servlet->object;
    my $ahResults;

    # delete image and all scaled childs
    my $hImageToDelete = $aImageDelete->[0];
    $hImageToDelete->{'src'} = $hImageToDelete->{'src'} // $hImageToDelete->{'name'};

    my (undef, $Name, $Extension) = fsplit($hImageToDelete->{'src'});
    my $FileName = Unicode2Filename($Name) . $Extension;

    eval { DeleteProductImage($Product, $FileName); };

    if (ExistsError()) {
        push @$ahResults, {
            'id'     => $hImageToDelete->{'id'},
            'source' => $FileName,
            'reason' => 'FILE_DELETE_ERROR'
        };
        Warning("Cannot delete file $FileName");
    }

    return $ahResults;
}

#========================================================================================
# function     _SortSlideShowImages
# state        private
#----------------------------------------------------------------------------------------
# syntax       _SortSlideShowImages($Product, $hImagesData, $NoSlideShow, $ahResults)
#----------------------------------------------------------------------------------------
# description  sort the existing ImagesSlideShowString by given image data
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet object | object
# input        $hImagesData | JSON data hash | hash
# input        $NoSlideShow | do not change the slideshow but se correct image attributes | boolean
# input        $ahResults | error results | array of hashes
#========================================================================================
sub _SortSlideShowImages {
    my ($Product, $hImagesData, $NoSlideShow, $ahResults) = @_;


    my $ProductPublicPath = $Product->get('PublicPath');

    my $JSONData     = from_json($hImagesData, { utf8  => 1 });
    my $aOrderData   = $JSONData->{'order'};

    # save the 1st image for writing product image attributes later
    my $FirstImageName = undef;

    # rewrite slideshowstring (use sort information for setting new string)
    my @SlideShowImages;

    # images are ordered by the given $ahImages list. We will write the imagesslideshowstring in the correct way
    # and set the new imageattributes if we have a new main image

    my $ahImages;
    foreach my $ImageID (@$aOrderData) {
        foreach my $hImageData (@{$JSONData->{'imgData'}}) {
            push @$ahImages, {$ImageID => $hImageData} if $hImageData->{'id'} eq $ImageID;
        }
    }

    # create the slideshow string hash
    foreach my $Image (@$ahImages) {
        my %Deref = %$Image;
        my ($ImageID, $hImageData) = each(%Deref);
        my $ImageName = $hImageData->{'src'} // $hImageData->{'name'};
        my (undef, $Name, $Extension) = fsplit($ImageName);
        my $FileName = Unicode2Filename($Name) . $Extension;

        # test if the files exists we sort
        if (ExistsFile($ProductPublicPath . '/' . $FileName)) {
            push @SlideShowImages, $FileName;
            $FirstImageName = $FileName unless defined $FirstImageName;
        }
        else {
            push @$ahResults, {
                'id'     => $ImageID,
                'source' => $FileName,
                'reason' => 'FILE_SORT_ERROR'
            };
        }
    }

    my $ImagesSlideShowString = scalar(@SlideShowImages) ? join(';', @SlideShowImages) : undef;
    # set the slideshow string if allowed
    unless($NoSlideShow){
        $Product->set({'ImagesSlideShowString' => $ImagesSlideShowString});
    }

    # add the correct images to the attributes
    _SetImageAttributes($Product,$FirstImageName,$ahImages);

    # test if we have the correct image (1st slideshow image) as product image on sub products
    foreach my $SubProduct (@{$Product->get('SubProducts')}){
        my $aSlideShow = $SubProduct->get('ImagesSlideShow');
        # we can  not assign an image because we don't have an image slideshow
        next unless scalar($aSlideShow);

        # set the 1st slideshow image as ImageLarge if not exists
        _SetImageAttributes($SubProduct,$aSlideShow->[0]->{'ImageName'},$ahImages) ;
    }

    return $ahResults;
}

sub _SetImageAttributes {
    my ($Product,$FirstImageName,$ahImages) = @_;

    my $ProductPublicPath = $Product->get('PublicPath');

    # are images to sort
    $ahImages = [] unless defined $ahImages;
    my $AreImagesToSort = scalar(@$ahImages) ? 1 : 0;

    # set product image attributes for the 1st product in list
    my $ExistingImageLarge = $Product->get('ImageLarge');

    # build attributes to set
    my $hScaledImages = {};
    if ((not defined $ExistingImageLarge) or ($ExistingImageLarge ne $FirstImageName)) {
        if ($AreImagesToSort and defined $FirstImageName) {
            my (undef, $FiName, $FiExtension) = fsplit($FirstImageName);
            foreach my $SizeExt (keys(%{PRODUCT_IMAGE_SIZE_NAMES()})) {
                next if PRODUCT_IMAGE_SIZE_NAMES->{$SizeExt} eq 'ImageLarge';
                my $SetImageName = $FiName . '_' . $SizeExt . $FiExtension;
                $SetImageName = ExistsFile($ProductPublicPath . '/' . Unicode2Filename($SetImageName)) ? $SetImageName : undef;
                $hScaledImages->{PRODUCT_IMAGE_SIZE_NAMES->{$SizeExt}} = $SetImageName;
            }
        } else {
            foreach my $SizeExt (keys(%{PRODUCT_IMAGE_SIZE_NAMES()})) {
                $hScaledImages->{PRODUCT_IMAGE_SIZE_NAMES->{$SizeExt}} = undef;
            }
        }
        $hScaledImages->{'ImageLarge'} = $FirstImageName;
        $Product->set($hScaledImages);
    }
}

#========================================================================================
# function     RequestPrettyUnityUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->RequestPrettyUnityUrl($Servlet);
#----------------------------------------------------------------------------------------
# description  Performs a HEAD request to the Unity storefront in order to get the
#               pretty URL of the given object out of its PrettyUnitUrl. In case it is
#               new or has changed, this value is written to the database to have faster
#               access to it next time. It is used to show the SEO preview at the
#               product detail page.
#----------------------------------------------------------------------------------------
# input $Servlet | servlet object | object
#========================================================================================
sub RequestPrettyUnityUrl {
    my $self = shift;
    my ($Servlet) = @_;

    my $Object = $Servlet->object;
    my $Shop   = $Object->getSite();
    my $ShopLanguageID = $Shop->get('LanguageID');

    my $Form           = $Servlet->form;
    my $AltLanguageID  = $Form->value('AltLanguageID');

    my $RelyOnStoredPrettyUrl = 0;
    my $PrettyUnityUrl = GetUnityPrettyUrl($Object, $ShopLanguageID, $AltLanguageID, $RelyOnStoredPrettyUrl);
    my $hResponse = {
        'prettyUnityUrl' => $PrettyUnityUrl
    };

    return $self->writeContentOutput($hResponse);
}

#========================================================================================
# function     JSONVimeoThumbnails
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->JSONVimeoThumbnails($Servlet);
#----------------------------------------------------------------------------------------
# description  Gets information from the Vimeo API about a vimeo video like thumbnails etc.
#----------------------------------------------------------------------------------------
# input $Servlet | servlet object | object
#========================================================================================
sub JSONVimeoThumbnails {
    my $self = shift;
    my ($Servlet) = @_;

    my $VimeoID = $Servlet->params('VimeoID');

    my $hResponse = {'success' => JSON::false};

    unless ($VimeoID) {
        $hResponse->{'error'} = 'NO_VIMEO_ID';
        return $self->writeContentOutput($hResponse);
    }

    my $VimeoAPIURL = VIMEO_API_URL;
    $VimeoAPIURL =~ s/\[videoid\]/$VimeoID/gi; 

    my $UserAgent = LWP::UserAgent->new();
    my $APIResponse = $UserAgent->get($VimeoAPIURL);

    if ($APIResponse->is_success){
        $hResponse->{'success'} = JSON::true;
        $hResponse->{'data'} = from_json($APIResponse->decoded_content, {utf8 => 1});
    } else {
         if($APIResponse->code eq HTTP_NOT_FOUND){
            $hResponse->{'error'} = 'INVALID_VIMEO_ID';
        } else{
            $hResponse->{'error'} = $APIResponse->decoded_content;
            $hResponse->{'error_rc'} = $APIResponse->code;
            Warning("Error connecting Vimeo API. RC:" . $APIResponse->code . " Content: " . $APIResponse->decoded_content );
        }
    }

    return $self->writeContentOutput($hResponse);
}

1;
