#========================================================================================
# package      DE_EPAGES::Shipping::API::Object::ShippingMethod
# state        public
# base         DE_EPAGES::Shop::API::Object::ShopObject
# description  object interface for ShippingMethod.
#========================================================================================
package DE_EPAGES::Shipping::API::Object::ShippingMethod;
use base DE_EPAGES::Shop::API::Object::ShopObject;

use strict;

use DE_EPAGES::Core::API::Error qw ( GetError Error ExistsError );
use DE_EPAGES::Core::API::TestParameter qw ( TestInteger TestString TestArray TestHash );
use DE_EPAGES::Object::API::Factory qw (LoadClassByAlias);
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use DE_EPAGES::Shipping::API::Table::ShippingMethodCustomerGroup qw ( DeleteShippingMethodCustomerGroup GetPKeyShippingMethodCustomerGroupsByShippingMethodID );
##no critic (RequireCorrectImports)
use DE_EPAGES::Shipping::API::XML::ShippingDeleteHandler;
use DE_EPAGES::Shipping::API::XML::ShippingExportDriver;
use DE_EPAGES::Shipping::API::XML::ShippingImportHandler;
##use critic

#========================================================================================
# function     featureName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $FeatureName = $ShippingMethod->featureName;
#----------------------------------------------------------------------------------------
# description  Returns the feature name.
#----------------------------------------------------------------------------------------
# return       $FeatureName | feature name (undef means no feature count) | String
#========================================================================================
sub featureName () { return undef; }

#========================================================================================
# function     className
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $ShippingMethod->className;
#----------------------------------------------------------------------------------------
# description  Returns the class name, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | String
#========================================================================================
sub className  { 'ShippingMethod' }

#========================================================================================
# function     dbPackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DbPackage = $ShippingMethod->dbPackage;
#----------------------------------------------------------------------------------------
# description  Returns the database package, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $DbPackage | database package name | String
#========================================================================================
sub dbPackage { __PACKAGE__ }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ShippingMethod = DE_EPAGES::Shipping::API::Object::ShippingMethod->insert($hInfo);
# example      my $Class = LoadClassByAlias('ShippingMethod');
#               my $ShippingMethod = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook 'OBJ_InsertShippingMethod' is triggered.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
# return       $ShippingMethod | new object | object
# hook         OBJ_InsertShippingMethod | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShippingMethodID - shipping method id   - int
#               </ul>
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;

    my $Shop = $hInfo->{'Shop'} || $hInfo->{'Parent'}->getSite();

    # test special shipping feature
    my $FeatureName = $class->featureName;
    Error( 'FeatureLimitExceeded', {
        Feature => $FeatureName,
        MaxValue => 0,
        Shop => $Shop
    }) if defined $FeatureName and not $Shop->featureMaxValue($FeatureName);


    my $DoSwitch = 0;
    if ($Shop->featureMaxValue('ShippingMethods') == 1) {
        # switch shipping
        my $ShopShippingMethod = $Shop->get('ShippingMethods')->[0];
        if (defined $ShopShippingMethod) {
            $ShopShippingMethod->delete();
            $DoSwitch = 1;
        }
    }

    $Shop->featureIncrementValue('ShippingMethods');
    my $self = $class->SUPER::insert($hInfo);

    if ($DoSwitch) {
        # activate shipping method and set it default
        $self->set({ 'IsDefault' => 1 });
    }

    TriggerHook('OBJ_InsertShippingMethod', {'Object' => $self, 'ShippingMethodID' => $self->id });
    delete $hInfo->{$_} foreach qw ( Shop ShippingType TaxClass IsActivated IsDefault ContainerSubTotalAttrName Region EnabledByDefault ); ## no critic qw(ProhibitLongLines)
    return $self;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ShippingMethod->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeleteShippingMethod' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeleteShippingMethod | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShippingMethodID - shipping method id   - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    TriggerHook('OBJ_DeleteShippingMethod', {'Object' => $self, 'ShippingMethodID' => $self->id });

    LoadClassByAlias('ShippingLevel')->deleteObjects('ByShippingMethodID', [$self->id]);
    DeleteShippingMethodCustomerGroup($_) foreach @{GetPKeyShippingMethodCustomerGroupsByShippingMethodID($self->id)};
    LoadClassByAlias('ShippingValues')->deleteObjects('ByShippingMethodID', [$self->id]);
    $self->SUPER::delete;

    #--- update feature value
    my $Shop = $self->getSite();
    $Shop->featureIncrementValue('ShippingMethods', -1);

    if( $Shop->featureMaxValue('ShippingMethodsPerProduct') ) {
        # go through all products and look if 'ShippingMethods' attribute could be set undef (indicates that all
        # shippings methods are enabled for a product)
        my $aShopShippingMethods = $Shop->get('ShippingMethods');
        foreach my $Product (@{$Shop->get('Products')}) {
            my $aProductShippingMethods = $Product->get('ShippingMethods');
            #if all shipping methods are enabled then this attribute should be undef
            if (scalar(@$aShopShippingMethods) == scalar(@$aProductShippingMethods)) {
                $Product->set({'ShippingMethods' => undef});
            }
        }
    }

    return;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ShippingMethod->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdateShippingMethod' and 'OBJ_AfterUpdateShippingMethod'.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
# input        $LanguageID | (optional; required for localized attributes) language id | int
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdateShippingMethod | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShippingMethodID - shipping method id   - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdateShippingMethod | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShippingMethodID - shipping method id   - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    TriggerHook('OBJ_BeforeUpdateShippingMethod', {'Object'=>$self, 'Values'=> $hValues, 'ShippingMethodID' => $self->id, 'LanguageID' => $LanguageID});
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdateShippingMethod',  {'Object'=>$self, 'Values'=> $hValues, 'ShippingMethodID' => $self->id, 'LanguageID' => $LanguageID});
    return;
}

#========================================================================================
# function     default
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = DE_EPAGES::Shipping::API::Object::ShippingMethod->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;
    return {
        %{$class->SUPER::default($Parent, $hInfo)},
        'IsActivated' => 0,
        'IsDefault' => 0,
        'Shop' => $Parent->getSite,
        'EnabledByDefault' => 1,
    };
}

sub importXML {
    my ($ObjectPackage, $Handler, $hElement, $Class, $hAttributes) = @_;

    my $Parent = $Handler->topID('Object');

    # get shop
    my $Shop = $Parent->getSite() or die 'No Shop found for object '.$Parent->alias;
    $hAttributes->{'Shop'} = $Shop;

    # get tax class
    if (exists $hAttributes->{'TaxClass'}) {
        my $TaxMatrix = $Shop->get('TaxMatrix');
        $hAttributes->{'TaxClass'} = $TaxMatrix->child($hAttributes->{'TaxClass'});
    }
    my $Object = $ObjectPackage->SUPER::importXML($Handler, $hElement, $Class, $hAttributes);

    # delete levels before import, because there is no unique key
    unless( $Handler->isReference ) {
        my $ShippingLevelClass = LoadClassByAlias('ShippingLevel');
        $ShippingLevelClass->deleteObjects('ByShippingMethodID', [$Object->id]);
    }

    return $Object;
}

#========================================================================================
# function     exportXMLAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hAttributes = $Object->exportXMLAttributes( $Driver, $hAllAttributes );
#----------------------------------------------------------------------------------------
# description  Returns plain attributes of object to XML export driver. This function
#               removes bit attributes with value 0.
#----------------------------------------------------------------------------------------
# input        $Driver | xml export driver with support function addObject | object
# input        $hAllAttributes | all attributes of object,
#                                 see $Object-><function exportableAttributes> | ref.hash
# return       $hAttributes | plain attributes | ref.hash.string
#========================================================================================
sub exportXMLAttributes {
    my $self = shift;
    my ($Driver, $hAttributes) = @_;
    $hAttributes = $self->SUPER::exportXMLAttributes($Driver, $hAttributes);

    # map TaxClass name to TaxClassID
    my $TaxClass = $self->get('TaxClass');
    $hAttributes->{'TaxClass'} = $TaxClass->alias if defined $TaxClass;
    return $hAttributes;
}

sub exportXMLElements {
    my $self = shift;
    my ($Driver, $allAttributes) = @_;
    $self->SUPER::exportXMLElements($Driver, $allAttributes);

    if ($self->class->get('ExportLevel') == $Driver->exportLevel) {
        DE_EPAGES::Object::API::XML::ObjectExportDriver::CustomerGroupsByShippingMethod($Driver);

        my $ShippingLevelClass = LoadClassByAlias('ShippingLevel');
        my $aShippingLevels = $ShippingLevelClass->getObjects('ByShippingMethodID', {'ShippingMethodID'=>$self->id});
        $Driver->addObject( 'ShippingLevel', $_ ) foreach sort {
            $a->get('CurrencyID') cmp $b->get('CurrencyID') || $a->get('LowerBound') <=> $b->get('LowerBound')
        } @$aShippingLevels;

        my $ShippingValuesClass = LoadClassByAlias('ShippingValues');
        my $aShippingValues = $ShippingValuesClass->getObjects('ByShippingMethodID', {'ShippingMethodID'=>$self->id});
        $Driver->addObject( 'ShippingValues', $_ ) foreach sort {
            $a->get('CurrencyID') cmp $b->get('CurrencyID')
        } @$aShippingValues;
    }
}

#========================================================================================
# function     price
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShippingPrice = $Object->price( $hVars );
#----------------------------------------------------------------------------------------
# example      $Product = LoadObjectByPath('/Shops/DemoShop/Products/0815');
#               $ShippingPrice = $ShippingPrice = $ShippingMethod->price({
#                   'CurrencyID' => 'EUR',
#                    'TaxModel'  => 1,
#                    'ShippingElements' => [ { 'Element' =>  $Product,
#                                              'Quantity' => 1,
#                                          } ]
#               });
#----------------------------------------------------------------------------------------
# description  Returns the shipping price by the given requirements. Needs to be
#               implemented for a new shipping method.
#----------------------------------------------------------------------------------------
# input        $hVars | <ul>
#                   <li>CurrencyID    - alpha currency code (ISO 4217) - char(3)
#                   <li>TaxModel      - gross or net price    - int
#                   <li>ShippingElements
#                       <ul>
#                          <li>Element   - e.g. Products       - object
#                          <li>Quantity  - quantity of element - float
#                      </ul> | ref.array.hash
#               </ul> | ref.hash
# return       $ShippingPrice | price of shipping | money
#========================================================================================
sub price {
    my $self = shift;
    my ($hVars) = @_;
    die 'please implement the function price for your shippingmethod ';
}

#========================================================================================
# function     discount
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShippingDiscount = $Object->discount( $hVars );
#----------------------------------------------------------------------------------------
# example      $ShippingDiscount = $ShippingMethod->discount( {
#                   CurrencyID => 'USD',
#                   TaxModel => $ShippingMethod->getSite()->get('TaxModel')
#               } );
#----------------------------------------------------------------------------------------
# description  Returns the shipping discount by the given requirements.
#               The hook 'ShippingDiscount' is triggered before the standard discount = 0.
#----------------------------------------------------------------------------------------
# input        $hVars | <ul>
#                   <li>CurrencyID    - alpha currency code (ISO 4217) - char(3)
#                   <li>TaxModel      - gross or net price    - int
#                   <li>ShippingElements
#                       <ul>
#                          <li>Element   - e.g. Products       - object
#                          <li>Quantity  - quantity of element - float
#                      </ul> | ref.array.hash
#               </ul> | ref.hash
# return       $ShippingDiscount | discount of shipping | money
#========================================================================================
sub discount {
    my $self = shift;
    my ($hVars) = @_;
    my %Vars = (
        %$hVars,
        'Object'            => $self,
        'ShippingMetghodID' => $self->id,
        'Discount'          => undef,
    );
    TriggerHook('ShippingDiscount', \%Vars);
    return $Vars{'Discount'} if defined $Vars{'Discount'};
    return 0;

}

#========================================================================================
# function     canDelete
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object->canDelete($aWarnings);
#----------------------------------------------------------------------------------------
# description  In case of the default shipping method a warning is added. The default
#               shipping method can't be deleted.
#----------------------------------------------------------------------------------------
# input        $aWarnings  | array of Warning Objects | array.ref
# return       $Deleteable | true if shipping method can be deleted | boolean
#========================================================================================
sub canDelete {
    my $self = shift;
    my ($aWarnings) = @_;

    if ($self->get('IsDefault')) {
        my $Warning = DE_EPAGES::Core::API::Warning->new(
            Code => 'CanNotDeleteDefault',
            Vars => {},
        );
        push @$aWarnings, $Warning;
    }

    return $self->SUPER::canDelete($aWarnings);
}
#========================================================================================
# function     usableAtStorefront
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Usable = $ShippingMethod->usableAtStorefront($hVars);
#----------------------------------------------------------------------------------------
# description  Tests if a shipping method with the given currency is visible in the storefront.
#               Uses function <function DE_EPAGES::Shipping::API::Object::ShippingMethod::price>
#               to evaluate if shipping methode has a price (price can be 0).
#----------------------------------------------------------------------------------------
# input        $hVars | <ul>
#                   <li>CurrencyID    - alpha currency code (ISO 4217) - char(3)
#                   <li>TaxModel      - gross or net price    - int
#               </ul> | ref.hash
# return       $Usable | true if shipping method is usable in the storefront | boolean
#========================================================================================
sub usableAtStorefront {
    my $self = shift;
    my ($hVars) = @_;

    return 0 unless $self->get('ShippingType')->get('IsActivated');
    eval { $self->price($hVars); };
    if (ExistsError()) {
        my $Error = GetError();
        return 0 if $Error->code eq 'ShippingMethodNoPrice';
        Error($Error);
    }
    return 1;
}

#========================================================================================
# function     levels
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aShippingLevels = $Object->levels;
#----------------------------------------------------------------------------------------
# description  Returns all shipping level objects of that shipping method.
#----------------------------------------------------------------------------------------
# return       $aShippingLevels | list of shipping level opbjects | ref.array.object
#========================================================================================
sub levels {
    my $self = shift;

    my $Class = LoadClassByAlias('ShippingLevel');
    return [
        sort { $a->get('LowerBound') <=> $b->get('LowerBound') }
        @{$Class->getObjects('ByShippingMethodID', { 'ShippingMethodID' => $self->id })}
    ];
}

#========================================================================================
# function     getLevels
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ahLevels = $Object->getLevels($CurrencyID, $TaxModel);
# syntax       $ahLevels = $Object->getLevels($CurrencyID, $TaxModel, $ExtraPrecision);
# syntax       $ahLevels = $Object->getLevels($CurrencyID, $TaxModel, $hParams);
#----------------------------------------------------------------------------------------
# description  Return the shipping levels for the given currency in the requested tax
#               model.
#----------------------------------------------------------------------------------------
# input        $CurrencyID | currency  id | char(3)
# input        $TaxModel | tax model (1=gross/including tax, 0=net/without tax) | int
# input        $ExtraPrecision | (optional, default=0) extra precision for net prices
#                           in back-office | boolean
#               or $hParams| (optional)= <ul>
# #                   <li>ExtraPrecision (optional, default=0) extra precision for net prices
# #                           in back-office | boolean
# #                   <li>TaxMatrix (optional) - tax matrix object - object
# #               </ul> | ref.hash
# return       $ahLevels | list of hashes, keys:
#                           BaseValue, LowerBound, UpperBound, Multiplier, ShippingLevelID
#                           | int
#========================================================================================
sub getLevels {
    my $self = shift;
    my ($CurrencyID, $TaxModel, $hParams) = @_;
    TestString('CurrencyID', $CurrencyID);
    TestInteger('TaxModel', $TaxModel);

    # make sure that we have a backward compatiblility for callers of this function
    # $ExtraPrecision can be in $hParams or given as parameter
    my $ExtraPrecision = ref( $hParams ) eq 'HASH' ? $hParams->{'ExtraPrecision'} : $hParams;
    $ExtraPrecision = 0 unless defined $ExtraPrecision;

    # fetch TaxMatrix from $hParams if given
    my $TaxMatrix = ref( $hParams ) eq 'HASH' ? $hParams->{'TaxMatrix'} : undef;
    $TaxMatrix = $self->getSite()->get('TaxMatrix') unless defined $TaxMatrix;

    my $ShippingLevelClass = LoadClassByAlias('ShippingLevel');
    my $aLevels = $ShippingLevelClass->getObjects('ByShippingMethodIDCurrencyID', {
        'ShippingMethodID' => $self->id,
        'CurrencyID' => $CurrencyID
    });

    # convert objects to hashes
    my @Levels = sort { $a->{'LowerBound'} <=> $b->{'LowerBound'} }
        map { $_->get([qw(BaseValue LowerBound UpperBound Multiplier ShippingLevelID)]) }
        @$aLevels;

    my $TaxClass = $self->get('TaxClass') ?  $TaxMatrix->child($self->get('TaxClass')->alias) : undef;

    foreach my $hLevel (@Levels) {
        $hLevel->{'BaseValue'} = $TaxMatrix->net2gross($CurrencyID, $hLevel->{'BaseValue'},
                                                      {'TaxModel' => $TaxModel,
                                                       'TaxClass' => $TaxClass,
                                                      },$ExtraPrecision);
    }

    return \@Levels;
}

#========================================================================================
# function     setLevels
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object->setLevels($CurrencyID, $TaxModel, $ahLevels);
#----------------------------------------------------------------------------------------
# description  Sets all shipping levels for the given currency.
#               Deletes shipping levels that are not included in the list.
#               Calculates net prices from gross prices if $TaxModel is TAX_MODEL_GROSS.
#----------------------------------------------------------------------------------------
# input        $CurrencyID | currency  id | char(3)
# input        $TaxModel | tax model (1=gross/including tax, 0=net/without tax) | int
# input        $ahLevels | list of hashes, keys:
#                           BaseValue, LowerBound, UpperBound, Multiplier
#                           | int
#========================================================================================
sub setLevels {
    my $self = shift;
    my ($CurrencyID, $TaxModel, $ahLevels) = @_;
    TestString('CurrencyID', $CurrencyID);
    TestInteger('TaxModel', $TaxModel);
    TestArray('ahLevels', $ahLevels);

    # prepare values, make a local copy
    my %LevelsByLowerBound;
    foreach my $hLevel (@$ahLevels) {
        my $LowerBound = $hLevel->{'LowerBound'};
        $LowerBound = 0 unless defined $LowerBound;
        my @Keys = qw(BaseValue LowerBound UpperBound Multiplier);
        $LevelsByLowerBound{$LowerBound}{$_} = $hLevel->{$_} foreach @Keys;
    }

    # convert net prices to gross
    my $TaxMatrix = $self->getSite()->get('TaxMatrix');
    my $TaxClass = $self->get('TaxClass');
    foreach my $hLevel (values %LevelsByLowerBound) {
        $hLevel->{'BaseValue'} = $TaxMatrix->gross2net($CurrencyID, $hLevel->{'BaseValue'},
                                                      {'TaxModel' => $TaxModel,
                                                       'TaxClass' => $TaxClass,
                                                      });
    }

    # update existing levels, delete obsolete ones
    my $ShippingLevelClass = LoadClassByAlias('ShippingLevel');
    my $aShippingLevels = $ShippingLevelClass->getObjects('ByShippingMethodIDCurrencyID', {
        'ShippingMethodID' => $self->id,
        'CurrencyID' => $CurrencyID,
    });
    foreach my $ShippingLevel (@$aShippingLevels) {
        my $LowerBound = $ShippingLevel->get('LowerBound');
        $LowerBound = 0 unless defined $LowerBound;
        if( exists $LevelsByLowerBound{$LowerBound} ) {
            $ShippingLevel->set( $LevelsByLowerBound{$LowerBound} );
            delete $LevelsByLowerBound{$LowerBound};
        }
        else {
            $ShippingLevel->delete;
        }
    }

    # add new levels
    foreach my $hLevel (values %LevelsByLowerBound) {
        $ShippingLevelClass->insertObject({
            'ShippingMethod' => $self,
            'CurrencyID' => $CurrencyID,
            'BaseValue'  => $hLevel->{'BaseValue'},
            'LowerBound' => $hLevel->{'LowerBound'},
            'UpperBound' => $hLevel->{'UpperBound'},
            'Multiplier' => $hLevel->{'Multiplier'},
        });
    }
}

#========================================================================================
# function     getShippingValues
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hShippingValues = $Object->getShippingValues($CurrencyID, $TaxModel);
# syntax       $hShippingValues = $Object->getShippingValues($CurrencyID, $TaxModel, $ExtraPrecision);
#----------------------------------------------------------------------------------------
# description  Return the shipping values for the given currency in the requested tax
#               model.
#----------------------------------------------------------------------------------------
# input        $CurrencyID | currency  id | char(3)
# input        $TaxModel | tax model (1=gross/including tax, 0=net/without tax) | int
# input        $ExtraPrecision | (optional, default=0) extra precision for net prices
#                           in back-office | boolean
# return       $hShippingValues | hash, keys:
#                           FreeShippingPrice (optional), MaximumWeight (optional)
#                           | hash.ref
#========================================================================================
sub getShippingValues {
    my $self = shift;
    my ($CurrencyID, $TaxModel, $ExtraPrecision) = @_;
    TestString('CurrencyID', $CurrencyID);
    TestInteger('TaxModel', $TaxModel);
    $ExtraPrecision = 0 unless defined $ExtraPrecision;

    my $ShippingValuesClass = LoadClassByAlias('ShippingValues');
    my $aShippingValues = $ShippingValuesClass->infoObjects('ByShippingMethodIDCurrencyID', {
        'ShippingMethodID' => $self->id,
        'CurrencyID' => $CurrencyID,
    });

    return {} if scalar(@$aShippingValues) == 0;

    # there should only be one line
    my $hShippingValues = $aShippingValues->[0];

    # convert net prices to gross
    if (defined($hShippingValues->{'FreeShippingPrice'})) {
        my $TaxMatrix = $self->getSite()->get('TaxMatrix');
        my $TaxClass = $self->get('TaxClass');
        $hShippingValues->{'FreeShippingPrice'} = $TaxMatrix->net2gross($CurrencyID, $hShippingValues->{'FreeShippingPrice'},
                                                      {'TaxModel' => $TaxModel,
                                                       'TaxClass' => $TaxClass,
                                                      },$ExtraPrecision);
    }
    return $hShippingValues;
}

#========================================================================================
# function     setShippingValues
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object->setShippingValues($CurrencyID, $TaxModel, $ahShippingValues);
#----------------------------------------------------------------------------------------
# description  Sets all shipping values for the given currency.
#               Calculates net prices from gross prices if $TaxModel is TAX_MODEL_GROSS.
#----------------------------------------------------------------------------------------
# input        $CurrencyID | currency  id | char(3)
# input        $TaxModel | tax model (1=gross/including tax, 0=net/without tax) | int
# input        $hShippingValues | hash, keys:
#                           FreeShippingPrice, MaximumWeight
#                           | hash.ref
#========================================================================================
sub setShippingValues {
    my $self = shift;
    my ($CurrencyID, $TaxModel, $hValues) = @_;
    TestString('CurrencyID', $CurrencyID);
    TestInteger('TaxModel', $TaxModel);
    TestHash('hValues', $hValues);

    # convert net prices to gross
    if (defined($hValues->{'FreeShippingPrice'})) {
        my $TaxMatrix = $self->getSite()->get('TaxMatrix');
        my $TaxClass = $self->get('TaxClass');
        $hValues->{'FreeShippingPrice'} = $TaxMatrix->gross2net($CurrencyID, $hValues->{'FreeShippingPrice'},
                                                      {'TaxModel' => $TaxModel,
                                                       'TaxClass' => $TaxClass,
                                                      });
    }

    # update existing levels, delete obsolete ones
    my $ShippingValuesClass = LoadClassByAlias('ShippingValues');
    my $ahShippingValues = $ShippingValuesClass->getObjects('ByShippingMethodIDCurrencyID', {
        'ShippingMethodID' => $self->id,
        'CurrencyID' => $CurrencyID,
    });

    if (scalar(@$ahShippingValues) == 0) {
        $ShippingValuesClass->insertObject({
            'ShippingMethod' => $self,
            'CurrencyID' => $CurrencyID,
            'FreeShippingPrice'  => $hValues->{'FreeShippingPrice'},
            'MaximumWeight' => $hValues->{'MaximumWeight'},
        });
    }
    else {
        @$ahShippingValues[0]->set( $hValues );
    }
}

sub addCustomerGroup {
    my $self = shift;
    my ($CustomerGroup ) = @_;

    # security check: shippingmethod and customer should belong to the same shop
    Error('InvalidShop',{ a=>$CustomerGroup->getSite()->id, b=>$self->getSite()->id}) unless $CustomerGroup->getSite()->id == $self->getSite()->id;

    # check if the customergroup is already assigned to the shippingmethod
    my $aCustomerGroups = $self->get('CustomerGroups');
    return if grep { $_->id == $CustomerGroup->id } @$aCustomerGroups;

    # add the customer to the shippingmethod
    push @$aCustomerGroups, $CustomerGroup;

    $self->set({ 'CustomerGroups' => $aCustomerGroups });

    return 1;
}

sub removeCustomerGroup {
    my $self = shift;
    my ($CustomerGroup) = @_;

    # security check: shippingmethod and customer group should belong to the same shop
    Error('InvalidShop',{ a=>$CustomerGroup->getSite()->id, b=>$self->getSite()->id}) unless $CustomerGroup->getSite()->id == $self->getSite()->id;

    # remove the customer group from the list
    my @CustomerGroups = grep { $_->id != $CustomerGroup->id } @{$self->get('CustomerGroups')};

    # save the new list
    $self->set({ 'CustomerGroups' => \@CustomerGroups });

    return 1;
}

#========================================================================================
# function     template
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Name = $Object->template($Name, $ObjectPageType);
#----------------------------------------------------------------------------------------
# description  This function is used to overwrite the given template for the purpose that
#               each shipping method can define there own template (visualisation).
#----------------------------------------------------------------------------------------
# input        $Name | include name | string
# input        $ObjectPageType | object related pagetype | DE_EPAGES::Presentation::API::Object::ObjectPageType
# return       $Name | include name | string
#========================================================================================
sub template {
    my $self = shift;
    my ($Name, $ObjectPageType) = @_;
    return $self->classTemplate($Name, $ObjectPageType);
}

#========================================================================================
# function     hasSettings
# state        public
#----------------------------------------------------------------------------------------
# syntax       $HasSettings = $Object->hasSettings;
#----------------------------------------------------------------------------------------
# description  Returns true if the shipping method has any settings, i.e. if the
#               tab page "Settings" should be displayed.
#               The default implementation always returns true.
#----------------------------------------------------------------------------------------
# return       $HasSettings | true if the shipping method has any settings | boolean
#========================================================================================
sub hasSettings {
    return 1;
}

1;
