#========================================================================================
# package      DE_EPAGES::Shipping::API::TLE::ShippingLevelHandler
# state        public
#========================================================================================
package DE_EPAGES::Shipping::API::TLE::ShippingLevelHandler;

use strict;
use DE_EPAGES::Core::API::Log   qw (GetLog);

#========================================================================================
# function     RegisterHandlerProc
# state        private
#----------------------------------------------------------------------------------------
# syntax       $RegisterHandlerProc( $hParams );
#----------------------------------------------------------------------------------------
# description  this function registers this attribute handle on the processor via hook
#               function on hook TLEProcessorRegistration
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments keys :
#               <ul>
#                   <li>Processor - tle processor - object
#               </ul> | ref.hash
#========================================================================================
sub RegisterHandlerProc {
    my ($Params) = @_;
    GetLog->debug( 'ShippingLevelHandler.RegisterHandlerProc' );

    __PACKAGE__->new()->register( $Params->{'Processor'} );
    return;
}

#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler = new DE_EPAGES::ShippingOption::API::TLE::ShippingLevelHandler;
# example      my $PrefixHandler  = new DE_EPAGES::ShippingOption::API::TLE::ShippingLevelHandler;
#----------------------------------------------------------------------------------------
# description  create a new handler
#----------------------------------------------------------------------------------------
# return       $Handler | blessed object | object
#========================================================================================
sub new { bless {}, shift; }

#========================================================================================
# function     register
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler->register( $Processor );
#----------------------------------------------------------------------------------------
# description  this function register this handle on the processor
#----------------------------------------------------------------------------------------
# input        $Processor | tle processor | object
#========================================================================================
sub register {
    my $self = shift;
    my ($Processor) = @_;
    GetLog->debug( 'ShippingLevelHandler.register' );

    $Processor->registerHandler('FunctionHandler', $self, 'ShippingLevels');
    return;
}

#========================================================================================
# function     ShippingLevels
# state        public
#----------------------------------------------------------------------------------------
# syntax       #ShippingLevels(#ShippingMethod, #Currency, #TaxModel, #ExtraPrecision)
# example      #LOOP(#ShippingLevels(#Object, #INPUT.Currency, #Shop.InputTaxModel, 1))
#                 #BaseValue[money]
#                 #Multiplier[float]
#                 #LowerBound[float]
#                 #UpperBound[float]
#               #ENDLOOP
#----------------------------------------------------------------------------------------
# description  Returns the shipping levels for a given currency and tax model ordered
#               by LowerBound ascending.
#----------------------------------------------------------------------------------------
# input        #ShippingMethod | shipping method object | object
# input        #Currency | currency code, e.g. "EUR" | string
# input        #TaxModel | tax calculation model (0=net, 1=gross) | int
# input        #ExtraPrecision | extra precision for net prices in back-office | boolean
#========================================================================================
sub ShippingLevels {
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($ShippingMethod, $CurrencyID, $TaxModel, $ExtraPrecision) = @$aParams;

    return $ShippingMethod->getLevels($CurrencyID, $TaxModel, {'ExtraPrecision' => $ExtraPrecision });
}

1;
