#========================================================================================
# package      DE_EPAGES::ShippingTypes::API::Object::ShippingMethodFixPrice
# state        public
# base         DE_EPAGES::Order::API::Object::ContainerShippingMethod
#----------------------------------------------------------------------------------------
# description  Object interface for ShippingMethodFixPrice.
#========================================================================================
package DE_EPAGES::ShippingTypes::API::Object::ShippingMethodFixPrice;
use base DE_EPAGES::Order::API::Object::ContainerShippingMethod;

use strict;
use DE_EPAGES::Core::API::Error qw (Error );

sub ErrorDefinition { return {
    'ShippingMethodNoPrice'    => 'shipping method ( #ShippingMethodID ) has no price.',
}}

#========================================================================================
# function     lineItem
#----------------------------------------------------------------------------------------
# syntax       my $hInfo = $ShippingMethod->lineItem($Container);
#----------------------------------------------------------------------------------------
# description  Returns lineItem for container, adds LineItemPrice.
#----------------------------------------------------------------------------------------
# input        $Container | container | object
# return       $hInfo | line item info | ref.hash
#========================================================================================
sub lineItem {
    my $self = shift;
    my ($Container) = @_;

    my $NewLineItemPrice = $self->calculateShippingPrice($Container);
    my %Info = (
        %{$self->SUPER::lineItem($Container)},
        'LineItemPrice' => $NewLineItemPrice,
    );
    return \%Info;
}


#========================================================================================
# function     price
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShippingPrice = $Object->price( $hVars );
#----------------------------------------------------------------------------------------
# description  Returns the shipping price by the given requirements. Doesn't process
#               ShippingElements.
#----------------------------------------------------------------------------------------
# input        $hVars | <ul>
#                   <li>CurrencyID    - alpha currency code (ISO 4217) - char(3)
#                   <li>TaxModel      - gross or net price    - int
#                   <li>ShippingElements
#                       <ul>
#                          <li>Element   - e.g. Products       - object
#                          <li>Quantity  - quantity of element - float
#                      </ul> | ref.array.hash
#               </ul> | ref.hash
#               </ul> | ref.hash
# return       $ShippingPrice | price of shipping | money
#========================================================================================
sub price {
    my $self = shift;
    my ($hValues) = @_;

    return $self->_price($hValues->{'CurrencyID'}, $hValues->{'TaxModel'});
}

#========================================================================================
# function     calculateShippingPrice
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $ShippingPrice = $ShippingMethod->calculateShippingPrice($Container);
#----------------------------------------------------------------------------------------
# description  Returns the shipping price by the given requirements.
#----------------------------------------------------------------------------------------
# input        $Container | line item container | object
# return       $ShippingPrice | price of shipping | money
#========================================================================================
sub calculateShippingPrice {
    my $self = shift;
    my ($Container) = @_;

    my $hInfo = $Container->get([qw (CurrencyID TaxModel TaxMatrix)]);
    return $self->_price($hInfo->{'CurrencyID'}, $hInfo->{'TaxModel'}, $hInfo->{'TaxMatrix'});
}

#========================================================================================
# function     _price
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $ShippingPrice = $ShippingMethod->_price($CurrencyID, $TaxModel, $TaxMatrix);
#----------------------------------------------------------------------------------------
# description  Returns the shipping price.
#----------------------------------------------------------------------------------------
# input        $CurrencyID | currency id | string(3)
# input        $TaxModel | tax model (0=net, 1=gross) | int
# input        $TaxMatrix | (optional) tax matrix | object
# return       $ShippingPrice | price of shipping | money
#========================================================================================
sub _price {
    my $self = shift;
    my ($CurrencyID, $TaxModel, $TaxMatrix) = @_;

    my $ahLevels = $self->getLevels($CurrencyID, $TaxModel, {'TaxMatrix' => $TaxMatrix});
    Error('ShippingMethodNoPrice', { 'ShippingMethodID'=>$self->id }) unless scalar @$ahLevels;
    my $hLevel = $ahLevels->[0];
    my $ShippingPrice = $hLevel->{'BaseValue'};

    # convert net prices to gross
    Error('ShippingMethodNoPrice', { 'ShippingMethodID'=>$self->id }) unless defined $ShippingPrice;

    return $ShippingPrice;
}

1;
