#========================================================================================
# package      DE_EPAGES::ShippingTypes::API::Object::ShippingMethodFreeLimit
# base         DE_EPAGES::Order::API::Object::ContainerShippingMethod
# state        public
#----------------------------------------------------------------------------------------
# description  object interface for ShippingMethodFreeLimit.
#========================================================================================
package DE_EPAGES::ShippingTypes::API::Object::ShippingMethodFreeLimit;
use base DE_EPAGES::Order::API::Object::ContainerShippingMethod;

use strict;
use DE_EPAGES::Core::API::Error         qw ( Error );

sub ErrorDefinition { return {
    'ShippingMethodNoPrice'    => 'shipping method ( #ShippingMethodID ) has no price.',
}}

#========================================================================================
# function     lineItem
#----------------------------------------------------------------------------------------
# syntax       my $hInfo = $ShippingMethod->lineItem($Container);
#----------------------------------------------------------------------------------------
# description  Returns lineItem for container, adds LineItemPrice.
#----------------------------------------------------------------------------------------
# input        $Container | container | object
# return       $hInfo | line item info | ref.hash
#========================================================================================
sub lineItem {
    my $self = shift;
    my ($Container) = @_;

    my $hShippingPriceInfo = $self->shippingPriceInfo($Container);
    my %Info = (
        %{$self->SUPER::lineItem($Container)},
        %$hShippingPriceInfo,
    );
    return \%Info;
}

#========================================================================================
# function     recalculateLineItem
#----------------------------------------------------------------------------------------
# syntax       $ShippingMethod->recalculateLineItem($LineItemShipping, $hValues);
#----------------------------------------------------------------------------------------
# description  Sets base price and line item price for this line item.
#----------------------------------------------------------------------------------------
# input        $LineItemShipping | LineItemShipping | ref.hash
# input        $hValues | values witch will be changed | ref.hash
#========================================================================================
sub recalculateLineItem {
    my $self = shift;
    my ($LineItem, $hValues) = @_;
    my $Container = $LineItem->container;

    $self->SUPER::recalculateLineItem($LineItem, $hValues);

    my $hShippingPriceInfo = $self->shippingPriceInfo($Container);
    $LineItem->set($hShippingPriceInfo);
    return;
}

#========================================================================================
# function     price
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShippingPrice = $Object->price( $hVars );
#----------------------------------------------------------------------------------------
# description  Returns the shipping price by the given requirements.
#----------------------------------------------------------------------------------------
# input        $hVars | <ul>
#                   <li>CurrencyID    - alpha currency code (ISO 4217) - char(3)</li>
#                   <li>TaxModel      - gross or net price    - int</li>
#                   <li>ShippingElements</li>
#                       <ul>
#                          <li>Element   - e.g. Products       - object</li>
#                          <li>Quantity  - quantity of element - float</li>
#                      </ul> | ref.array.hash
#               </ul> | ref.hash
# return       $ShippingPrice | price of shipping | money
#========================================================================================
sub price {
    my $self = shift;
    my ($hValues) = @_;

    #get sum of shipping
    my $SumForShipping = 0;
    my $ShippingElements  = $hValues->{'ShippingElements'};

    foreach my $ShippingElement ( @$ShippingElements ) {
        my $Quantity    = $ShippingElement->{'Quantity'};
        my $PriceQuantity = $ShippingElement->{'Element'}->getWithParent('PriceQuantity');
        $PriceQuantity = 1 unless defined $PriceQuantity;

        my $hVars = {
           'CurrencyID' => $hValues->{'CurrencyID'},
           'TaxModel'   => $hValues->{'TaxModel'},
           'Quantity'   => $Quantity
        };

        $SumForShipping += $ShippingElement->{'Element'}->price($hVars)
                           * ($Quantity / $PriceQuantity)
                           - $ShippingElement->{'Element'}->discount($hVars);
    }

    return $self->_price($hValues->{'CurrencyID'}, $hValues->{'TaxModel'}, $SumForShipping);
}
#========================================================================================
# function     calculateShippingPrice
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $ShippingPrice = $ShippingMethod->calculateShippingPrice($Container);
#----------------------------------------------------------------------------------------
# description  Returns the shipping price by the given requirements.
#----------------------------------------------------------------------------------------
# input        $Container | LineItemContainer | ref.hash
# return       $ShippingPrice | price of shipping | money
#========================================================================================
sub calculateShippingPrice {
    my $self = shift;
    my ($Container) = @_;

    my $hPriceInfo  = $self->shippingPriceInfo($Container);

    return $hPriceInfo->{'LineItemPrice'} ;
}

#========================================================================================
# function     shippingPriceInfo
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $hShippingPriceInfo = $ShippingMethod->shippingPriceInfo($Container);
#----------------------------------------------------------------------------------------
# description  Returns the shipping price, and the BasePrice by the given requirements.
#----------------------------------------------------------------------------------------
# input        $Container | LineItemContainer | ref.hash
# return       $hShippingPriceInfo| <ul>
#                   <li>LineItemPrice - price of shipping  - float</li>
#                   <li>BasePrice     - price of container - float</li>
#               </ul> | ref.hash
#========================================================================================
sub shippingPriceInfo {
    my $self = shift;
    my ($Container) = @_;

    #--- select LineItemSubTotal, SalesDiscountsSubTotal, DiscountsSubTotal
    my $AttributeName = $self->get('ContainerSubTotalAttrName');
    $AttributeName = 'LineItemsSubTotal' unless defined $AttributeName;
    my $SumForShipping = $Container->get($AttributeName);

    my $hInfo = $Container->get([qw (CurrencyID TaxModel TaxMatrix)]);
    my $ShippingPrice = $self->_price($hInfo->{'CurrencyID'}, $hInfo->{'TaxModel'}, $SumForShipping, $hInfo->{'TaxMatrix'});

    return {
        'LineItemPrice' => $ShippingPrice,
        'BasePrice'     => $SumForShipping,
    };
}

#========================================================================================
# function     _price
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $ShippingPrice = $ShippingMethod->_price($CurrencyID, $TaxModel, $BasketPrice, $TaxMatrix);
#----------------------------------------------------------------------------------------
# description  Returns the shipping price for the given basket price.
#----------------------------------------------------------------------------------------
# input        $CurrencyID | currency id | string(3)
# input        $TaxModel | tax model (0=net, 1=gross) | int
# input        $BasketPrice | basket price | money
# input        $TaxMatrix | tax matrix | object
# return       $ShippingPrice | price of shipping | money
#========================================================================================
sub _price {
    my $self = shift;
    my ($CurrencyID, $TaxModel, $BasketPrice, $TaxMatrix) = @_;

    my $ahLevels = $self->getLevels($CurrencyID, $TaxModel, {'TaxMatrix' => $TaxMatrix});
    Error('ShippingMethodNoPrice', { 'ShippingMethodID'=>$self->id }) unless scalar @$ahLevels;
    my $hLevel = $ahLevels->[0];
    my $ShippingPrice = $hLevel->{'BaseValue'};
    Error('ShippingMethodNoPrice', { 'ShippingMethodID'=>$self->id }) unless defined $ShippingPrice;

    if (defined $hLevel->{'UpperBound'} and $BasketPrice >= $hLevel->{'UpperBound'}) {
        $ShippingPrice = 0;
    }
    return $ShippingPrice;
}

#========================================================================================
# function     valueToFreeDelivery
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $MissingValueForFreeDelivery = $ShippingMethod->valueToFreeDelivery($Container);
#----------------------------------------------------------------------------------------
# description  Returns the missing value to get free delivery.
#----------------------------------------------------------------------------------------
# input        $Container | LineItemContainer | ref.hash
# return       $MissingValueForFreeDelivery| missing value | float
#========================================================================================
sub valueToFreeDelivery {
    my $self = shift;
    my ($Container) = @_;

    #--- select LineItemSubTotal, SalesDiscountsSubTotal, DiscountsSubTotal
    my $AttributeName = $self->get('ContainerSubTotalAttrName');
    $AttributeName = 'LineItemsSubTotal' unless defined $AttributeName;
    my $SumForShipping = $Container->get($AttributeName);

    my $LineItemShipping = $Container->get('Shipping');
    my $Value = $LineItemShipping->get('ExemptionLimitForFreeDelivery') - $SumForShipping;
    return $Value > 0 ? $Value : 0;
}

1;
