#========================================================================================
# package      DE_EPAGES::ShippingTypes::API::Object::ShippingMethodPriceOfBasket
# state        public
# base         DE_EPAGES::Order::API::Object::ContainerShippingMethod
#----------------------------------------------------------------------------------------
# description  object interface for ShippingMethodPriceOfBasket.
#========================================================================================
package DE_EPAGES::ShippingTypes::API::Object::ShippingMethodPriceOfBasket;
use base DE_EPAGES::Order::API::Object::ContainerShippingMethod;

use strict;
use DE_EPAGES::Core::API::Error qw ( Error );

sub ErrorDefinition {
    return {
    'ShippingMethodNoPrice'    => 'shipping method ( #ShippingMethodID ) has no price.',
    }
}

#========================================================================================
# function     price
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShippingPrice = $Object->price( $hValues );
#----------------------------------------------------------------------------------------
# description  Returns the shipping price by the given requirements.
#----------------------------------------------------------------------------------------
# input        $hValues | <ul>
#                   <li>CurrencyID    - alpha currency code (ISO 4217) - char(3)
#                   <li>TaxModel      - gross or net price    - int
#                   <li>TaxMatrix     - tax matrix    - object
#                   <li>ShippingElements
#                       <ul>
#                          <li>Element   - e.g. Products       - object
#                          <li>Quantity  - quantity of element - float
#                      </ul> | ref.array.hash
#               </ul> | ref.hash
# return       $ShippingPrice | price of shipping | money
#========================================================================================
sub price {
    my $self = shift;
    my ($hValues) = @_;

    # get sum of shipping
    my $SumWithDownloadProducts = $self->get('SumWithDownloadProducts');
    my $SumForShipping = 0;
    my $ShippingElements  = $hValues->{'ShippingElements'};

    foreach my $ShippingElement ( @$ShippingElements ) {
        next if (not $SumWithDownloadProducts and $ShippingElement->{'Element'}->class->alias eq 'Product' and $ShippingElement->{'Element'}->get('IsDownloadProduct'));

        my $Quantity = $ShippingElement->{'Quantity'};
        my $PriceQuantity = $ShippingElement->{'Element'}->getWithParent('PriceQuantity');
        $PriceQuantity = 1 unless defined $PriceQuantity;

        # get the price for 1! product with the quantity of the products in basket
        # pricelists are also triggered (hook)
        my $ShippingElementPrice = $ShippingElement->{'Element'}->price({
            'CurrencyID' => $hValues->{'CurrencyID'},
            'TaxModel'   => $hValues->{'TaxModel'},
            'Quantity'   => $Quantity
        });

        # the productprice for shipping calculation is:
        # "the basket quanity" * "price for 1 product" / "product quantity for the price of 1 product"
        $SumForShipping += $Quantity * $ShippingElementPrice / $PriceQuantity;
    }

    return $self->_price($hValues->{'CurrencyID'}, $hValues->{'TaxModel'}, $SumForShipping, $hValues->{'TaxMatrix'});
}

#========================================================================================
# function     calculateShippingPrice
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $ShippingPrice = $ShippingMethod->calculateShippingPrice($Container);
#----------------------------------------------------------------------------------------
# description  Returns the shipping price by the given requirements.
#----------------------------------------------------------------------------------------
# input        $Container | LineItemContainer | ref.hash
# return       $ShippingPrice | price of shipping | money
#========================================================================================
sub calculateShippingPrice {
    my $self = shift;
    my ($Container) = @_;

    #--- select LineItemSubTotal, SalesDiscountsSubTotal, DiscountsSubTotal
    my $AttributeName = $self->get('ContainerSubTotalAttrName');
    $AttributeName = 'LineItemsSubTotal' unless defined $AttributeName;

    #--- append WithoutDownloadProducts if option was set
    $AttributeName .= 'WithoutDownloadProducts' unless $self->get('SumWithDownloadProducts');

    #--- get product (and discount) sum
    my $SumForShipping = $Container->get($AttributeName);

    my $hInfo = $Container->get([qw (CurrencyID TaxModel TaxMatrix)]);
    return $self->_price($hInfo->{'CurrencyID'}, $hInfo->{'TaxModel'}, $SumForShipping, $hInfo->{'TaxMatrix'});
}

#========================================================================================
# function     _price
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $ShippingPrice = $ShippingMethod->_price($CurrencyID, $TaxModel, $BasketPrice);
#----------------------------------------------------------------------------------------
# description  Selects the corresponding shipping level and calculates the shipping
#               price for the given basket price.
#----------------------------------------------------------------------------------------
# input        $CurrencyID | currency id | string(3)
# input        $TaxModel | tax model (0=net, 1=gross) | int
# input        $BasketPrice | basket price | money
# input        $TaxMatrix | tax matrix | object
# return       $ShippingPrice | price of shipping | money
#========================================================================================
sub _price {
    my $self = shift;
    my ($CurrencyID, $TaxModel, $BasketPrice, $TaxMatrix) = @_;

    my $AbsBasketPrice = abs($BasketPrice);
    my $ahLevels = $self->getLevels($CurrencyID, $TaxModel, {'TaxMatrix' => $TaxMatrix});

    my $ShippingPrice;
    foreach my $hLevel (@$ahLevels) {
        if ($hLevel->{'LowerBound'} <= $AbsBasketPrice && $AbsBasketPrice < $hLevel->{'UpperBound'}) {
            $ShippingPrice = $hLevel->{'Multiplier'} * $AbsBasketPrice + $hLevel->{'BaseValue'};
            last;
        }
    }

    Error('ShippingMethodNoPrice', { 'ShippingMethodID'=>$self->id }) unless defined $ShippingPrice;

    # shipping price should never be negative
    $ShippingPrice = 0 if $ShippingPrice < 0;
    return $ShippingPrice;
}


1;
