#========================================================================================
# package      DE_EPAGES::ShopMigration::API::PriceListGlobalValueDiscountsCommon
# state        public
#----------------------------------------------------------------------------------------
# description common functions for exporting/importing PriceList GlobalValueDiscounts
#========================================================================================
package DE_EPAGES::ShopMigration::API::PriceListGlobalValueDiscountsCommon;
use base qw(Exporter);

use strict;
use feature 'switch';

our @EXPORT_OK = qw(
    GetPriceListGlobalValueDiscounts
    SetPriceListGlobalValueDiscounts
    SetPriceListGlobalValueDiscountData
    GetPriceListGlobalValueDiscountData
);
use DE_EPAGES::Core::API::PerlTools qw(ElementOf);
use DE_EPAGES::Core::API::Error qw(Error);
use DE_EPAGES::Object::API::Factory qw (LoadClassByAlias);
use DE_EPAGES::PriceList::API::Table::GlobalValueDiscount qw (GetGlobalValueDiscountsByPriceListID);

use DE_EPAGES::ShopMigration::API::Constants qw(
    PRICELIST_GLOBALVALUEDISCOUNT_ATTRIBUTES
);
use DE_EPAGES::PriceList::API::PriceList qw (GetExportablePriceLists);

#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'REFERENCED_PRICELIST_NOT_FOUND' => 'Cannot create global value discount as Pricelist #Name was not found in shop #Shop',
    };
}
#ERRORDEFINITION END



#========================================================================================
# function     GetPriceListGlobalValueDiscounts
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $ahPriceLisGlobalValueDiscounts = GetPriceListGlobalValueDiscounts($Shop);
#----------------------------------------------------------------------------------------
# description  Get the global value discounts for a shop
#----------------------------------------------------------------------------------------
# input        $Shop | BaseShop (epages6) | object
#----------------------------------------------------------------------------------------
# return       $ahPriceListGlobalValueDiscounts| see GetPriceListGlobalValueDiscountData() |
#               <ul>
#                   <li>PriceListAlias   - pricelist alias | - string
#                   <li>Value - value | - decimal
#                   <li>Discount - discount | - double
#                   <li>DiscountAbsolute - discount absolute | - decimal
#               </ul> | reference to array of hashes
#========================================================================================
sub GetPriceListGlobalValueDiscounts {
    my ($Shop) = @_;
    my @Items;

    my $PriceLists = GetExportablePriceLists($Shop);
    foreach my $PriceList (@$PriceLists) {
        my $GlobalValueDiscountsForPriceList = GetGlobalValueDiscountsByPriceListID($Shop->id, $PriceList->id);

        foreach my $GlobalValueDiscountEntry (@$GlobalValueDiscountsForPriceList) {
            $GlobalValueDiscountEntry->{'PriceListAlias'} = $PriceList->alias;
            push @Items, $GlobalValueDiscountEntry;
        }
    }

    return \@Items;
}

#========================================================================================
# function     GetPriceListGlobalValueDiscountData
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $hPriceListGlobalValueDiscount = GetPriceListGlobalValueDiscountData($Shop, $PriceListGlobalValueDiscounts);
#----------------------------------------------------------------------------------------
# description  Constructs a hash with the exportable fields for a single priceList global value discount
#----------------------------------------------------------------------------------------
# input        $Shop | BaseShop (epages6) | object
# input        $PriceListGlobalValueDiscount | PriceListGlobalValueDiscount | object
#----------------------------------------------------------------------------------------
# return       $hPriceListGlobalValueDiscount | hash with following fields
#               <ul>
#                   <li>Attributes
#                       <ul>
#                           <li>PriceListAlias                   - string</li>
#                           <li>Value                            - integer</li>
#                           <li>Discount                         - double</li>
#                           <li>DiscountAbsolute                 - double</li>
#                       </ul>
#                   </li>
#               </ul>| reference to hash
#========================================================================================
sub GetPriceListGlobalValueDiscountData {
    my ($Shop, $PriceListGlobalValueDiscounts) = @_;
    my %Info = ();

    my $Attributes = [ @{PRICELIST_GLOBALVALUEDISCOUNT_ATTRIBUTES()} ];
    foreach my $Attribute (@$Attributes) {
        $Info{$Attribute} = $PriceListGlobalValueDiscounts->{$Attribute} if defined $PriceListGlobalValueDiscounts->{$Attribute};
    }
    return \%Info;
}

#========================================================================================
# function     SetPriceListGlobalValueDiscounts
# state        public
#----------------------------------------------------------------------------------------
# syntax       SetPriceListGlobalValueDiscounts($Shop, $ahPriceListGlobalValueDiscounts);
#----------------------------------------------------------------------------------------
# description  sets attributes on PriceListGlobalValueDiscount as specified in the $ahPriceListGlobalValueDiscounts
#----------------------------------------------------------------------------------------
# input        $Shop         | baseshop or prepshop | object
# input        $ahPriceListGlobalValueDiscounts  | see SetPriceListGlobalValueDiscountData() | reference to array of hashes
#========================================================================================
sub SetPriceListGlobalValueDiscounts {
    my ($Shop, $ahPriceListGlobalValueDiscounts) = @_;

    foreach my $hPriceListGlobalValueDiscount (@$ahPriceListGlobalValueDiscounts) {
        SetPriceListGlobalValueDiscountData($Shop, $hPriceListGlobalValueDiscount);
    }
}

#========================================================================================
# function     SetPriceListGlobalValueDiscountData
# state        public
#----------------------------------------------------------------------------------------
# syntax       SetPriceListGlobalValueDiscountData($Shop, $hPriceListGlobalValueDiscount);
#----------------------------------------------------------------------------------------
# description  sets attributes on a PriceListGlobalValueDiscount as specified in the hash $hPriceListGlobalValueDiscount
#----------------------------------------------------------------------------------------
# input        $Shop         | baseshop or prepshop | object
# input        $hPriceListGlobalValueDiscount    | hash with following fields
#               <ul>
#                   <li>Attributes
#                       <ul>
#                           <li>PriceListAlias                   - string</li>
#                           <li>Value                            - integer</li>
#                           <li>Discount                         - double</li>
#                           <li>DiscountAbsolute                 - double</li>
#                       </ul>
#                   </li>
#               </ul>| reference to hash
#========================================================================================
sub SetPriceListGlobalValueDiscountData {
    my ($Shop, $hPriceListGlobalValueDiscount) = @_;
    my @Attributes = @{PRICELIST_GLOBALVALUEDISCOUNT_ATTRIBUTES()};
    
    foreach my $Key (keys %{$hPriceListGlobalValueDiscount->{Attributes}}) {
        die "Invalid Element for Attributes: $Key" unless ElementOf($Key, @Attributes);
    }

    my $PriceListAlias;
    if ($Shop->get('IsMigrationPrepShop')) {
        #add temporary Prefix "PL_epages6_" to $PriceListAlias
        $PriceListAlias = "PL_epages6_".$hPriceListGlobalValueDiscount->{'PriceListAlias'};
    } else{
        $PriceListAlias = $hPriceListGlobalValueDiscount->{'PriceListAlias'};
    }

    my $PriceList = $Shop->folder('PriceLists')->existsChild($PriceListAlias) ?
        $Shop->folder('PriceLists')->child($PriceListAlias) :
        Error('REFERENCED_PRICELIST_NOT_FOUND', { Name => $PriceListAlias, Shop => $Shop->alias });

    #do not import globalvaluediscounts for pricelist with assigned IsNonMemberDefault customergoup
    my $aAssignedCustomerGroup = $PriceList->get('CustomerGroups');
    return if @$aAssignedCustomerGroup[0]->get('IsNonMemberDefault');

    LoadClassByAlias('GlobalValueDiscount')->insertObject({
        'PriceList'         => $PriceList,
        'Value'             => $hPriceListGlobalValueDiscount->{'Value'},
        'Discount'          => $hPriceListGlobalValueDiscount->{'Discount'},
        'DiscountAbsolute'  => $hPriceListGlobalValueDiscount->{'DiscountAbsolute'},
    });
    
    return;
}

1;
