#========================================================================================
# package      DE_EPAGES::ShopMigration::t::CustomerGroupsExportImport
# base         DE_EPAGES::Test::API::UnitTest::Test
#----------------------------------------------------------------------------------------
# description  unit test for DE_EPAGES::ShopMigration::API::JSONLines::CustomerGroupsExportImport
#========================================================================================
package DE_EPAGES::ShopMigration::t::CustomerGroupsExportImport;
use base DE_EPAGES::Test::API::StoreUnitTest;
## no critic (Tics::ProhibitLongLines)
use strict;
use utf8;
use warnings;
use File::Spec::Functions;
use DE_EPAGES::Core::API::File qw(GetFileContent GetFileContentLines ExistsFile DeleteDirectory DeleteFile Text2Unix GetFileNames);
use DE_EPAGES::Core::API::PerlTools qw(ElementOf);
use DE_EPAGES::Object::API::Factory qw(LoadRootObject LoadClassByAlias);
use DE_EPAGES::Object::API::Language qw(GetPKeyLanguageByCode);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw(ImportObjects);
use DE_EPAGES::ShopMigration::API::JSON qw(DecodeJSON DecodeJSONArray);
use DE_EPAGES::ShopMigration::API::JSONLines::CustomerGroupsExportImport qw(ExportCustomerGroupsJSON ImportCustomerGroupsJSON);
use DE_EPAGES::ShopMigration::API::MigrationShop qw(CreatePrepShop ExistsPrepShop SetShopFeatureValues);
use File::Temp qw(tempdir);
use Test::More;
use Test::Differences qw(context_diff eq_or_diff) ;

my $TmpDir = tempdir();

if ($ENV{SKIP_IN_PR_TESTS}) {
    plan skip_all => 'Skipping it because SKIP_IN_PR_TESTS == 1';
}

sub test {
    context_diff();
    testExportWithCustomerGroupsExportImportBase2Prep();
    testImportWithCustomerGroupsExportImportBase2Prep();
    testExportWithCustomerGroupsExportImportPrep2Base();
    testImportWithCustomerGroupsExportImportPrep2Base();

    done_testing();
}

my %TestData = (
    TestShop      => undef,
    TestShopAlias => 'TestShop',
    ImportPath    => '/Test/Shops/TestShop',
);
use constant FILENAME => 'customergroups.json';
use constant FILE_DIR => [ qw(ShopData.d export TestShop) ];
use constant FILE_LINES => 2;
use constant FILE_LINES_PREP => 19;

# This is a bit hacky, the json exporter *can* export a numeric value either as 1 or as "1" (string) depending on how it was used before
# I got random results, so we just check both (as it just concerns the "IsActive" key)
sub testExportWithCustomerGroupsExportImportBase2Prep {
    my $Shop = $TestData{TestShop};
    note('test CustomerGroupsExportDriver (Base2Prep)');
    isnt($Shop, undef, 'Testshop exists');

    foreach my $Lang (qw(de en)) {
        my $LanguageID = GetPKeyLanguageByCode($Lang);
        $Shop->addLanguage($LanguageID) unless $Shop->existsLanguage($LanguageID);
    }

    foreach my $Locale (qw(de_DE en_GB)) {
        $Shop->addLocale($Locale) unless $Shop->existsLocale($Locale);
    }

    is(scalar @{$Shop->get('Languages')}, 2, 'Testshop has now two Languages');
    is(scalar @{$Shop->get('SiteLocales')}, 2, 'Testshop has two SiteLocales');
    my $Exporter = ExportCustomerGroupsJSON($Shop, $TmpDir);


    foreach my $Lang (qw(de en ru)) {
        my $LanguageID = GetPKeyLanguageByCode($Lang);
        $Shop->addLanguage($LanguageID) unless $Shop->existsLanguage($LanguageID);
    }

    foreach my $Locale (qw(ru_RU de_DE en_GB)) {
        $Shop->addLocale($Locale) unless $Shop->existsLocale($Locale);
    }

    is(scalar @{$Shop->get('Languages')}, 3, 'Testshop has now three Languages');
    ok(ExistsFile($Exporter->fileName), 'export file exists: ' . $Exporter->fileName);
    my $LinesCount = scalar @{GetFileContentLines($Exporter->fileName, { encoding => 'utf8' })};
    is($LinesCount, FILE_LINES, "export file lines count is ${\FILE_LINES}");
    ok($Exporter->fileName =~ /${\FILENAME}/, "export file name matches ${\FILENAME}");
    my $sFileContent = GetFileContent($Exporter->fileName, { encoding => 'utf8' });
    my $sExpected = GetFileContent(catfile(@{FILE_DIR()}, 'base2prep_' . FILENAME), { encoding => 'utf8' });
    compareCustomerGroupHashes($sFileContent, $sExpected);
    DeleteFile($Exporter->fileName());
    ok(!ExistsFile($Exporter->fileName()), 'export file cleaned up');
    is(scalar @{$Shop->get('SiteLocales')}, 3, 'Testshop has now three SiteLocales');
    $Exporter = ExportCustomerGroupsJSON($Shop, $TmpDir);
    ok(ExistsFile($Exporter->fileName), 'export file exists: ' . $Exporter->fileName);
    is(scalar @{GetFileContentLines($Exporter->fileName, { encoding => 'utf8' })}, 2, 'Export file has 2 lines');
    my $aContentLines = GetFileContentLines($Exporter->fileName, { encoding => 'utf8' });
    my $hValues = DecodeJSON($aContentLines->[0]);
    ok(exists $hValues->{LocalizedAttributes}{en}, 'export file has localized attributes for language en');
    ok(exists $hValues->{LocalizedAttributes}{de}, 'export file has localized attributes for language de');
    DeleteFile($Exporter->fileName());
    ok(!ExistsFile($Exporter->fileName()), 'export file cleaned up');
}

sub testImportWithCustomerGroupsExportImportBase2Prep {
    my $ExpectedValues = {
        NewCustomer => {
            'Attributes'              => {
                'Alias' => 'NewCustomer',
                'IsNonMemberDefault' => '1',
                'IsNewMemberDefault' => '1'
            },
            'LocalizedAttributes' => {
                'en' => { 'Name' => 'TestCase: New customer' },
                'de' => { 'Name' => 'TestCase: Neukunde' }
            },
        },
        RegularCustomer => {
            'Attributes'          => {
                'Alias' => 'RegularCustomer',
                'IsNonMemberDefault' => '0',
                'IsNewMemberDefault' => '0',
            },
            'LocalizedAttributes' => {
                'en' => { 'Name' => 'TestCase: Regular customer' },
                'de' => { 'Name' => 'TestCase: Stammkunde' }
            }
        }
    };

    my $PrepShop = $TestData{PrepShop};
    my $DirName = catdir('ShopData.d', 'import');
    note('test CustomerGroupsImportDriver (Base2Prep)');
    isnt($PrepShop, undef, 'Prepshop exists');
    $PrepShop->folder('Groups')->deleteChildren;
    is(scalar @{$PrepShop->folder('Groups')->findChildren}, 0, 'no customer groups on PrepShop');
    my $Importer = ImportCustomerGroupsJSON($PrepShop, $DirName);
    is(scalar @{$PrepShop->folder('Groups')->findChildren}, 2, 'imported 2 customer groups');
    ok($Importer->fileName() =~ /TestShop.base2prep_customergroups[.]json/, 'ImportFileName was constructed properly');
    foreach my $Alias (keys %$ExpectedValues) {
        my $CustomerGroupAlias = $ExpectedValues->{$Alias}{'Attributes'}{'Alias'};
        my $CustomrGroup = $PrepShop->folder('Groups')->child($CustomerGroupAlias);
        foreach my $Attribute (keys %{$ExpectedValues->{$Alias}{Attributes}}) {
            is($CustomrGroup->get($Attribute), $ExpectedValues->{$Alias}{Attributes}{$Attribute}, "attribute $Attribute matches")
        }
        foreach my $LanguageCode (keys %{$ExpectedValues->{$Alias}{LocalizedAttributes}}) {
            my $LanguageID = GetPKeyLanguageByCode($LanguageCode);
            foreach my $Attribute (keys %{$ExpectedValues->{$Alias}{LocalizedAttributes}{$LanguageCode}}) {
                is($CustomrGroup->get($Attribute, $LanguageID),
                    $ExpectedValues->{$Alias}{LocalizedAttributes}{$LanguageCode}{$Attribute},
                    "localized attribute $Attribute for $LanguageCode matches ");
            }
        }
    }
    ## critic
}

sub testExportWithCustomerGroupsExportImportPrep2Base {
    my $Shop = $TestData{PrepShop};
    note('test CustomerGroupsExportDriver (Prep2Base)');
    isnt($Shop, undef, 'PrepShop exists');

    foreach my $Lang (qw(de en)) {
        my $LanguageID = GetPKeyLanguageByCode($Lang);
        $Shop->addLanguage($LanguageID) unless $Shop->existsLanguage($LanguageID);
    }

    foreach my $Locale (qw(de_DE en_GB)) {
        $Shop->addLocale($Locale) unless $Shop->existsLocale($Locale);
    }

    is(scalar @{$Shop->get('Languages')}, 2, 'PrepShop has now two Languages');
    is(scalar @{$Shop->get('SiteLocales')}, 2, 'PrepShop has two SiteLocales');
    my $Exporter = ExportCustomerGroupsJSON($Shop, $TmpDir);

    foreach my $Lang (qw(de en ru)) {
        my $LanguageID = GetPKeyLanguageByCode($Lang);
        $Shop->addLanguage($LanguageID) unless $Shop->existsLanguage($LanguageID);
    }

    foreach my $Locale (qw(ru_RU de_DE en_GB)) {
        $Shop->addLocale($Locale) unless $Shop->existsLocale($Locale);
    }

    is(scalar @{$Shop->get('Languages')}, 3, 'PrepShop has now three Languages');
    ok(ExistsFile($Exporter->fileName), 'export file exists: ' . $Exporter->fileName);
    my $LinesCount = scalar @{GetFileContentLines($Exporter->fileName, { encoding => 'utf8' })};
    is($LinesCount, FILE_LINES, "export file lines count is ${\FILE_LINES}");
    ok($Exporter->fileName =~ /${\FILENAME}/, "export file name matches ${\FILENAME}");
    my $sFileContent = GetFileContent($Exporter->fileName, { encoding => 'utf8' });
    my $sExpected = GetFileContent(catfile(@{FILE_DIR()}, 'prep2base_' . FILENAME), { encoding => 'utf8' });
    compareCustomerGroupHashes($sFileContent, $sExpected);
    DeleteFile($Exporter->fileName());
    ok(!ExistsFile($Exporter->fileName()), 'export file cleaned up');
    is(scalar @{$Shop->get('SiteLocales')}, 3, 'PrepShop has now three SiteLocales');
    $Exporter = ExportCustomerGroupsJSON($Shop, $TmpDir);
    ok(ExistsFile($Exporter->fileName), 'export file exists: ' . $Exporter->fileName);
    is(scalar @{GetFileContentLines($Exporter->fileName, { encoding => 'utf8' })}, 2, 'Export file has 2 lines');
    my $aContentLines = GetFileContentLines($Exporter->fileName, { encoding => 'utf8' });
    my $hValues = DecodeJSON($aContentLines->[0]);
    ok(exists $hValues->{LocalizedAttributes}{en}, 'export file has localized attributes for language en');
    ok(exists $hValues->{LocalizedAttributes}{de}, 'export file has localized attributes for language de');
    DeleteFile($Exporter->fileName());
    ok(!ExistsFile($Exporter->fileName()), 'export file cleaned up');
}

sub testImportWithCustomerGroupsExportImportPrep2Base {

    my $ExpectedValues = {
        NewCustomer => {
            'Attributes'              => {
                'Alias' => 'NewCustomer',
                'IsNonMemberDefault' => '1',
                'IsNewMemberDefault' => '1'
            },
            'LocalizedAttributes' => {
                'en' => { 'Name' => 'TestCase: New customer' },
                'de' => { 'Name' => 'TestCase: Neukunde' }
            },
        },
        RegularCustomer => {
            'Attributes'          => {
                'Alias' => 'RegularCustomer',
                'IsNonMemberDefault' => '0',
                'IsNewMemberDefault' => '0',
            },
            'LocalizedAttributes' => {
                'en' => { 'Name' => 'TestCase: Regular customer' },
                'de' => { 'Name' => 'TestCase: Stammkunde' }
            }
        }
    };

    my $Shop = $TestData{TestShop};
    my $DirName = catdir('ShopData.d', 'import');
    note('test CustomerGroupsImportHandler (Prep2Base)');
    isnt($Shop, undef, 'Testshop exists');
    my $Importer = ImportCustomerGroupsJSON($Shop, $DirName);
    is(scalar @{$Shop->get('CustomerGroups')}, 2, 'imported 2 customergroups');
    ok($Importer->fileName() =~ /TestShop.prep2base_customergroups[.]json/, 'ImportFileName was constructed properly');
    foreach my $CustomerGroupAlias (keys %$ExpectedValues) {
        my $ShopCustomerGroup = $Shop->folder('Groups')->child($CustomerGroupAlias);
        foreach my $Attribute (keys %{$ExpectedValues->{$CustomerGroupAlias}{Attributes}}) {
            is($ShopCustomerGroup->get($Attribute), $ExpectedValues->{$CustomerGroupAlias}{Attributes}{$Attribute}, "attribute $Attribute matches")
        }
        foreach my $LanguageCode (keys %{$ExpectedValues->{$CustomerGroupAlias}{LocalizedAttributes}}) {
            my $LanguageID = GetPKeyLanguageByCode($LanguageCode);
            foreach my $Attribute (keys %{$ExpectedValues->{$CustomerGroupAlias}{LocalizedAttributes}{$LanguageCode}}) {
                is($ShopCustomerGroup->get($Attribute, $LanguageID),
                    $ExpectedValues->{$CustomerGroupAlias}{LocalizedAttributes}{$LanguageCode}{$Attribute},
                    "localized attribute $Attribute for $LanguageCode matches ");
            }
        }
    }
    ## critic
}

sub compareCustomerGroupHashes {
    my ($CurrentContent, $ExpectedContent) = @_;

    # Removes all unknown (not installed language elements) from $Expected otherwise
    # we get a diff, as it contains all enabled languages after installing all langpacks
    my $aCurrent = DecodeJSONArray($$CurrentContent);
    my $aExpected = DecodeJSONArray($$ExpectedContent);
    my @KeepLanguages = ();
    foreach my $Code2 (map {$_->{'Code2'}} @{LoadRootObject()->get('Languages')}) {
        push @KeepLanguages, $Code2;
    }
    foreach my $Line ( @$aExpected ) {
        foreach my $Lang (keys %{$Line->{'LocalizedAttributes'}}) {
            next if ElementOf($Lang, @KeepLanguages);
            delete $Line->{'LocalizedAttributes'}{$Lang};
        }
    }
    eq_or_diff($aCurrent, $aExpected, 'file content matches');
}


# We need to create a fake ShopTypeRef UnityDE + ECom5000 in case they don't exists
# otherwise the Mapper won't work
sub init {
    my $self = shift;

    $self->SUPER::init;
    my $ShopAlias = $TestData{TestShopAlias};
    my $System = LoadRootObject();
    my $Shop;
    my $TestFolder = $System->folder('Test')->folder('Shops');
    my $ShopTypeRefFolder = $System->folder('ShopTypeRefs');
    my $Class = LoadClassByAlias('ShopTypeRef');
    $Class->insertObject({ Alias => 'UnityDE', 'Parent' => $ShopTypeRefFolder })
        unless ($ShopTypeRefFolder->existsChild('UnityDE'));
    $Class->insertObject({ Alias => 'ECom5000', 'Parent' => $ShopTypeRefFolder })
        unless ($ShopTypeRefFolder->existsChild('ECom5000'));
    if ($TestFolder->existsChild($ShopAlias)) {
        $Shop = $TestFolder->child($ShopAlias);
    } else {
        $Shop = $TestFolder->insertChild('Shop', {
            Alias                   => $ShopAlias,
            ShopTypeRef             => $ShopTypeRefFolder->child('ECom5000'),
            WebServerScriptNamePart => $ShopAlias,
        });
        SetShopFeatureValues($Shop,
            [
                { Alias => 'Regions', MaxValue => '2' },
                { Alias => 'Languages', MaxValue => '15' },
                { Alias => 'CustomerGroups', MaxValue => '20' },
                { Alias => 'Languages', MaxValue => '15' }
            ]);
    }

    $TestData{TestShop} = $Shop;

    $TestData{PrepShop} = ExistsPrepShop($Shop) ? $Shop->get('MigrationPrepShop') : CreatePrepShop($Shop);
    foreach my $JSONFile (@{GetFileNames('ShopData.d', { 'recursive' => 1 })}) {
        Text2Unix($JSONFile) if ($^O eq 'linux');
    }
    ImportObjects('ImportFiles/CustomerGroupsBase.xml', $TestData{TestShop}->pathString());
    return;
}

sub cleanup {
    my $self = shift;

    my $ShopAlias = $TestData{TestShopAlias};
    my $TestFolder = LoadRootObject()->folder('Test')->folder('Shops');
    if ($TestFolder->existsChild($ShopAlias)) {
        my $Shop = $TestFolder->child($ShopAlias);
        my $PrepShop = $Shop->get('MigrationPrepShop');
        $PrepShop->delete() if (defined $PrepShop);
        $Shop->delete();
        LoadRootObject()->folder('Test')->delete();
    }
    DeleteDirectory($TmpDir, { 'recursive' => 1 });
    return;
}

__PACKAGE__->runScript();

1;
