#========================================================================================
# package      DE_EPAGES::ShopMigration::t::PriceListsExportImport
# base         DE_EPAGES::Test::API::UnitTest::Test
#----------------------------------------------------------------------------------------
# description  unit test for DE_EPAGES::ShopMigration::API::JSONLines::PriceListsExportImport
#========================================================================================
package DE_EPAGES::ShopMigration::t::PriceListsExportImport;
use DE_EPAGES::ShopMigration::API::PriceListsCommon qw (CreateAdditionalPriceListsPrepShop);
use base DE_EPAGES::Test::API::StoreUnitTest;
## no critic (Tics::ProhibitLongLines)
use strict;
use utf8;
use warnings;
use File::Spec::Functions;
use DE_EPAGES::Core::API::File qw(GetFileContent GetFileContentLines ExistsFile DeleteDirectory DeleteFile Text2Unix GetFileNames);
use DE_EPAGES::Core::API::PerlTools qw(ElementOf);
use DE_EPAGES::Object::API::Factory qw(LoadRootObject LoadClassByAlias);
use DE_EPAGES::Object::API::Language qw(GetPKeyLanguageByCode);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw(ImportObjects);
use DE_EPAGES::ShopMigration::API::JSON qw(DecodeJSON DecodeJSONArray);
use DE_EPAGES::ShopMigration::API::JSONLines::PriceListsExportImport qw(ExportPriceListsJSON ImportPriceListsJSON);
use DE_EPAGES::ShopMigration::API::MigrationShop qw(CreatePrepShop ExistsPrepShop SetShopFeatureValues);
use File::Temp qw(tempdir);
use Test::More;
use Test::Differences qw(context_diff eq_or_diff) ;

my $TmpDir = tempdir();

if ($ENV{SKIP_IN_PR_TESTS}) {
    plan skip_all => 'Skipping it because SKIP_IN_PR_TESTS == 1';
}

sub test {
    context_diff();
    testExportWithPriceListsExportImportBase2Prep();
    testExportWithPriceListsExportImportPrep2Base();
    testImportWithPriceListsExportImportBase2Prep();
    testImportWithPriceListsExportImportPrep2Base();
    cleanup();
    done_testing();
}

my %TestData = (
    TestShop      => undef,
    TestShopAlias => 'TestShop',
    ImportPath    => '/Test/Shops/TestShop',
);
use constant FILENAME => 'pricelists.json';
use constant FILE_DIR => [ qw(ShopData.d export TestShop) ];
use constant FILE_LINES => 1;
use constant FILE_LINES_PREP => 19;

# This is a bit hacky, the json exporter *can* export a numeric value either as 1 or as "1" (string) depending on how it was used before
# I got random results, so we just check both (as it just concerns the "IsActive" key)
sub testExportWithPriceListsExportImportBase2Prep {
    my $Shop = $TestData{TestShop};
    note('test PriceListsExportDriver (Base2Prep)');
    isnt($Shop, undef, 'Testshop exists');

    foreach my $Lang (qw(de en)) {
        my $LanguageID = GetPKeyLanguageByCode($Lang);
        $Shop->addLanguage($LanguageID) unless $Shop->existsLanguage($LanguageID);
    }
    
    foreach my $Locale (qw(de_DE en_GB)) {
        $Shop->addLocale($Locale) unless $Shop->existsLocale($Locale);
    }

    is(scalar @{$Shop->get('Languages')}, 2, 'Testshop has now two Languages');
    is(scalar @{$Shop->get('SiteLocales')}, 2, 'Testshop has two SiteLocales');
    my $Exporter = ExportPriceListsJSON($Shop, $TmpDir);

    foreach my $Lang (qw(de en ru)) {
        my $LanguageID = GetPKeyLanguageByCode($Lang);
        $Shop->addLanguage($LanguageID) unless $Shop->existsLanguage($LanguageID);
    }
    
    foreach my $Locale (qw(ru_RU de_DE en_GB)) {
        $Shop->addLocale($Locale) unless $Shop->existsLocale($Locale);
    }

    is(scalar @{$Shop->get('Languages')}, 3, 'Testshop has now three Languages');
    ok(ExistsFile($Exporter->fileName), 'export file exists: ' . $Exporter->fileName);
    my $LinesCount = scalar @{GetFileContentLines($Exporter->fileName, { encoding => 'utf8' })};
    is($LinesCount, FILE_LINES, "export file lines count is ${\FILE_LINES}");
    ok($Exporter->fileName =~ /${\FILENAME}/, "export file name matches ${\FILENAME}");
    my $sFileContent = GetFileContent($Exporter->fileName, { encoding => 'utf8' });
    my $sExpected = GetFileContent(catfile(@{FILE_DIR()}, 'base2prep_' . FILENAME), { encoding => 'utf8' });
    comparePriceListHashes($sFileContent, $sExpected);
    DeleteFile($Exporter->fileName());
    ok(!ExistsFile($Exporter->fileName()), 'export file cleaned up');

    is(scalar @{$Shop->get('SiteLocales')}, 3, 'Testshop has now three SiteLocales');
    $Exporter = ExportPriceListsJSON($Shop, $TmpDir);
    ok(ExistsFile($Exporter->fileName), 'export file exists: ' . $Exporter->fileName);
    is(scalar @{GetFileContentLines($Exporter->fileName, { encoding => 'utf8' })}, 1, 'Export file has 1 lines');
    my $aContentLines = GetFileContentLines($Exporter->fileName, { encoding => 'utf8' });
    my $hValues = DecodeJSON($aContentLines->[0]);
    ok(exists $hValues->{LocalizedAttributes}{en}, 'export file has localized attributes for language en');
    ok(exists $hValues->{LocalizedAttributes}{de}, 'export file has localized attributes for language de');
    DeleteFile($Exporter->fileName());
    ok(!ExistsFile($Exporter->fileName()), 'export file cleaned up');
}

sub testExportWithPriceListsExportImportPrep2Base {
    my $PrepShop = $TestData{PrepShop};
    note('test PriceListsExportDriver (Prep2Base)');
    isnt($PrepShop, undef, 'PrepShop exists');

    foreach my $Lang (qw(de en)) {
        my $LanguageID = GetPKeyLanguageByCode($Lang);
        $PrepShop->addLanguage($LanguageID) unless $PrepShop->existsLanguage($LanguageID);
    }

    foreach my $Locale (qw(de_DE en_GB)) {
        $PrepShop->addLocale($Locale) unless $PrepShop->existsLocale($Locale);
    }

    is(scalar @{$PrepShop->get('Languages')}, 2, 'Testshop has now two Languages');
    is(scalar @{$PrepShop->get('SiteLocales')}, 2, 'Testshop has two SiteLocales');
    my $Exporter = ExportPriceListsJSON($PrepShop, $TmpDir);

    foreach my $Lang (qw(de en ru)) {
        my $LanguageID = GetPKeyLanguageByCode($Lang);
        $PrepShop->addLanguage($LanguageID) unless $PrepShop->existsLanguage($LanguageID);
    }

    foreach my $Locale (qw(ru_RU de_DE en_GB)) {
        $PrepShop->addLocale($Locale) unless $PrepShop->existsLocale($Locale);
    }

    is(scalar @{$PrepShop->get('Languages')}, 3, 'Testshop has now three Languages');
    ok(ExistsFile($Exporter->fileName), 'export file exists: ' . $Exporter->fileName);
    my $LinesCount = scalar @{GetFileContentLines($Exporter->fileName, { encoding => 'utf8' })};
    is($LinesCount, FILE_LINES, "export file lines count is ${\FILE_LINES}");
    ok($Exporter->fileName =~ /${\FILENAME}/, "export file name matches ${\FILENAME}");
    my $sFileContent = GetFileContent($Exporter->fileName, { encoding => 'utf8' });
    my $sExpected = GetFileContent(catfile(@{FILE_DIR()}, 'prep2base_' . FILENAME), { encoding => 'utf8' });
    comparePriceListHashes($sFileContent, $sExpected);
    DeleteFile($Exporter->fileName());
    ok(!ExistsFile($Exporter->fileName()), 'export file cleaned up');

    is(scalar @{$PrepShop->get('SiteLocales')}, 3, 'Testshop has now three SiteLocales');
    $Exporter = ExportPriceListsJSON($PrepShop, $TmpDir);
    ok(ExistsFile($Exporter->fileName), 'export file exists: ' . $Exporter->fileName);
    is(scalar @{GetFileContentLines($Exporter->fileName, { encoding => 'utf8' })}, 1, 'Export file has 1 lines');
    my $aContentLines = GetFileContentLines($Exporter->fileName, { encoding => 'utf8' });
    my $hValues = DecodeJSON($aContentLines->[0]);
    ok(exists $hValues->{LocalizedAttributes}{en}, 'export file has localized attributes for language en');
    ok(exists $hValues->{LocalizedAttributes}{de}, 'export file has localized attributes for language de');
    DeleteFile($Exporter->fileName());
    ok(!ExistsFile($Exporter->fileName()), 'export file cleaned up');
}

sub testImportWithPriceListsExportImportBase2Prep {

    my $ExpectedValues = {
   RegularCustomer => {
     "Attributes" => {
       "Alias" => "RegularCustomer",
       "CurrencyID" => "GBP"
     },
     "CustomerGroups" => [
       {
        "Alias" => "RegularCustomer"
       }
     ],
     "LocalizedAttributes" => {
       "de" => {
         "Name" => "F\x{fc}r Stammkunden (GBP)"
       },
       "en" => {
         "Name" => "For regular customers (GBP)"
       }
     }
   },
 };

    my $PrepShop = $TestData{PrepShop};
    my $DirName = catdir('ShopData.d', 'import');
    note('test PriceListsImportHandler (Base2Prep)');
    isnt($PrepShop, undef, 'Prepshop exists');
    $PrepShop->folder('PriceLists')->deleteChildren;
    is(scalar @{$PrepShop->folder('PriceLists')->findChildren}, 0, 'no PriceLists on PrepShop');
    my $Importer = ImportPriceListsJSON($PrepShop, $DirName);
    is(scalar @{$PrepShop->folder('PriceLists')->findChildren}, 1, 'imported 1 priceList');
    CreateAdditionalPriceListsPrepShop($PrepShop);
    ok($Importer->fileName() =~ /TestShop.base2prep_pricelists[.]json/, 'ImportFileName was constructed properly');
    foreach my $PriceListAlias (keys %$ExpectedValues) {
        my $ShopPriceList = $PrepShop->folder('PriceLists')->child($PriceListAlias);
        foreach my $Attribute (keys %{$ExpectedValues->{$PriceListAlias}{Attributes}}) {
            is($ShopPriceList->get($Attribute), $ExpectedValues->{$PriceListAlias}{Attributes}{$Attribute}, "attribute $Attribute matches")
        }
        foreach my $LanguageCode (keys %{$ExpectedValues->{$PriceListAlias}{LocalizedAttributes}}) {
            my $LanguageID = GetPKeyLanguageByCode($LanguageCode);
            foreach my $Attribute (keys %{$ExpectedValues->{$PriceListAlias}{LocalizedAttributes}{$LanguageCode}}) {
                is($ShopPriceList->get($Attribute, $LanguageID),
                    $ExpectedValues->{$PriceListAlias}{LocalizedAttributes}{$LanguageCode}{$Attribute},
                    "localized attribute $Attribute for $LanguageCode matches ");
            }
        }
    }
    ## critic
}

sub testImportWithPriceListsExportImportPrep2Base {

    my $ExpectedValues = {
        RegularCustomer => {
            "Attributes" => {
                "Alias" => "RegularCustomer",
                "CurrencyID" => "GBP"
            },
            "CustomerGroups" => [
                {
                    "Alias" => "RegularCustomer"
                }
            ],
            "LocalizedAttributes" => {
                "de" => {
                    "Name" => "F\x{fc}r Stammkunden (GBP)"
                },
                "en" => {
                    "Name" => "For regular customers (GBP)"
                }
            }
        },
    };

    my $Shop = $TestData{TestShop};
    my $DirName = catdir('ShopData.d', 'import');
    note('test PriceListsImportHandler (Prep2Base)');
    isnt($Shop, undef, 'Testshop exists');
    $Shop->folder('PriceLists')->deleteChildren;
    is(scalar @{$Shop->folder('PriceLists')->findChildren}, 0, 'no PriceLists on PrepShop');
    my $Importer = ImportPriceListsJSON($Shop, $DirName);
    is(scalar @{$Shop->folder('PriceLists')->findChildren}, 1, 'imported 1 priceList');
    ok($Importer->fileName() =~ /TestShop.prep2base_pricelists[.]json/, 'ImportFileName was constructed properly');
    foreach my $PriceListAlias (keys %$ExpectedValues) {
        my $ShopPriceList = $Shop->folder('PriceLists')->child($PriceListAlias);
        foreach my $Attribute (keys %{$ExpectedValues->{$PriceListAlias}{Attributes}}) {
            is($ShopPriceList->get($Attribute), $ExpectedValues->{$PriceListAlias}{Attributes}{$Attribute}, "attribute $Attribute matches")
        }
        foreach my $LanguageCode (keys %{$ExpectedValues->{$PriceListAlias}{LocalizedAttributes}}) {
            my $LanguageID = GetPKeyLanguageByCode($LanguageCode);
            foreach my $Attribute (keys %{$ExpectedValues->{$PriceListAlias}{LocalizedAttributes}{$LanguageCode}}) {
                is($ShopPriceList->get($Attribute, $LanguageID),
                    $ExpectedValues->{$PriceListAlias}{LocalizedAttributes}{$LanguageCode}{$Attribute},
                    "localized attribute $Attribute for $LanguageCode matches ");
            }
        }

    }
    ## critic
}



sub comparePriceListHashes {
    my ($CurrentContent, $ExpectedContent) = @_;

    my $aCurrent = DecodeJSONArray($$CurrentContent);
    my $aExpected = DecodeJSONArray($$ExpectedContent);
    my @KeepLanguages = ();
    foreach my $Code2 (map {$_->{'Code2'}} @{LoadRootObject()->get('Languages')}) {
        push @KeepLanguages, $Code2;
    }
    foreach my $Line ( @$aExpected ) {
        foreach my $Lang (keys %{$Line->{'LocalizedAttributes'}}) {
            next if ElementOf($Lang, @KeepLanguages);
            delete $Line->{'LocalizedAttributes'}{$Lang};
        }
    }
    eq_or_diff($aCurrent, $aExpected, 'file content matches');
}


# We need to create a fake ShopTypeRef UnityDE + ECom5000 in case they don't exists
# otherwise the Mapper won't work
sub init {
    my $self = shift;

    $self->SUPER::init;
    my $ShopAlias = $TestData{TestShopAlias};
    my $System = LoadRootObject();
    my $Shop;
    my $TestFolder = $System->folder('Test')->folder('Shops');
    my $ShopTypeRefFolder = $System->folder('ShopTypeRefs');
    my $Class = LoadClassByAlias('ShopTypeRef');
    $Class->insertObject({ Alias => 'UnityDE', 'Parent' => $ShopTypeRefFolder })
        unless ($ShopTypeRefFolder->existsChild('UnityDE'));
    $Class->insertObject({ Alias => 'ECom5000', 'Parent' => $ShopTypeRefFolder })
        unless ($ShopTypeRefFolder->existsChild('ECom5000'));
    if ($TestFolder->existsChild($ShopAlias)) {
        $Shop = $TestFolder->child($ShopAlias);
    } else {
        $Shop = $TestFolder->insertChild('Shop', {
            Alias                   => $ShopAlias,
            ShopTypeRef             => $ShopTypeRefFolder->child('ECom5000'),
            WebServerScriptNamePart => $ShopAlias,
        });
        SetShopFeatureValues($Shop,
            [ 
                { Alias => 'Regions', MaxValue => '2' }, 
                { Alias => 'Languages', MaxValue => '15' },
                { Alias => 'CustomerGroups', MaxValue => '2' },
                { Alias => 'PriceLists', MaxValue => '2'},
                { Alias => 'Languages', MaxValue => '15' } 
            ]);
    }

    $TestData{TestShop} = $Shop;

    $TestData{PrepShop} = ExistsPrepShop($Shop) ? $Shop->get('MigrationPrepShop') : CreatePrepShop($Shop);
    SetShopFeatureValues($TestData{PrepShop}, [ 
            { Alias => 'Regions', MaxValue => '2' }, 
            { Alias => 'Languages', MaxValue => '15' },
            { Alias => 'CustomerGroups', MaxValue => '2' },
            { Alias => 'PriceLists', MaxValue => '2'},
            { Alias => 'Languages', MaxValue => '15' } 
         ]);
    foreach my $JSONFile (@{GetFileNames('ShopData.d', { 'recursive' => 1 })}) {
        Text2Unix($JSONFile) if ($^O eq 'linux');
    }
    ImportObjects('ImportFiles/PriceListsBase.xml', $TestData{TestShop}->pathString());
    ImportObjects('ImportFiles/PriceListsNow.xml', $TestData{PrepShop}->pathString());

    return;
}

sub cleanup {
    my $self = shift;

    my $ShopAlias = $TestData{TestShopAlias};
    my $TestFolder = LoadRootObject()->folder('Test')->folder('Shops');
    if ($TestFolder->existsChild($ShopAlias)) {
        my $Shop = $TestFolder->child($ShopAlias);
        my $PrepShop = $Shop->get('MigrationPrepShop');
        $PrepShop->delete() if (defined $PrepShop);
        $Shop->delete();
        LoadRootObject()->folder('Test')->delete();
    }
    DeleteDirectory($TmpDir, { 'recursive' => 1 });
    return;
}

__PACKAGE__->runScript();

1;
