#========================================================================================
# package      DE_EPAGES::Shore::API::TLE::ShoreHandler
# state        public
#----------------------------------------------------------------------------------------
# description Implements a TLE Function which returns preformated Shore widget HTML code
#========================================================================================
package DE_EPAGES::Shore::API::TLE::ShoreHandler;

use strict;
use DE_EPAGES::Core::API::Log qw( GetLog );

#========================================================================================
# function     RegisterHandlerProc
# state        private
#----------------------------------------------------------------------------------------
# syntax       $RegisterHandlerProc( $hParams );
#----------------------------------------------------------------------------------------
# description  this function registers this attribute handle on the processor via hook
#               function on hook TLEProcessorRegistration
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments keys :
#               <ul>
#                   <li>Processor - tle processor - object
#               </ul> | ref.hash
#========================================================================================
sub RegisterHandlerProc {
    my ($Params) = @_;
    GetLog->debug( 'ShoreHandler.RegisterHandlerProc' );

    __PACKAGE__->new()->register( $Params->{'Processor'} );
    return;
}

#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler = new DE_EPAGES::Shore::API::TLE::ShoreHandler;
# example      my $PrefixHandler  = new DE_EPAGES::Shore::API::TLE::ShoreHandle;
#----------------------------------------------------------------------------------------
# description  create a new handler
#----------------------------------------------------------------------------------------
# return       $Handler | blessed object | object
#========================================================================================
sub new { bless {}, shift;}

#========================================================================================
# function     tle
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Value = $Handler->tle( $Processor, $TLEName);
# example      my $Value = $Handler->tle( $Processor, 'Name');
#----------------------------------------------------------------------------------------
# description  This function is called by tle processor to get the value of a tle
#               variable.
#----------------------------------------------------------------------------------------
# input        $Processor | tle processor | object
# input        $TLEName | tle name | string
# return       $Value | return value | integer
#========================================================================================
sub tle {
    my $self = shift;
    my ($Processor, $TLEName) = @_;

    my $Object = $Processor->tle('Object');
    my $Shop = $Object->getSite();

    if( $TLEName eq 'FormatedShoreHTMLCode' ) {
        my $ShoreHTMLCode = $Shop->get('ShoreHTMLCode');
        $ShoreHTMLCode =~ /(window\.shoreBookingSettings = \{[^([{=|]*text: '[^']*'[^([{=|]*\};)/s;
        my $FormattedShoreHTMLCode = $1;

        return $FormattedShoreHTMLCode;
    }
    return undef;
}

#========================================================================================
# function     existsTLE
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Value = $Handler->existsTLE( $Processor, $TLEName);
# example      my $Value = $Handler->existsTLE( $Processor, 'Products_Count');
#----------------------------------------------------------------------------------------
# description  This function is called by the tle processor to test if a TLE variable
#               is available.
#----------------------------------------------------------------------------------------
# input        $Processor | tle processor | object
# input        $TLEName | tle name | string
# return       $Value | yes or no | bool
#========================================================================================
sub existsTLE {
    my $self = shift;
    my ($Processor, $TLEName) = @_;
    GetLog->debug( "ShoreHandler.existsTLE($TLEName)" );
    return (defined $self->tle($Processor, $TLEName)) ? 1 : 0;
}

sub register {
    my $self = shift;
    my ($Processor) = @_;
    GetLog->debug( 'ShoreHandler.register' );
    $Processor->registerHandler('VariableHandler', $self, 'FormatedShoreHTMLCode');
    return;
}
1;

