#========================================================================================
# package      DE_EPAGES::Tax::API::Object::TaxMatrix
# state        public
# base         DE_EPAGES::Shop::API::Object::ShopObject
# description  object interface for TaxMatrix.
#========================================================================================
package DE_EPAGES::Tax::API::Object::TaxMatrix;
use base DE_EPAGES::Shop::API::Object::ShopObject;

use strict;
use DE_EPAGES::Address::API::Address qw(CompareZipCodes);
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::Object::NumberFormat;
use DE_EPAGES::Core::API::TestParameter qw (TestFloat TestString TestObject TestInteger);
use DE_EPAGES::Object::API::Factory qw (LoadObject Transaction);
use DE_EPAGES::Object::API::Table::Currency qw ( GetInfoCurrency );
use DE_EPAGES::Tax::API::Table::TaxAreaCountry qw ( GetInfoTaxAreaCountriesByTaxMatrixID DeleteTaxAreaCountry UpdateTaxAreaCountry );
use DE_EPAGES::Tax::API::Table::TaxRate qw ( ExistsTaxRate GetInfoTaxRate InsertTaxRate GetInfoTaxRatesByTaxAreaID );
use DE_EPAGES::Tax::API::XML::TaxMatrixImportHandler; ## no critic qw( RequireCorrectImports )
use DE_EPAGES::Tax::API::XML::TaxMatrixDeleteHandler; ## no critic qw( RequireCorrectImports )
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use DE_EPAGES::Shop::API::Constants qw( TAX_MODEL_GROSS );

sub ErrorDefinition {
    return {
        'NOT_SAME_MATRIX' => 'tax class (#Class)and area (#Area) not from same matrix',
        'NO_DEFAULT_TAX_AREA' => 'matrix (#ObjectID, #Alias) has no default tax area',
        'NO_FALLBACK_TAX_AREA' => 'matrix (#ObjectID, #Alias) has no fallback tax area',
        'NO_DEFAULT_TAX_CLASS' => 'matrix (#ObjectID, #Alias) has no default tax class',
    }
}

#========================================================================================
# function     className
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $TaxMatrix->className;
#----------------------------------------------------------------------------------------
# description  Returns the class name, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | String
#========================================================================================
sub className  { 'TaxMatrix' }

#========================================================================================
# function     dbPackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DbPackage = $TaxMatrix->dbPackage;
#----------------------------------------------------------------------------------------
# description  Returns the database package, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $DbPackage | database package name | String
#========================================================================================
sub dbPackage { __PACKAGE__ }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $TaxMatrix = DE_EPAGES::Tax::API::Object::TaxMatrix->insert($hInfo);
# example      my $Class = LoadClassByAlias('TaxMatrix');
#               my $TaxMatrix = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook 'OBJ_InsertTaxMatrix' is triggered.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
# return       $TaxMatrix | new object | object
# hook         OBJ_InsertTaxMatrix | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>TaxMatrixID   - tax matrix object id - int
#               </ul>
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;

    my $self = $class->SUPER::insert($hInfo);

    TriggerHook('OBJ_InsertTaxMatrix', {'Object' => $self, 'TaxMatrixID' => $self->id });
    delete $hInfo->{$_} foreach qw ( LineItemClass ); ## no critic qw(ProhibitLongLines)
    return $self;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $TaxMatrix->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeleteTaxMatrix' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeleteTaxMatrix | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>TaxMatrixID   - tax matrix object id - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    TriggerHook('OBJ_DeleteTaxMatrix', {'Object' => $self, 'TaxMatrixID' => $self->id });
    $self->SUPER::delete;
    return;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $TaxMatrix->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdateTaxMatrix' and 'OBJ_AfterUpdateTaxMatrix'.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
# input        $LanguageID | (optional; required for localized attributes) language id | int
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdateTaxMatrix | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>TaxMatrixID   - tax matrix object id - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdateTaxMatrix | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>TaxMatrixID   - tax matrix object id - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    TriggerHook('OBJ_BeforeUpdateTaxMatrix', {'Object'=>$self, 'Values'=> $hValues, 'TaxMatrixID' => $self->id, 'LanguageID' => $LanguageID});
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdateTaxMatrix',  {'Object'=>$self, 'Values'=> $hValues, 'TaxMatrixID' => $self->id, 'LanguageID' => $LanguageID});
    return;
}

#========================================================================================
# function     clone
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Clone = $Object->clone( $hInfo );
# syntax       $Clone = $Object->clone( $hInfo, $Recursive );
#----------------------------------------------------------------------------------------
# description  Inserts a new object into the database as copy from old object. This
#               function copies also tax rates of the matrix, which are no objects.
#----------------------------------------------------------------------------------------
# input        $hInfo | hash with mandatory attribute values.
#                       <ul>
#                         <li>ClassID - class id - int</li>
#                         <li>Alias - alias - varchar(255)</li>
#                         <li>ParentID - parent objectid - int</li>
#                         <li>Inherit - (optional) inherit permissions from parent object - boolean</li>
#                       </ul> | hash ref
# input        $Recursive | clone child object too (optional, default is false) | boolean
# return       $Clone | the new object | object
#========================================================================================
sub clone {
    my $self = shift;
    my ($hInfo, $Recursive) = @_;
    $Recursive = 0 unless defined $Recursive;

    my $Clone;
    Transaction( sub {
        $Clone = $self->SUPER::clone($hInfo, $Recursive);

        ### clone tax rates ###
        # map id to alias
        my %SelfTaxClassesByID;
        $SelfTaxClassesByID{$_->id} = $_->alias foreach @{$self->get('TaxClasses')};
        my %SelfTaxAreasByID;
        $SelfTaxAreasByID{$_->id} = $_->alias foreach @{$self->get('TaxAreas')};

        # map aliases to object
        my %CloneTaxClassesByAlias;
        $CloneTaxClassesByAlias{$_->alias} = $_->id foreach @{$Clone->get('TaxClasses')};
        my %CloneTaxAreasByAlias;
        $CloneTaxAreasByAlias{$_->alias} = $_->id foreach @{$Clone->get('TaxAreas')};

        # set defaults and fallback
        $Clone->set({ 'DefaultTaxClass' => $Clone->child($self->defaultTaxClass->alias) });
        $Clone->set({ 'DefaultTaxArea' => $Clone->child($self->defaultTaxArea->alias) });
        $Clone->set({ 'FallbackTaxArea' => $Clone->child($self->fallbackTaxArea->alias) });

        # insert tax rates
        foreach my $TaxAreaID (keys %SelfTaxAreasByID) {
            my $CloneTaxAreaID = $CloneTaxAreasByAlias{$SelfTaxAreasByID{$TaxAreaID}};
            my $aTaxRates = GetInfoTaxRatesByTaxAreaID($TaxAreaID);
            my %TaxRates;
            $TaxRates{$_->{'TaxClassID'}} = $_->{'Rate'} foreach @$aTaxRates;
            foreach my $TaxClassID (keys %SelfTaxClassesByID) {
                my $CloneTaxClassID = $CloneTaxClassesByAlias{$SelfTaxClassesByID{$TaxClassID}};
                $TaxRates{$TaxClassID} = 0 unless defined $TaxRates{$TaxClassID};
                InsertTaxRate({'TaxAreaID'=> $CloneTaxAreaID, 'TaxClassID'=> $CloneTaxClassID, 'Rate'=> $TaxRates{$TaxClassID}});
            }
        }
    });

    return $Clone;
}

#========================================================================================
# function     taxRate
#----------------------------------------------------------------------------------------
# syntax       $TaxRate = $TaxMatrix->taxRate($Area, $Class);
#----------------------------------------------------------------------------------------
# description  Tax rate of matrix defined by tax area and class.
#----------------------------------------------------------------------------------------
# input        $Area | tax area | object
# input        $Class | tax class | object
# return       $TaxRate | percentage of tax (eg 0.16 for 16 %) | float
#========================================================================================
sub taxRate {
    my $self = shift;
    my ($Area, $Class) = @_;
    TestObject('TaxArea', $Area);
    TestObject('TaxClass', $Class);

    if ($Class->parent->id != $self->id or $Area->parent->id != $self->id) {
        my %Vars = ( 'self_TaxMatrixID' => $self->id );
        $Vars{'Class'} = $Class->id;
        $Vars{'Area'}  = $Area->id;
        Error('NOT_SAME_MATRIX', \%Vars);
    }

    return 0 unless ExistsTaxRate($Area->id, $Class->id);
    return GetInfoTaxRate($Area->id, $Class->id)->{'Rate'};
}

#========================================================================================
# function     gross2net
#----------------------------------------------------------------------------------------
# syntax       $NetPrice = $TaxMatrix->gross2net($CurrencyID, $GrossPrice, $hTaxParams);
# example      $NetPrice = $TaxMatrix->gross2net('EUR', 19.99,
#                                          {'TaxModel' => $TaxModel,
#                                           'TaxArea' =>  $TaxArea,
#                                           'TaxClass' => $TaxMatrix->defaultTaxClass, });
#----------------------------------------------------------------------------------------
# description  Calculates the net price from the gross price if required by the given
#               tax model. Uses the tax rate for the given tax class from the default
#               tax area if no area is given. Adds one extra digit of precision
#               for example: 16.798 EUR (unless Precision is passed as parameter).
#----------------------------------------------------------------------------------------
# input        $hTaxParams | hash with tax parameters.
#                       <ul>
#                         <li>TaxModel - tax model           - object</li>
#                         <li>TaxArea  - (optional) tax area - object</li>
#                         <li>TaxClass - tax class           - object</li>
#                         <li>Precision - (optional) number of digits after comma - integer</li>
#                       </ul> | hash ref
# input        $CurrencyID | currency id | object
# input        $GrossPrice | gross price | float
# return       $NetPrice   | net price   | float
#========================================================================================
sub gross2net {
    my $self = shift;
    my ($CurrencyID, $Price, $hParams) = @_;

    TestInteger('TaxModel',$hParams->{'TaxModel'}, 0, 1);
    TestObject('TaxClass', $hParams->{'TaxClass'});
    TestObject('TaxArea',  $hParams->{'TaxArea'}) if defined $hParams->{'TaxArea'};
    TestString('CurrencyID', $CurrencyID , 3, 3);
    TestFloat('Price', $Price);
    TestInteger('Precision', $hParams->{'Precision'}) if defined $hParams->{'Precision'};

    if ($hParams->{'TaxModel'} == TAX_MODEL_GROSS) {
        my $TaxArea = $hParams->{'TaxArea'} ? $hParams->{'TaxArea'} : $self->defaultTaxArea;
        my $DefaultTaxRate = $self->taxRate($TaxArea, $hParams->{'TaxClass'});
        $Price = $Price / (1 + $DefaultTaxRate);
    }

    my $Formatter = DE_EPAGES::Core::API::Object::NumberFormat->new();
    my $Precision = $hParams->{'Precision'} // (GetInfoCurrency($CurrencyID)->{'MinorUnit'} + 1);
    return $Formatter->round($Price, $Precision);
}

#========================================================================================
# function     net2gross
#----------------------------------------------------------------------------------------
# syntax       $GrossPrice = $TaxMatrix->net2gross($CurrencyID, $NetPrice,$hTaxParams,$ExtraPrecision);
# example      $GrossPrice = $TaxMatrix->net2gross('EUR', 16.798,
#                                          {'TaxModel' => $TaxModel,
#                                           'TaxArea' =>  $TaxArea,
#                                           'TaxClass' => $TaxMatrix->defaultTaxClass, },1);
#----------------------------------------------------------------------------------------
# description  Calculates the gross price from the net price if required by the given
#               tax model. Uses the tax rate for the given tax class from the default
#               tax area if no tax area is given.
#               Rounds the result to the standard precision of the currency, for example: 19.99 EUR.
#----------------------------------------------------------------------------------------
# input        $hTaxParams | hash with tax parameters.
#                       <ul>
#                         <li>TaxModel - tax model           - object</li>
#                         <li>TaxArea  - (optional) tax area - object</li>
#                         <li>TaxClass - tax class           - object</li>
#                         <li>Precision - (optional) number of digits after comma - integer</li>
#                       </ul> | hash ref
# input        $CurrencyID | currency id | object
# input        $NetPrice | net price | float
# input        $ExtraPrecision | extra precision | int
# return       $GrossPrice | gross price | float
#========================================================================================
sub net2gross {
    my $self = shift;
    my ($CurrencyID, $Price, $hParams, $ExtraPrecision) = @_;

    TestInteger('TaxModel', $hParams->{'TaxModel'}, 0, 1);
    TestObject('TaxClass', $hParams->{'TaxClass'});
    TestObject('TaxArea',  $hParams->{'TaxArea'}) if defined $hParams->{'TaxArea'} ;
    TestString('CurrencyID', $CurrencyID , 3, 3);
    TestFloat('Price', $Price);
    TestInteger('Precision', $hParams->{'Precision'}) if defined $hParams->{'Precision'};

    my $MinorUnit = GetInfoCurrency($CurrencyID)->{'MinorUnit'};
    if ($hParams->{'TaxModel'} == TAX_MODEL_GROSS) {
        my $TaxArea = $hParams->{'TaxArea'} ? $hParams->{'TaxArea'} : $self->defaultTaxArea;
        my $DefaultTaxRate = $self->taxRate($TaxArea, $hParams->{'TaxClass'});
        $Price = $Price * (1 + $DefaultTaxRate);
    } else {
        $MinorUnit++ if $ExtraPrecision;
    }

    my $Formatter = DE_EPAGES::Core::API::Object::NumberFormat->new();
    my $Precision = $hParams->{'Precision'} // $MinorUnit;
    return $Formatter->round($Price, $Precision);
}

#========================================================================================
# function     defaultTaxArea
#----------------------------------------------------------------------------------------
# syntax       $TaxArea = $TaxMatrix->defaultTaxArea;
#----------------------------------------------------------------------------------------
# description  Gets the default tax area of the matrix
#----------------------------------------------------------------------------------------
# return       $Area | tax area | object
#========================================================================================
sub defaultTaxArea {
    my $self = shift;

    my $DefaultTaxArea = $self->get('DefaultTaxArea');
    Error('NO_DEFAULT_TAX_AREA', {
        ObjectID => $self->id,
        Alias => $self->alias
    }) unless defined $DefaultTaxArea;

    return $DefaultTaxArea;
}

#========================================================================================
# function     fallbackTaxArea
#----------------------------------------------------------------------------------------
# syntax       $TaxArea = $TaxMatrix->fallbackTaxArea;
#----------------------------------------------------------------------------------------
# description  Gets the default tax area of the matrix for undefined countries
#----------------------------------------------------------------------------------------
# return       $Area | tax area | object
#========================================================================================
sub fallbackTaxArea {
    my $self = shift;

    my $FallbackTaxArea = $self->get('FallbackTaxArea');
    Error('NO_FALLBACK_TAX_AREA', {
        ObjectID => $self->id,
        Alias => $self->alias
    }) unless defined $FallbackTaxArea;

    return $FallbackTaxArea;
}

#========================================================================================
# function     defaultTaxClass
#----------------------------------------------------------------------------------------
# syntax       $TaxClass = $TaxMatrix->defaultTaxClass;
#----------------------------------------------------------------------------------------
# description  Gets the default tax class of the matrix
#----------------------------------------------------------------------------------------
# return       $TaxClass | tax class | object
#========================================================================================
sub defaultTaxClass {
    my $self = shift;

    my $DefaultTaxClass = $self->get('DefaultTaxClass');
    Error('NO_DEFAULT_TAX_CLASS', {
        ObjectID => $self->id,
        Alias => $self->alias
    }) unless defined $DefaultTaxClass;

    return $DefaultTaxClass;
}

sub exportXML {
    my $self = shift;
    my ($Driver) = @_;

    $self->SUPER::exportXML($Driver);

    my $Site = $self->getSite();
    if ($Site->class->existsAttribute('TaxMatrix')) {
        my $SiteMatrix = $Site->get('TaxMatrix');
        if (defined $SiteMatrix and $SiteMatrix->id == $self->id) {
            my $XMLParentObject = $Driver->topID('XMLParentObject');
            $Driver->writeElement( $Site->class->alias, {
                'Path' => $XMLParentObject->relativePath($Site),
                'TaxMatrix' => $XMLParentObject->relativePath($SiteMatrix),
                'reference' => 1,
            });
        }
    }
    return;
}

sub exportXMLElements {
    my $self = shift;
    my ($Driver, $allAttributes) = @_;

    $self->SUPER::exportXMLElements($Driver, $allAttributes);

    # add TaxRate elements
    if( $Driver->exportLevel == 1 ) {
        my $aTaxAreas = $self->get('TaxAreas');
        my $aTaxClases = $self->get('TaxClasses');
        foreach my $TaxArea (@$aTaxAreas) {
            foreach my $TaxClass (@$aTaxClases) {
                my $TaxRate = $self->taxRate($TaxArea, $TaxClass);
                if( defined $TaxRate ) {
                    $Driver->writeElement( 'TaxRate', {
                        'TaxClass' => $TaxClass->alias,
                        'TaxArea'  => $TaxArea->alias,
                        'Rate'     => $TaxRate,
                    });
                }
            }
        }

        #export DefaultTaxClass
        if ($self->get('DefaultTaxClass')){
            $Driver->writeElement( 'TaxMatrixValue', {
                'Name' => 'DefaultTaxClass',
                'Value'  => $self->get('DefaultTaxClass')->alias,
            });
        }

        #export DefaultTaxArea
        if ($self->get('DefaultTaxArea')){
            $Driver->writeElement( 'TaxMatrixValue', {
                'Name' => 'DefaultTaxArea',
                'Value'  => $self->get('DefaultTaxArea')->alias,
            });
        }

        #export FallbackTaxArea
        if ($self->get('FallbackTaxArea')){
            $Driver->writeElement( 'TaxMatrixValue', {
                'Name' => 'FallbackTaxArea',
                'Value'  => $self->get('FallbackTaxArea')->alias,
            });
        }
    }
    return;
}




#========================================================================================
# function     taxAreaByCountry
# state        public
#----------------------------------------------------------------------------------------
# syntax       $TaxArea = $TaxMatrix->taxAreaByCountry($CountryID);
# syntax       $TaxArea = $TaxMatrix->taxAreaByCountry($CountryID, $Zipcode);
# syntax       $TaxArea = $TaxMatrix->taxAreaByCountry($CountryID, $Zipcode, $ZipcodeTo);
#----------------------------------------------------------------------------------------
# description  Returns the TaxArea by country and zip code.
#----------------------------------------------------------------------------------------
# input        $CountryID     | country id    | int
# input        $Zipcode       | zipcode (optional) | string
# input        $ZipcodeTo     | max zipcode (optional) | string
# return       $TaxArea       | tax area      | object
#========================================================================================
sub taxAreaByCountry {
    my $self = shift;
    my ($CountryID, $Zipcode, $ZipcodeTo) = @_;
    TestInteger('CountryID', $CountryID);
    TestString('Zipcode', $Zipcode) if $Zipcode;
    TestString('ZipcodeTo', $ZipcodeTo) if $ZipcodeTo;
    my $TaxAreaID;
    my $FallbackTaxAreaID;

    foreach my $hTaxAreaCountry (@{GetInfoTaxAreaCountriesByTaxMatrixID($self->id, $CountryID)}) {

        if ($Zipcode && $ZipcodeTo) {
            $TaxAreaID = $hTaxAreaCountry->{'TaxAreaID'}
                if defined $hTaxAreaCountry->{'ZipcodeFrom'}
                    && (CompareZipCodes($hTaxAreaCountry->{'ZipcodeFrom'},$Zipcode) == 0)
                    && (CompareZipCodes($hTaxAreaCountry->{'ZipcodeTo'},$ZipcodeTo) == 0);
        } elsif ($Zipcode) {
            $TaxAreaID = $hTaxAreaCountry->{'TaxAreaID'}
                if defined $hTaxAreaCountry->{'ZipcodeFrom'}
                    && (CompareZipCodes($hTaxAreaCountry->{'ZipcodeFrom'},$Zipcode) != 1)
                    && (CompareZipCodes($hTaxAreaCountry->{'ZipcodeTo'},$Zipcode) != -1)
        } else {
            $TaxAreaID = $hTaxAreaCountry->{'TaxAreaID'}
                if !defined $hTaxAreaCountry->{'ZipcodeFrom'}
                    && !defined $hTaxAreaCountry->{'ZipcodeTo'};
        }
        $FallbackTaxAreaID = $hTaxAreaCountry->{'TaxAreaID'}
            if !defined $hTaxAreaCountry->{'ZipcodeFrom'}
                && !defined $hTaxAreaCountry->{'ZipcodeTo'};
    }
    $TaxAreaID = $FallbackTaxAreaID unless $TaxAreaID;

    my $TaxArea;
    if ($TaxAreaID) {
        $TaxArea = LoadObject($TaxAreaID);
    } else {
        $TaxArea = $self->fallbackTaxArea();
    }

    return $TaxArea;
}

#========================================================================================
# function     taxAreaCountries
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ahTaxAreaCountries = $TaxMatrix->taxAreaCountries();
#----------------------------------------------------------------------------------------
# description  Returns objects of TaxAreaCountry by TaxMatrix.
#----------------------------------------------------------------------------------------
# return       $ahTaxAreaCountries | with following keys:
#               <ul>
#                   <li>TaxMatrixID   - tax matrix id        (part of primary key)- int
#                   <li>CountryID     - country id           (part of primary key)- int
#                   <li>TaxAreaID     - tax area id          - int
#               </ul> | reference to array of hashes
#========================================================================================
sub taxAreaCountries {
    my $self = shift;
    return GetInfoTaxAreaCountriesByTaxMatrixID($self->id);
}

#========================================================================================
# function     removeTaxAreaCountry
# state        public
#----------------------------------------------------------------------------------------
# syntax       $TaxMatrix->removeTaxAreaCountry($TaxAreaCountryID);
#----------------------------------------------------------------------------------------
# description  Removes a TaxAreaCountryID.
#----------------------------------------------------------------------------------------
# input        $TaxAreaCountryID | TaxAreaCountry identifier | int
#========================================================================================
sub removeTaxAreaCountry {
    my $self = shift;
    my ($TaxAreaCountryID) = @_;
    TestInteger( 'TaxAreaCountryID', $TaxAreaCountryID);
    DeleteTaxAreaCountry( $TaxAreaCountryID );
}

#========================================================================================
# function     updateTaxAreaCountry
# state        public
#----------------------------------------------------------------------------------------
# syntax       $TaxMatrix->updateTaxAreaCountry($TaxAreaCountryID, $hTaxAreaCountry);
#----------------------------------------------------------------------------------------
# description  Updates TaxAreaCountry.
#----------------------------------------------------------------------------------------
# input        $TaxAreaCountryID | TaxAreaCountry identifier | int
# input        $hTaxAreaCountry | with following keys:
#               <ul>
#                   <li>TaxMatrixID   - tax matrix id        - int
#                   <li>CountryID     - country id           - int
#                   <li>TaxAreaID     - tax area id          - int
#                   <li>ZipcodeFrom   - min zipcode          (optional)- varchar(25)
#                   <li>ZipcodeTo     - max zipcode          (optional)- varchar(25)
#               </ul> | reference to hash
#========================================================================================
sub updateTaxAreaCountry {
    my $self = shift;
    my ($TaxAreaCountryID, $hTaxAreaCountry) = @_;
    TestInteger( 'TaxAreaCountryID', $TaxAreaCountryID);
    UpdateTaxAreaCountry($TaxAreaCountryID, $hTaxAreaCountry);
}

#========================================================================================
# function     default
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = DE_EPAGES::Tax::API::Object::TaxMatrix->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;
    return {
        %{$class->SUPER::default($Parent, $hInfo)},
    };
}

#========================================================================================
# function     getTaxMatrixTaxClassByAlias
#----------------------------------------------------------------------------------------
# syntax       $TaxClass = $TaxMatrix->getTaxMatrixTaxClassByAlias($TaxClassAlias);
#----------------------------------------------------------------------------------------
# description  Finds the correct TaxClass by the given alias.
#               Fallback is "reduced" then "normal" if the alias was not found
#----------------------------------------------------------------------------------------
# input        $TaxClassAlias | alias of tax class | string
# return       $TaxClass | tax class | object
#========================================================================================
sub getTaxMatrixTaxClassByAlias {
    my $self = shift;
    my ($TaxClassAlias) = @_;

    return $self->child($TaxClassAlias) if $self->existsChild($TaxClassAlias);

    # use a fallback for the taxarea
    # if the fallback not exists we use the default tax area for EU
    my $FallbackAlias = 'reduced';
    my $DefaultAlias  = 'normal';

    $FallbackAlias = $self->existsChild($FallbackAlias) ? $FallbackAlias : $DefaultAlias;
    return $self->child($FallbackAlias);

}

1;
