#========================================================================================
# package      DE_EPAGES::Tax::API::Shop
# state        public
# description  This module includes a function to get shop tax method
#========================================================================================
package DE_EPAGES::Tax::API::Shop;
use base qw( Exporter );
use strict;

use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle );
use DE_EPAGES::Tax::API::Constants qw ($SHOPWIZARD_TAXMATRIX_MAPPING NO_SPLIT_SHIPPING_TAX NO_SPLIT_PAYMENT_TAX);
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

our @EXPORT_OK = qw (
    GetTaxMethodsFolder
    DisableShopTaxMethod
    EnableShopTaxMethod
    ShouldSplitShippingAndPaymentTax
    TaxMatrixByEUAddress
);

sub ErrorDefinition {
    return {
            'TaxMatrixNotCustomized' => 'TaxMatrix not customized. Shop: #ShopAlias, TaxMatrix: #TaxMatrixPath, Store: #Store',
            'TaxMethodNotUsableAtStorefront' => 'TaxMethod #TaxMethod is not usable at storefront. Store: #Store',
    };
}

#========================================================================================
# function     GetTaxMethodsFolder
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Folder = GetTaxMethodsFolder( $Shop );
#----------------------------------------------------------------------------------------
# description  Returns the folder of tax methods
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# return       $Folder | folder object for tax methods | object
#========================================================================================
sub GetTaxMethodsFolder {
    my ($Shop) = @_;
    TestObject('Shop',$Shop,'DE_EPAGES::Shop::API::Object::Shop');
    return $Shop->folder('TaxMethods', {'Position'=>'110'});
}

#========================================================================================
# function     DisableShopTaxMethod
# state        public
#----------------------------------------------------------------------------------------
# syntax       DisableShopTaxMethod( $Shop );
#----------------------------------------------------------------------------------------
# description  Resets the TaxMethod on the shop to undef.
#----------------------------------------------------------------------------------------
# input        $Shop | Shop | object
# hook         API_DisableShopTaxMethod | hook parameter keys :
#               <ul>
#                 <li>TaxMethod   - tax method          - object
#                 <li>Shop        - shop                - object
#               </ul>
# return       none
#========================================================================================
sub DisableShopTaxMethod {
    my ($Shop) = @_;
    TestObject('Shop',$Shop,'DE_EPAGES::Shop::API::Object::Shop');
    my $TaxMethod = $Shop->get('TaxMethod');
    return unless defined $TaxMethod;
    $Shop->set({'TaxMethod' => undef });

    TriggerHook('API_DisableShopTaxMethod', {'TaxMethod' => $TaxMethod, 'Shop' => $Shop });
    return;
}

#========================================================================================
# function     EnableShopTaxMethod
# state        public
#----------------------------------------------------------------------------------------
# syntax       EnableShopTaxMethod( $Shop, $TaxMethod );
#----------------------------------------------------------------------------------------
# description  Sets the given TaxMethod on the shop. This can only be done if a
#               customized TaxMatrix is used. Throws TaxMatrixNotCustomized error, if
#               Matrix is not customized.
#----------------------------------------------------------------------------------------
# input        $Shop | Shop | object
# input        $TaxMethod | tax method | object
# input        $TaxMatrix | tax matrix | object (optional)
# hook         API_EnableShopTaxMethod | hook parameter keys :
#               <ul>
#                 <li>TaxMethod   - tax method          - object
#                 <li>Shop        - shop                - object
#               </ul>
# return       none
#========================================================================================
sub EnableShopTaxMethod {
    my ($Shop, $TaxMethod, $TaxMatrix) = @_;
    TestObject('Shop',$Shop,'DE_EPAGES::Shop::API::Object::Shop');
    TestObject('TaxMethod',$TaxMethod,'DE_EPAGES::Tax::API::Object::TaxMethod');

    TriggerHook('API_BeforeEnableShopTaxMethod', {'TaxMethod' => $TaxMethod, 'Shop' => $Shop });

    $TaxMatrix = $Shop->get('TaxMatrix') unless defined $TaxMatrix;

    if (! $TaxMethod->usableAtStorefront()) {
        Error('TaxMethodNotUsableAtStorefront',{
            'TaxMethod'     => $TaxMethod->get('Path'),
            'Store'         => GetCurrentDBHandle()->cacheKey,
        });
    }

    if ($TaxMatrix->parent->id != $Shop->id) {
        Error('TaxMatrixNotCustomized',{
            'Shop'          => $Shop,
            'ShopAlias'     => $Shop->alias,
            'TaxMatrixPath' => $TaxMatrix->get('Path'),
            'Store'         => GetCurrentDBHandle()->cacheKey,
        })
    }
    $Shop->set({'TaxMethod' => $TaxMethod});

    TriggerHook('API_EnableShopTaxMethod', {'TaxMethod' => $TaxMethod, 'Shop' => $Shop });

    #correct the LineItemClass
    $Shop->get('TaxMatrix')->set({'LineItemClass' => $TaxMethod->get('TaxType')->get('LineItemClass')});

    return;
}

#========================================================================================
# function     ShouldSplitShippingAndPaymentTax
# state        public
#----------------------------------------------------------------------------------------
# syntax       my ($SplitShippingTaxes, $SplitPaymentTaxes) = @{ ShouldSplitShippingAndPaymentTax($Shop) };
#----------------------------------------------------------------------------------------
# description  Returns, if the taxes for shipping and payment should be splitted
#               for the given shop (address)
#               If the delivery threshold regulation is active we test the shipping country
#               from the container and make then the decision if we split or not
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# input        $Container | (optional) lineitem container | object
# return       $SplitShippingTaxes | split shipping taxes | boolean
# return       $SplitPaymentTaxes  | split payment taxes  | boolean
#========================================================================================
sub ShouldSplitShippingAndPaymentTax {
    my ($Shop, $Container) = @_;

    my $hInfoShop = $Shop->get([qw(Address DeliveryThresholdIsActive)]);
    my $TargetCountry = defined $hInfoShop->{'Address'} ? $hInfoShop->{'Address'}->get('Country') : undef;

    # test if we have a container and an active delivery threshold regulation
    # then we set the new target country to find out if we need splitting of payment and shipping
    if (defined $TargetCountry and defined $Container and $hInfoShop->{'DeliveryThresholdIsActive'}) {

        my $ShippingAddress = $Container->parent->get('ShippingAddress');

        unless (defined $ShippingAddress) {
            my $AttributeName = ($Container->parent->instanceOf('CustomerOrder') or
                                 $Container->parent->instanceOf('Basket')) ? 'BillingAddress' : 'Address' ;
            $ShippingAddress = $Container->parent->get($AttributeName);
        }

        $TargetCountry = $ShippingAddress->get('Country') if defined $ShippingAddress;
    }

    # EPG-18579 Pro-rata tax for delivery costs doesn't apply for France
    # split taxes on shipping if we are not in a country of the list NO_SPLIT_SHIPPING_TAX
    # shipping in @NO_SPLIT_SHIPPING_TAX do have the normal tax rate also if the product tax rate is reduced
    # that means, if we have a book with e.g. 7% tax we have to use 19% tax for the shipping method
    my $SplitShippingTaxes = (defined $TargetCountry and ElementOf($TargetCountry->{'Code2'},
        NO_SPLIT_SHIPPING_TAX)) ? 0 : 1;

    # EPG-20021 Pro-rata VAT for payment costs
    # in some countries like "FR GB ES" taxes for payment are not splitted (defined in: NO_SPLIT_PAYMENT_TAX)
    my $SplitPaymentTaxes = (defined $TargetCountry and ElementOf($TargetCountry->{'Code2'},
        NO_SPLIT_PAYMENT_TAX)) ? 0 : 1;

    return [$SplitShippingTaxes, $SplitPaymentTaxes];
}

#========================================================================================
# function     TaxMatrixByEUAddress
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $TaxMatrix = TaxMatrixByEUAddress($ShippingAddress);
#----------------------------------------------------------------------------------------
# description  Returns the TaxMatrix for the given shipping address country if a TaxMatrix
#               exists for the country on the system object
#----------------------------------------------------------------------------------------
# input        $ShippingAddress | address object | object
# return       $TaxMatrix  | tax matrix object  | object
#========================================================================================
sub TaxMatrixByEUAddress {
    my ($ShippingAddress) = @_;

    # test if we have a TaxMatrix for the Shipping Country
    my $EnglishCountry = $ShippingAddress->get('Country')->{'English'};
    if (defined $SHOPWIZARD_TAXMATRIX_MAPPING->{$EnglishCountry}) {
        $EnglishCountry = $SHOPWIZARD_TAXMATRIX_MAPPING->{$EnglishCountry};
    }

    my $TaxMatrixName = 'TaxMatrix' . $EnglishCountry;

    my $System = LoadRootObject();
    return $System->existsChild($TaxMatrixName) ? $System->child($TaxMatrixName) : undef;

}

1;
