#========================================================================================
# package      DE_EPAGES::Zettle::API::TokenHelper
# state        public
#----------------------------------------------------------------------------------------
# description  handles token-related (access token/refresh token) behaviour
#========================================================================================
package DE_EPAGES::Zettle::API::TokenHelper;

use strict;

use DateTime;
use DateTime::Duration;
use Readonly;

use DE_EPAGES::Core::API::PerlTools qw ( Compare GetCurrentDateTime isValid );
use DE_EPAGES::Core::API::TestParameter  qw( TestObject );

# Names of the token fields in Zettle API response
Readonly my $ACCESS_TOKEN_API_KEY => 'access_token';
Readonly my $REFRESH_TOKEN_API_KEY => 'refresh_token';
Readonly my $ACCESS_TOKEN_EXPIRATION_API_KEY => 'expires_in';


#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::Zettle::API::TokenHelper->new($Environment);
#----------------------------------------------------------------------------------------
# description  constructor
#----------------------------------------------------------------------------------------
# input        $Environment | the Zettle environment object | DE_EPAGES::Zettle::API::Object::ZettleEnvironment
#----------------------------------------------------------------------------------------
# return       $self | self | DE_EPAGES::Zettle::API::TokenHelper
#========================================================================================
sub new {
    my $class = shift;
    my ($Environment)  = @_;
    TestObject('ZettleEnvironment', $Environment, 'DE_EPAGES::Zettle::API::Object::ZettleEnvironment');

    my $self  = bless {
        'Environment' => $Environment,
    }, $class;

    return $self;
}

#========================================================================================
# function     isTokenAcquired
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->isTokenAcquired();
#----------------------------------------------------------------------------------------
# description  test if tokens were acquired from Zettle
#----------------------------------------------------------------------------------------
# return       return TRUE if AccessToken and RefreshToken are present in Zettle environment, false otherwise
#========================================================================================
sub isTokenAcquired {
    my $self = shift;
    
    my $Environment = $self->_getEnvironment();
    my $AccessToken = $Environment->accessToken();
    my $RefreshToken = $Environment->refreshToken();
    
    return isValid($AccessToken) && isValid($RefreshToken);
}

#========================================================================================
# function     isTokenExpired
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->isTokenExpired();
#----------------------------------------------------------------------------------------
# description  test if zettle Access token is expired
#----------------------------------------------------------------------------------------
# return       return TRUE if AccessToken is expired
#========================================================================================
sub isTokenExpired {
    my $self = shift;
    
    my $Environment = $self->_getEnvironment();
    my $TokenExpiration = $Environment->accessTokenExpiration();
    return (defined $TokenExpiration && (Compare('DateTime',GetCurrentDateTime(),$TokenExpiration) >= 0)) ? 1 : 0;
}

#========================================================================================
# function     updateTokenData
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->updateTokenData($hTokenData);
#----------------------------------------------------------------------------------------
# description  update token information in Zettle environment
#----------------------------------------------------------------------------------------
# input        $hTokenData | Zettle API Tokens response with following keys:
#               <ul>
#                   <li>access_token - access token for API calls - string
#                   <li>refresh_token - refresh token for refreshing access token - string
#                   <li>expires_in - time to access_token expiration (seconds) - integer
#               </ul> | ref.hash
#========================================================================================
sub updateTokenData {
    my $self = shift;
    my ($hTokenData) = @_;
    
    my $ExpirationDateTime;
    my $ExpiresIn = $hTokenData->{$ACCESS_TOKEN_EXPIRATION_API_KEY};
    # NOTE: Substract 10 seconds to expiration date as a safety margin
    $ExpiresIn -= 10;
    $ExpirationDateTime = GetCurrentDateTime() + DateTime::Duration->new(seconds => $ExpiresIn) if defined $ExpiresIn;

    my $Environment = $self->_getEnvironment();
    $Environment->updateTokenData(
                    $hTokenData->{$ACCESS_TOKEN_API_KEY},
                    $hTokenData->{$REFRESH_TOKEN_API_KEY},
                    $ExpirationDateTime);
}

#========================================================================================
# function     deleteTokenData
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->deleteTokenData();
#----------------------------------------------------------------------------------------
# description  delete token information from environment
#========================================================================================
sub deleteTokenData {
    my $self = shift;

    my $Environment = $self->_getEnvironment();
    $Environment->updateTokenData(undef, undef, undef);
}


#========================================================================================


#========================================================================================
# function     _getEnvironment
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Environment = $self->_getEnvironment();
#----------------------------------------------------------------------------------------
# description  Returns the Zettle Environment.
#----------------------------------------------------------------------------------------
# return       $Environment | Zettle Environment | DE_EPAGES::Zettle::API::Object::ZettleEnvironment
#========================================================================================
sub _getEnvironment {
    return shift->{'Environment'};
}

1;