#========================================================================================
# package      DE_EPAGES::Zettle::API::Tools
# state        public
#========================================================================================
package DE_EPAGES::Zettle::API::Tools;
use base Exporter;

use strict;

use DateTime::Format::ISO8601;
use Time::HiRes qw ( time );

use DE_EPAGES::Core::API::PerlTools qw ( GetIPAddress Ip2Hex );

our @EXPORT_OK = qw (
    GenerateUUID
    ParseDateTime
);

#========================================================================================
# function     GenerateUUID
# state        public
#----------------------------------------------------------------------------------------
# syntax       $GUID = GenerateUUID();
#----------------------------------------------------------------------------------------
# description  Returns 36-character long strings containing UUIDv1. These look something like:
#               2091a200-1dd2-11b2-8000-080027b246c3.
#               See RFC: https://datatracker.ietf.org/doc/html/rfc4122#section-4
#               More info: https://www.uuidtools.com/uuid-versions-explained
#----------------------------------------------------------------------------------------
# return       $UUID | general unique identifier | string
#========================================================================================
sub GenerateUUID () {
    ## no critic qw(ProhibitMagicNumbers)
    my ($time_low, $time_mid, $time_hi_and_version, $clock_seq_hi_and_reserved, $clock_seq_low)
        = (time, rand(256*256), rand(256*256), rand(256), rand(256));
    my $uuid = sprintf ( '%8.8x-%4.4x-%4.4x-%2.2x%2.2x-%s%2.2x%2.2x', $time_low, $time_mid,
                     $time_hi_and_version, $clock_seq_hi_and_reserved, $clock_seq_low,
                     lc(Ip2Hex(GetIPAddress(),8)), $$ % 256, rand(256) % 256 );
    # set UUID version (v1)
    substr($uuid, 14, 1, '1');
    # set UUID variant
    substr($uuid, 19, 1, '2');
    # force MAC multicast bit on per RFC
    my $multicast = substr($uuid, 25, 1);
    my $multicast_dec = hex($multicast);
    if(!($multicast_dec & 1)){
        $multicast = sprintf('%1.1x', $multicast_dec+1);
        substr($uuid, 25, 1, $multicast);
    }
    return $uuid;
    ## use critic
}

#========================================================================================
# function     ParseDateTime
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DateTime = ParseDateTime($DTString);
# syntax       $DateTime = ParseDateTime($DTString, $TimeZone);
#----------------------------------------------------------------------------------------
# description  Parse the string representation of a date in ISO8601 format and return a DateTime object.
#----------------------------------------------------------------------------------------
# input        $DTString | a string representation of a datetime (sample: '2021-05-31T13:48:49.978+0000') | string
# input        $TimeZone | optional. Timezone that will be applied to resulting DateTime (default: 'Europe/Berlin') | string
#----------------------------------------------------------------------------------------
# return       $DateTime | a DateTime object | DateTime
#========================================================================================
sub ParseDateTime {
    my ($DTString, $TimeZone) = @_;

    $TimeZone //= 'Europe/Berlin';

    # Reference: https://metacpan.org/pod/DateTime::Format::ISO8601#SYNOPSIS
    my $dt = DateTime::Format::ISO8601->parse_datetime($DTString);
    # Reference: https://metacpan.org/pod/DateTime#$dt-%3Eset_time_zone($tz)
    $dt->set_time_zone($TimeZone);

    return $dt;
}

1;