#========================================================================================
# package      DE_EPAGES::Zettle::API::WebService::DTO::DTOFactory
# state        public
#----------------------------------------------------------------------------------------
# description  Factory for creating different DTO objects.
#========================================================================================
package DE_EPAGES::Zettle::API::WebService::DTO::DTOFactory;

use strict;

use DE_EPAGES::Core::API::TestParameter  qw( TestObject );
use DE_EPAGES::Object::API::Table::Country qw ( GetPKeyCountryByCode2 );
use DE_EPAGES::Zettle::API::Tools qw ( GenerateUUID );
use DE_EPAGES::Zettle::API::WebService::DTO::ProductDTO;
use DE_EPAGES::Zettle::API::WebService::DTO::VariationDTO;
use DE_EPAGES::Zettle::API::WebService::VariationOptionHelper qw ( BuildOptionDefinitions BuildVariantOptions );

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::Zettle::API::WebService::DTO::DTOFactory->new($Shop);
#----------------------------------------------------------------------------------------
# description  constructor
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | DE_EPAGES::Shop::API::Object::Shop
#----------------------------------------------------------------------------------------
# return       $self | self | DE_EPAGES::Zettle::API::WebService::DTO::DTOFactory
#========================================================================================
sub new {
    my $class = shift;
    my ($Shop) = @_;
    TestObject('Shop', $Shop, 'DE_EPAGES::Shop::API::Object::Shop');

    my $self = bless {
        'Shop' => $Shop,
        'Environment' => $Shop->get('ZettleEnvironment'),
    }, $class;
    
    return $self;
}

#========================================================================================
# function     buildProductDTO
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->buildProductDTO( $Product, $IncludePrice );
#----------------------------------------------------------------------------------------
# description  build product DTO required for Zettle updates based on the given master product
#----------------------------------------------------------------------------------------
# input        $Product | a master product | DE_EPAGES::Product::API::Object::Product
# input        $IncludePrice | determines whether price information will be included (optional) | boolean
#========================================================================================
sub buildProductDTO {
    my $self = shift;
    my ($Product, $IncludePrice) = @_;
    TestObject( 'Product', $Product, 'DE_EPAGES::Product::API::Object::Product' );    

    my $Shop = $self->_getSite();
    my $LanguageID = $Shop->get('LanguageID');
    my $Environment = $self->_getEnvironment();
    my $ZettleCurrencyCode = $Environment->getMerchantCurrency();
    my $ZettleCountryCode = $Environment->getMerchantCountry();
    
    #TODO: name requirements are associated to product API and shall be applied when generating hash from DTO (ProductDTO::toHash()).
    # my $ProductName = $self->_getProductName($Product, $LanguageID);
    my $ProductName = $Product->get('Name', $LanguageID);
    #TODO: description requirements are associated to product API and shall be applied when generating hash from DTO (ProductDTO::toHash()).
    # my $ProductDescription = $self->_getProductDescription($Product, $LanguageID);
    my $ProductDescription = $Product->get('Description', $LanguageID);
    my $ProductVariations = $self->_buildProductVariations($Product, $IncludePrice, $LanguageID, $ZettleCurrencyCode);
    #TODO: implement ImageService and use it to upload product images
    # my $ProductImagePath = $self->_getProductImagePath($Product);
    #TODO: check that price is only extracted from variations, not from master product
    # my $ProductPrice = $self->_getProductPrice($Product, $ZettleCurrencyCode);
    #TODO: confirm that we should use ZettleCountryCode (what if shop country does not match zettle country?)
    my $VatPercentage = $self->_getVatPercentage($Product, $ZettleCountryCode);
    my $VariantOptionDefinitions = BuildOptionDefinitions($Product);

    my $ProductDTO = DE_EPAGES::Zettle::API::WebService::DTO::ProductDTO->new();
    my $Uuid = $self->_getProductUUID($Product);
    $ProductDTO->uuid($Uuid);
    # $ProductDTO->categories($MainCategory);
    $ProductDTO->name($ProductName);
    $ProductDTO->description($ProductDescription);
    #TODO: upload images using ImageService and pass in LookupKeys here
    # $ProductDTO->imageLookupKeys($ProductImageKeys);
    # $ProductDTO->presentation($ProductPresentation);
    $ProductDTO->variations($ProductVariations);
    $ProductDTO->externalReference($Product->guid);
    # $ProductDTO->unitName($Product->get('Unit'));???
    $ProductDTO->vatPercentage($VatPercentage);
    # $ProductDTO->online($Online);
    $ProductDTO->variantOptionDefinitions($VariantOptionDefinitions);
    # $ProductDTO->taxCode($TaxCode);
    # $ProductDTO->category($Category);
    # $ProductDTO->taxRates($TaxRates);
    # $ProductDTO->taxExempt($TaxExempt);

    return $ProductDTO;
}

sub _buildProductVariations {
    my $self = shift;
    my ($Product, $IncludePrice, $LanguageID, $CurrencyID) = @_;

    my @aZettleVariations = ();

    my $aSubProducts = $Product->get('SubProducts');
    foreach my $SubProduct (@$aSubProducts) {
        my $Uuid = $self->_getProductUUID($SubProduct);
        my $hAttributes;
        $hAttributes = $SubProduct->get(['Name', 'Description'], $LanguageID);
        my $Name = $hAttributes->{'Name'};
        my $Description = $hAttributes->{'Description'};
        $hAttributes = $SubProduct->get(['ManufacturerSKU', 'UPCEAN']);
        my $Sku = $hAttributes->{'ManufacturerSKU'};
        my $Barcode = $hAttributes->{'UPCEAN'};
        my $Options = BuildVariantOptions($SubProduct);

        my $VariationDTO = DE_EPAGES::Zettle::API::WebService::DTO::VariationDTO->new();
        $VariationDTO->uuid($Uuid);
        # We fill the name field for the sake of completeness but when Zettle processes variation it assigns a new name based on variation options values (i.e: blue, M, Cotton)
        $VariationDTO->name($Name) if $Name;
        $VariationDTO->description($Description) if $Description;
        $VariationDTO->sku($Sku) if $Sku;
        $VariationDTO->barcode($Barcode) if $Barcode;
        #TODO: upload images using ImageService and pass in LookupKeys here
        # $VariationDTO->imageLookupKeys($VariationImageKeys);
        # $VariationDTO->presentation($VariationPresentation);
        #TODO: verify price behaviour. Prices are supposed to be optional but in swagger API docs the associated fields are marked as required.
        if($IncludePrice){
            my $Price = $self->_getProductPrice($SubProduct, $CurrencyID);
            #TODO: how to calculate cost price? Is it available in ePages?
            my $CostPrice = 0;
            #TODO: check that CurrencyID is ISO4217 code (char 3).
            my $CurrencyCode = $CurrencyID;
            # NOTE: Price in cents. See DE_EPAGES/Zettle/Documents/API/ZettlePartnerDocumentation-Extracts.md, section 4.5.1.4. Product prices
            $VariationDTO->price($Price*100);
            $VariationDTO->costPrice($CostPrice*100);
            $VariationDTO->currencyCode($CurrencyCode);
            # GetLog()->debug("Zettle::UI::Shop::_buildProductVariations():\n"
            #                 ."\tPrice=".$Price."\n"
            #                 ."\tDumper(Price)=".Devel::Peek::Dump($Price)
            #                 ."\tfloat(Price)=".float($Price)
            #                 ."\t\$VariationDTO->price=".$VariationDTO->price
            #                 ."\tDumper(\$VariationDTO)=".Devel::Peek::Dump($VariationDTO));            
        }
        $VariationDTO->options($Options);
        push(@aZettleVariations, $VariationDTO);
    }

    return \@aZettleVariations;
}

# sub _getProductImagePath {
#     my $self = shift;
#     my ($Product) = @_;
    
#     my $Image = $Product->get('ImageLarge');
#     my $WebPath   = $Product->get('WebPath');
#     my $ImagePath = $Image ? join('/', $WebPath, $Image) : undef;

#     return $ImagePath;
# }

#TODO: confirm that this is the right value (CustomerPriceLists, ManufacturerPrices ...)
sub _getProductPrice {
    my $self = shift;
    my ($Product, $CurrencyID) = @_;

    #TODO: check if it works fine with variations where HasSubOwnPrice=0 (meaning price is stored at super product)
    my $DisplayPrice = $Product->displayPrice({
        CurrencyID => $CurrencyID,
        TaxModel => $Product->getSite()->get('TaxModel'),
    });
    # GetLog()->debug("Zettle::UI::Shop::_getProductPrice(): DisplayPrice=".$DisplayPrice);
    return $DisplayPrice;
}

#========================================================================================
# function     _getVatPercentage
#----------------------------------------------------------------------------------------
# syntax       $VatPercentage = $self->_getVatPercentage($Product, $Country);
#----------------------------------------------------------------------------------------
# description  Returns VAT percentage for $Product applied in $Country.
#----------------------------------------------------------------------------------------
# input        $Product  | a product | DE_EPAGES::Product::API::Product
# input        $Country  | country iso code alpha 2 | char(2)
#----------------------------------------------------------------------------------------
# return       $VatPercentage | VAT applied to $Product in $Country (min: 0, max: 100) | int
#========================================================================================
sub _getVatPercentage {
    my $self = shift;
    my ($Product, $Country) = @_;

    my $CountryID = GetPKeyCountryByCode2($Country); # GB -> 826;
    my $Shop = $Product->getSite();
    my $TaxMatrix = $Shop->get('TaxMatrix');
    my $TaxClass = $Product->get('TaxClass')
                    // $TaxMatrix->defaultTaxClass;
    my $TaxArea = $TaxMatrix->taxAreaByCountry($CountryID)
                    // $TaxMatrix->defaultTaxArea;
    my $TaxRate = $TaxMatrix->taxRate($TaxArea,  $TaxClass);
    # Minimum 0, maximum 100
    my $VatPercentage = $TaxRate*100;
    # GetLog()->debug("Zettle::UI::Shop::_getTaxRate: VatPercentage=$VatPercentage [TaxMatrix=".$TaxMatrix->alias().", TaxClass=".$TaxClass->alias().", TaxArea=".$TaxArea->alias()."]");

    return $VatPercentage;
}

#========================================================================================
# function     _getProductUUID
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Uuid = $self->_getProductUUID($Product);
#----------------------------------------------------------------------------------------
# description  Retrieve Zettle UUID for $Product. If UUID is empty it will be generated and updated.
#----------------------------------------------------------------------------------------
# input        $Product | a (sub) product | DE_EPAGES::Product::API::Object::Product
#----------------------------------------------------------------------------------------
# return       $Uuid | Product UUID used for Zettle | string
#========================================================================================
sub _getProductUUID {
    my $self = shift;
    my ($Product) = @_;

    my $Uuid = $Product->get('ZettleUUID');

    # If product has not been assigned a UUID, generate it and update product attribute
    if(not defined $Uuid){
        $Uuid = GenerateUUID();
        $Product->set({'ZettleUUID' => $Uuid});
    }

    return $Uuid;
}

sub _getSite {
    return shift->{'Shop'};
}

sub _getEnvironment {
    return shift->{'Environment'};
}

1;