#========================================================================================
# package      DE_EPAGES::Zettle::API::WebService::ProductService
# base         DE_EPAGES::Zettle::API::WebService::RestService
# state        private
#----------------------------------------------------------------------------------------
# description  Service wrapper around Zettle Product and Image API
#               This service is based on the methods provided by Product API: https://products.izettle.com/swagger#/
#========================================================================================
package DE_EPAGES::Zettle::API::WebService::ProductService;
use base DE_EPAGES::Zettle::API::WebService::RestService;

use strict;

use HTTP::Status qw( HTTP_NO_CONTENT HTTP_NOT_FOUND );
use JSON;

use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Object::API::Factory qw( LoadRootObject );
use DE_EPAGES::Zettle::API::WebService::DTO::DTOFactory;
use DE_EPAGES::Zettle::API::WebService::DTO::ProductResponseDTO;
use DE_EPAGES::Zettle::API::WebService::DTO::Requirements qw ( IsProductExportable );

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::Zettle::API::WebService::ProductService->new($Environment);
#----------------------------------------------------------------------------------------
# description  creates a new ProductService object
#----------------------------------------------------------------------------------------
# input        $Environment | any Zettle environment object | DE_EPAGES::Object::API::Object:Object
#----------------------------------------------------------------------------------------
# return       $self | self | DE_EPAGES::Zettle::API::WebService::ProductService
#========================================================================================
sub new {
    my $class = shift;
    my ($Environment) = @_;

    my $self = $class->SUPER::new($Environment);
    return $self;
}

#========================================================================================
# function     exportProducts
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hResponse = $self->exportProducts($aProducts, $IncludePrices);
#----------------------------------------------------------------------------------------
# description  Export all product information from the shop to Zettle.
#               Only products that match requirements will be exported: see <function DE_EPAGES::Zettle::API::WebService::DTO::Requirements::IsProductExportable>.
#               This is an asynchronous process so export state shall be requested afterwards
#               See <function DE_EPAGES::Zettle::API::WebService::ProductService::getInitialExportState>.
#               API reference: https://products.izettle.com/swagger#/import/importLibraryV2
#----------------------------------------------------------------------------------------
# input        $aProducts | Products to be exported | ref.array.object
# input        $IncludePrices | Determines whether price information will be exported | boolean
#----------------------------------------------------------------------------------------
# return       $hResponse | with following keys:
#               <ul>
#                   <li>uuid - export identifier (UUIDv1) - string
#                   <li>state - current export state, any of "IMPORTING", "FINISHED_SUCCESS", "FINISHED_FAILED" - string
#                   <li>items - number of items to be processed - integer
#                   <li>created - datetime when export was received by Zettle (ISO8601, i.e: "2021-07-17T18:52:15.885+0000") - string
#                   <li>finished - datetime when export was completed in Zettle (ISO8601, i.e: "2021-07-17T18:52:15.927+0000") - string or null
#                   <li>errorMessage - additional message when errors occur - string
#               </ul> | ref.hash
#========================================================================================
# throws WebserviceError
#    {
#    "developerMessage": "string",
#    "errorType": "string",
#    "violations": [
#        {
#        "propertyName": "string",
#        "developerMessage": "string",
#        "constraintType": "string" (MUST_BE_FALSE, MUST_BE_TRUE, MUST_BE_LESS_THAN, MUST_BE_GREATER_THAN, DIGIT_OUT_OF_BOUNDS, DATE_MUST_BE_IN_THE_FUTURE, MUST_BE_LESS_OR_EQUAL_THAN, MUST_BE_GREATER_OR_EQUAL_TO, MUST_NOT_BE_NULL, MUST_BE_NULL, DATE_MUST_BE_IN_THE_PAST, MUST_MATCH_PATTERN, INVALID_SIZE, INVALID_EMAIL, MUST_NOT_BE_EMPTY, INVALID_RANGE, UNKNOWN)
#        "expected": "string",
#        "value": "string"
#        }
#    ]
#    }
#========================================================================================
sub exportProducts {
    my $self = shift;
    my ($aProducts, $IncludePrices) = @_;

    # Compile shop product data and send it to Zettle
    my $BulkExportUrl = $self->_getProductEndpoint().'/organizations/self/import/v2';
    my $hHeadersParams = {
        'Content-type' => 'application/json',
    };
    my $hContent = $self->_buildBulkExportContent($aProducts, $IncludePrices);

    #TODO: Avoid duplicates. How to proceed with existing products in Zettle? Retrieve existing products and compare externalReference to setup UUID for those who match alias
    #INFO: if $hContent contains products whose ZettleUUID is already present in Zettle platform those products will get updated
    my $HttpResponse = $self->sendPostRequest($BulkExportUrl, $hHeadersParams, $hContent);
    
    # TODO: Check for WS error responses
    # List of values for $hResponse->{errorType} and sample content
    # - VAT_NOT_ALLOWED_IN_COUNTRY
    # - INVALID_CLIENT_JSON: {"developerMessage":"Failed to deserialize JSON. Type=ProductCreateRequest, Reason=Cannot deserialize value of type `long` from String \"92.4\": not a valid `long` value\n at [Source: (String)\"{\"externalReference\":\"24721\",\"name\":\"[Example] Upango Test 001\",\"description\":\"<p>Test description</p>\",\"uuid\":\"60dee52d-3fbb-178a-236a-b976395fed89\",\"vatPercentage\":5,\"variants\":[{\"options\":[{\"value\":\"blue\",\"name\":\"Colour\"},{\"value\":\"M\",\"name\":\"Size\"},{\"value\":\"Cotton\",\"name\":\"Material\"}],\"sku\":\"9114-BMC\",\"name\":\"[Example] Upango Test 001 (BMC)\",\"price\":{\"amount\":\"92.4\",\"currencyId\":\"GBP\"},\"barcode\":\"8434639020573-BMC\",\"description\":\"<p>Test description BMC</p>\",\"uuid\":\"60dee52d-2f3c-11ec-2d2d-\"[truncated 1395 chars]; line: 1, column: 368] (through reference chain: com.izettle.product.library.api.request.ProductCreateRequest[\"variants\"]->java.util.ArrayList[0]->com.izettle.product.library.api.VariantDTO[\"price\"]->com.izettle.product.library.api.valueobject.Price[\"amount\"])","errorType":"INVALID_CLIENT_JSON","violations":[]}
    # - INTERNAL_SERVER_ERROR: {"developerMessage":"InvalidTypeException: 60d9cbce-13ad-8409-d74c-b976395f414c is not a Type 1 (time-based) UUID","errorType":"INTERNAL_SERVER_ERROR","violations":[]}
    # - UNPARSABLE_ENTITY: {"developerMessage":"Invalid input:Invalid UUID string: 2\u0012\u0011r","errorType":"UNPARSABLE_ENTITY","violations":[]}; Status=400
    # - ACCESS_TOKEN_EXPIRED: {"developerMessage":"Access token expired at: NumericDate{1626531460 -> Jul 17, 2021, 2:17:40 PM UTC}","errorType":"ACCESS_TOKEN_EXPIRED"}; Status=401
    # - VAT_MISSING_ON_VAT_USER: {"developerMessage":"VAT is mandatory. 2 products without VAT","errorType":"VAT_MISSING_ON_VAT_USER","violations":[]}; Status=400
    # - CONSTRAINT_VIOLATION:
    #    {"developerMessage":"The request entity has constraint violations","errorType":"CONSTRAINT_VIOLATION","violations":[{"developerMessage":"Multivariant product needs to contain variantOptionDefinitions","constraintType":"UNKNOWN","value":"FullProductUpdateRequest(uuid=60de1c7c-0c5d-175d-2af8-b976395f90f1, categories=[], name=UPNG_T002, ...
    #    {"developerMessage":"The request entity has constraint violations","errorType":"CONSTRAINT_VIOLATION","violations":[{"propertyName":"variants","developerMessage":"List of variants must not be empty","constraintType":"UNKNOWN","value":"FullProductUpdateRequest(uuid=60f3f266-4932-1a1e-2019-b976395fee89, categories=[], name=[Example] Framed Butterfly Painting, description=Optional[<p>This detailed drawing is an eye-catcher in any home. The drawing comes in a high-quality, black, wooden frame. Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua.</p>], imageLookupKeys=[], presentation=Optional.empty, variants=[], externalReference=Optional[23092], unitName=Optional.empty, vatPercentage=Optional[20], online=Optional.empty, variantOptionDefinitions=Optional.empty, taxCode=null, category=Optional.empty, metadata=Optional[MetadataDTO(inPos=true, source=Optional[SourceDTO(name=epages, external=true)])], taxRates=null, taxExempt=null)"}]}
    
    # {"uuid":"1aca6d65-e730-11eb-9cc8-b68971ccb386","state":"IMPORTING","items":10,"created":"2021-07-17T18:52:15.885+0000","finished":null,"errorMessage":null}
    my $hResponse = $self->decodeResponseContent($HttpResponse);
    
    my $Environment = $self->getEnvironment();
    my $InitialExportUUID = $hResponse->{uuid};
    $Environment->setupInitialExport($InitialExportUUID, $IncludePrices);

    return $hResponse;
}

#========================================================================================
# function     getInitialExportState
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->getInitialExportState();
#----------------------------------------------------------------------------------------
# description  Retrieve updated status from Zettle for bulk product export.
#----------------------------------------------------------------------------------------
# input        $ExportUUID | optional. Export UUID returned by Zettle | string
#----------------------------------------------------------------------------------------
# return       $State | state field from Zettle response: any of "IMPORTING", "FINISHED_SUCCESS", "FINISHED_FAILED" | string
#========================================================================================
sub getInitialExportState {
    my $self = shift;
    my ($ExportUUID) = @_;

    # If $State is not present, query Zettle for an updated state
    my $ExportStatusURL = $self->_getBulkExportStatusURL($ExportUUID);
    
    my $hHeadersParams = {
        'Content-type' => 'application/json',
    };

    my $HttpResponse = $self->sendGetRequest($ExportStatusURL, $hHeadersParams);

    # {"uuid":"1aca6d65-e730-11eb-9cc8-b68971ccb386", "state":"FINISHED_SUCCESS", "items":10, "created":"2021-07-17T18:52:15.885+0000", "finished":"2021-07-17T18:52:15.927+0000", "errorMessage": null}
    my $hResponse = $self->decodeResponseContent($HttpResponse);

    return $hResponse->{state};
}

#========================================================================================
# function     getProductCount
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->getProductCount();
#----------------------------------------------------------------------------------------
# description  Retrieve all of the products from Zettle.
#               See: https://products.izettle.com/swagger#/products/countAllProducts
#----------------------------------------------------------------------------------------
# return       $hResponse | Product count in a hash with the key 'productCount' | ref.hash
#========================================================================================
sub getProductCount {
    my $self = shift;

    my $Url = $self->_getProductEndpoint().'/organizations/self/products/v2/count';

    my $HttpResponse = $self->sendGetRequest($Url);
    
    my $hResponse = $self->decodeResponseContent($HttpResponse);

    return $hResponse;
}

#========================================================================================
# function     getAllProducts
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aProductResponseDTO = $self->getAllProducts();
#----------------------------------------------------------------------------------------
# description  Retrieve all of the products from Zettle.
#               See: https://products.izettle.com/swagger#/products/getAllProductsInPos
#               See: https://products.izettle.com/swagger#/products/getAllProductsV2
#----------------------------------------------------------------------------------------
# return       $aProductResponseDTO | Array of ProductResponseDTO (see DE_EPAGES::Zettle::API::WebService::DTO::ProductResponseDTO) | ref.array.object
#========================================================================================
sub getAllProducts {
    my $self = shift;

    my $Url = $self->_getProductEndpoint().'/organizations/self/products/v2';

    #TODO: include 'sort' parameter in request if passed as an argument in method call
    my $HttpResponse = $self->sendGetRequest($Url);
    # print "HttpResponse=".Data::Dumper::Dumper($HttpResponse)."\n";
    
    #TODO: check why decodeResponseContent() returns undef when processing a considerable amount of products (more than 50 products)
    # my $hResponse = $self->decodeResponseContent($HttpResponse);    
    # print "hResponse=".Data::Dumper::Dumper($hResponse)."\n";
    
    my $DecodedContent = $HttpResponse->decoded_content();
    # print "DecodedResponse=".Data::Dumper::Dumper($DecodedResponse)."\n";
    my $hResponse = decode_json($DecodedContent);

    my $aProductResponseDTO;
    foreach my $ProductResponse (@{$hResponse}){
        my $ProductResponseDTO = DE_EPAGES::Zettle::API::WebService::DTO::ProductResponseDTO->parseResponse($ProductResponse);
        push(@{$aProductResponseDTO}, $ProductResponseDTO);
    }

    return $aProductResponseDTO;
}

#========================================================================================
# function     deleteProduct
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Deleted = $self->deleteProduct($ProductUUID);
#----------------------------------------------------------------------------------------
# description  Delete a single product from Zettle.
#               See: https://products.izettle.com/swagger#/products/deleteProduct
#----------------------------------------------------------------------------------------
# input        $ProductUUID | Product UUID | string
#----------------------------------------------------------------------------------------
# return       $Deleted | TRUE if product was deleted, false if product was not found | boolean
#========================================================================================
sub deleteProduct {
    my $self = shift;
    my ($ProductUUID) = @_;

    my $Url = $self->_getProductEndpoint().'/organizations/self/products/'.$ProductUUID;

    eval {
        $self->sendDeleteRequest($Url);
    };
    if(ExistsError()) {
        my $Error = GetError();
        if($Error->vars()->{'Status'} == HTTP_NOT_FOUND){
            return 0;
        }
        Error($Error); #forward all other Errors
    }

    return 1;
}

#========================================================================================
# function     deleteProducts
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->deleteProducts($aProductUUIDs);
#----------------------------------------------------------------------------------------
# description  Delete a list of products from Zettle.
#               See: https://products.izettle.com/swagger#/products/deleteProducts
#----------------------------------------------------------------------------------------
# input        $aProductUUIDs | List of product UUIDs to delete | ref.array.string
#----------------------------------------------------------------------------------------
# return       $Deleted | TRUE if products were deleted | boolean
#========================================================================================
sub deleteProducts {
    my $self = shift;
    my ($aProductUUIDs) = @_;

    my $Url = $self->_getProductEndpoint().'/organizations/self/products';

    #TODO: Only 45 products may be safely deleted at once (in one request). Group products in batches and send as many requests as necessary.
    #INFO: URL length recommendations say maximum length should be 2048 characters which substracting 57 (delete URL length) and divided by 42 (length of "&uuid=<PRODUCT-UUID>") results in 47 approx.
    # Following recommendations we should make batches of 45 products (leaving some margin for changes in delete URL).
    my $hContent = {
        'uuid' => $aProductUUIDs
    };
    # Zettle wrongfully defined delete products API request with HTTP Parameter Pollution (HPP): multiple parameters with the same key (uuid) and different values.
    # In order to construct query parameters we use https://metacpan.org/pod/URI#$uri-%3Equery_form(-\%hash-) passing a reference to an array of strings.
    my $hOptions = {
        'content' => $hContent,
    };

    my $HttpResponse = $self->sendDeleteRequest($Url, $hOptions);

    return ($HttpResponse->code == HTTP_NO_CONTENT);
}


#========================================================================================


sub _getProductEndpoint {
    my $self = shift;

    my $System = LoadRootObject();
    my $ProductEndpoint = $System->get('Zettle_ProductEndpoint');

    return $ProductEndpoint;
}

#========================================================================================
# function     _getBulkExportStatusURL
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->_getBulkExportStatusURL($ExportUUID);
#----------------------------------------------------------------------------------------
# description  Retrieve URL used to query bulk export status
#               See: https://products.izettle.com/swagger#/import/getStatusByUuid
#               See: https://products.izettle.com/swagger#/import/getLatestImportStatus
#----------------------------------------------------------------------------------------
# input        $ExportUUID | optional. Export UUID (default: query last bulk export status) | string
#----------------------------------------------------------------------------------------
# return       $StatusURL | Export status URL | string
#========================================================================================
sub _getBulkExportStatusURL {
    my $self = shift;
    my ($ExportUUID) = @_;
    $ExportUUID //= '';
    my $ExportStatusURL = "https://products.izettle.com/organizations/self/import/status/";
    return $ExportStatusURL.$ExportUUID;
}

sub _buildBulkExportContent {
    my $self = shift;
    my ($aProducts, $ExportPrices) = @_;

    my $Environment = $self->getEnvironment();
    my $Shop = $Environment->getSite();
    my $Factory = DE_EPAGES::Zettle::API::WebService::DTO::DTOFactory->new($Shop);
    
    my @ZettleProducts = ();
    #TODO: return an array with skipped products in case we need to inform merchant
    foreach my $Product (grep {IsProductExportable($_)} @{$aProducts}) {
        my $ProductDTO = $Factory->buildProductDTO($Product, $ExportPrices);
        push(@ZettleProducts, $ProductDTO->toHash());
    }

    return { 'products' => \@ZettleProducts };
}

1;