#========================================================================================
# package      DE_EPAGES::Zettle::API::WebService::RestService
# state        public
#----------------------------------------------------------------------------------------
# description  Generic REST Service for requesting to epagesj
#========================================================================================
package DE_EPAGES::Zettle::API::WebService::RestService;

use strict;

use HTTP::Response;
use LWP::UserAgent;
use URI;

use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::EPagesJ::API::Object::RestClient;
use DE_EPAGES::WebService::API::CodecRegistry;
use DE_EPAGES::Zettle::API::Logger;
use DE_EPAGES::Zettle::API::TokenHelper;
use DE_EPAGES::Zettle::API::WebService::AuthService;

# Http verbs
use constant HTTP_GET    => 'GET';
use constant HTTP_POST   => 'POST';
use constant HTTP_PUT    => 'PUT';
use constant HTTP_DELETE => 'DELETE';

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::Zettle::API::WebService::RestService->new();
#----------------------------------------------------------------------------------------
# description  constructor
#----------------------------------------------------------------------------------------
# input        $Environment | a ZettleEnvironment object | DE_EPAGES::Zettle::API::Object:ZettleEnvironment
#----------------------------------------------------------------------------------------
# return       $self | self | DE_EPAGES::Zettle::API::WebService::RestService
#========================================================================================
sub new {
    my $class = shift;
    my ($Environment) = @_;
    my $self = bless { 'Environment' => $Environment }, $class;

    return $self;
}

#========================================================================================
# function     getEnvironment
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->getEnvironment();
#----------------------------------------------------------------------------------------
# description  Retrieve ZettleEnvironment object.
#----------------------------------------------------------------------------------------
# return       $Environment | a ZettleEnvironment object | DE_EPAGES::Zettle::API::Object:ZettleEnvironment
#========================================================================================
sub getEnvironment {
    my $self = shift;
    return $self->{Environment};
}

#========================================================================================
# function     sendGetRequest
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->sendGetRequest($Url, $hHeaders, $Authenticated);
#----------------------------------------------------------------------------------------
# description  sends an HTTP POST request
#----------------------------------------------------------------------------------------
# throws       JsonParserError, WebserviceError, HttpEncodingError
#----------------------------------------------------------------------------------------
# input        $Url | target endpoint URL | string
# input        $hHeaders | HTTP header data see <function DE_EPAGES::EPagesJ::API::Object::RestClient::new> | ref.hash
# input        $Authenticated | optional. determines if request requires authorization header (default: true) | boolean
#----------------------------------------------------------------------------------------
# return       $hResponse | response as object | HTTP::Response
#========================================================================================
sub sendGetRequest {
    my $self = shift;
    my ($Url, $hHeaders, $Authenticated) = @_;

    my %hOptions = (
        'method'  => HTTP_GET,
        'url'     => $Url,
        'headers' => $hHeaders,
    );
    $hOptions{'authenticated'} = $Authenticated if defined $Authenticated;
    my $Response = $self->_sendRequest(%hOptions);

    return $Response;
}

#========================================================================================
# function     sendPostRequest
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->sendPostRequest($Url, $hHeaders, $hContent);
#----------------------------------------------------------------------------------------
# description  sends an HTTP POST request
#----------------------------------------------------------------------------------------
# throws       JsonParserError, WebserviceError, HttpEncodingError
#----------------------------------------------------------------------------------------
# input        $Url | target endpoint URL | string
# input        $hHeaders | HTTP header data see <function DE_EPAGES::EPagesJ::API::Object::RestClient::new> | ref.hash
# input        $hContent | content data | ref.hash or arrayref
# input        $Authenticated | optional. determines if request requires authorization header (default: true) | boolean
#----------------------------------------------------------------------------------------
# return       $hResponse | response as object | HTTP::Response
#========================================================================================
sub sendPostRequest {
    my $self = shift;
    my ($Url, $hHeaders, $hContent, $Authenticated) = @_;
    
    my %hOptions = (
        'method'  => HTTP_POST,
        'url'     => $Url,
        'headers' => $hHeaders,
        'content' => $hContent,
    );
    $hOptions{'authenticated'} = $Authenticated if defined $Authenticated;
    my $Response = $self->_sendRequest(%hOptions);

    return $Response;
}

#========================================================================================
# function     sendDeleteRequest
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hResponse = $self->sendDeleteRequest($Url, $hOptions)
#----------------------------------------------------------------------------------------
# description  sends an HTTP DELETE request
#----------------------------------------------------------------------------------------
# throws       WebserviceError
#----------------------------------------------------------------------------------------
# input        $Url | target endpoint URL | string
# input        $hOptions | optional. Additional request data with the following keys:
#                          <ul>
#                            <li>headers - optional. HTTP header data see <function DE_EPAGES::EPagesJ::API::Object::RestClient::new> - ref.hash
#                            <li>content - optional. content data - ref.hash
#                            <li>authenticated - optional. determines if request requires authorization header (default: true) - boolean
#                          </ul> | ref.hash
#----------------------------------------------------------------------------------------
# return       $hResponse | response as object | HTTP::Response
#========================================================================================
sub sendDeleteRequest {
    my $self = shift;
    my ($Url, $hOptions) = @_;
    
    $hOptions //= {};
    $hOptions->{'method'} = HTTP_DELETE;
    $hOptions->{'url'} = $Url;
    my $Response = $self->_sendRequest(%{$hOptions});

    return $Response;
}


#========================================================================================
# function     decodeResponseContent
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->decodeResponseContent($HttpResponse);
#----------------------------------------------------------------------------------------
# description  Decode response content based on Content-Type.
#               Use <DE_EPAGES::WebService::API::CodecRegistry> to retrieve the appropiate <DE_EPAGES::WebService::API::Codec> object to decode HTTP content.
#----------------------------------------------------------------------------------------
# throws       HttpDecodingError
#----------------------------------------------------------------------------------------
# input        $HttpResponse | HTTP response object | HTTP::Response
#----------------------------------------------------------------------------------------
# return       $DecodedContent | decoded content | any
#========================================================================================
sub decodeResponseContent {
    my $self = shift;
    my ($HttpResponse) = @_;

    my $ContentType = $HttpResponse->content_type();
    # Default content type
    $ContentType = 'application/json' if not isValid($ContentType);
    
    my $DecodedContent;
    my $Codec = DE_EPAGES::WebService::API::CodecRegistry->get($ContentType);
    if($Codec){
        $DecodedContent = $Codec->decode($HttpResponse);
    }else{
        Error('HttpDecodingError', { 'Message' => "No codec found for Content-Type: $ContentType" });
    }
    
    return $DecodedContent;
}


#========================================================================================


#========================================================================================
# function     _encodeContent
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_encodeContent($Content, $ContentType);
#----------------------------------------------------------------------------------------
# description  Use <DE_EPAGES::WebService::API::CodecRegistry> to retrieve the appropiate <DE_EPAGES::WebService::API::Codec> object to encode HTTP content.
#----------------------------------------------------------------------------------------
# throws       HttpEncodingError
#----------------------------------------------------------------------------------------
# input        $Content | HTTP raw body | any
# input        $ContentType | Optional. HTTP content type (default: application/json) | String
#----------------------------------------------------------------------------------------
# return       $EncodedContent | encoded content | any
#========================================================================================
sub _encodeContent {
    my $self = shift;
    my ($Content, $ContentType) = @_;

    # Default content type
    $ContentType = 'application/json' if not isValid($ContentType);
    
    my $Codec = DE_EPAGES::WebService::API::CodecRegistry->get($ContentType);
    if($Codec){
        my $Message = HTTP::Message->new();
        $Codec->encode($Message, $Content);
        return $Message->content();
    }else{
        Error('HttpEncodingError', { 'Message' => "No codec found for Content-Type: $ContentType" });
    }
    
    return;
}

#========================================================================================
# function     _sendRequest
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->_sendRequest($hOptions);
#----------------------------------------------------------------------------------------
# description  sends an HTTP request
#----------------------------------------------------------------------------------------
# throws       JsonParserError, WebserviceError, HttpEncodingError
#----------------------------------------------------------------------------------------
# input        $hOptions | arguments and options used to construct HTTP request with keys:
#                          <ul>
#                              <li>method - HTTP method: get/put/post/delete - string
#                              <li>url - target endpoint URL - string
#                              <li>headers - HTTP header data see <function DE_EPAGES::EPagesJ::API::Object::RestClient::new> - ref.hash
#                              <li>content - optional. content data - ref.hash or arrayref
#                              <li>authenticated - optional. determines if request requires authorization header (default: true) - boolean
#                          </ul> | ref.hash
#----------------------------------------------------------------------------------------
# return       response as object | HTTP::Response
#========================================================================================
sub _sendRequest {
    my $self = shift;
    my %hOptions = @_;
    my $MethodName = (caller(0))[3];

    my $HttpMethod = $hOptions{'method'};
    my $Url = $hOptions{'url'};
    my $hHeaders = $hOptions{'headers'} // {};
    my $hContent = $hOptions{'content'};
    # All requests are authenticated (include Bearer token) by default, unless otherwise specified
    my $Authenticated = $hOptions{'authenticated'} // 1;

    # Add Zettle partner ID to headers
    $self->_addPartnerHeader($hHeaders);
    # Add bearer authorization to headers
    $self->_addAuthorization($hHeaders) if $Authenticated;

    my $Logger = $self->_getLogger();
    my $hLog = { 'Method' => $HttpMethod, 'URL' => $Url, 'Headers' => $hHeaders };
    $hLog->{'Content'} = $hContent if defined $hContent;
    $Logger->logCommunication($MethodName.' (HTTP Request)', $hLog);

    my $RequestUrl = URI::URL->new($Url);

    my $Response;
    eval {
        # Underlying client: DE_EPAGES::EPagesJ::API::Object::HTTPClient
        my $RestClient = $self->_getRestClient();
        if($HttpMethod eq HTTP_GET){
            $RequestUrl->query_form($hContent) if defined $hContent;
            $Response = $RestClient->client()->get($RequestUrl, $hHeaders);
        }elsif($HttpMethod eq HTTP_POST){
            my $EncodedRequest = $self->_encodeContent($hContent, $hHeaders->{'Content-type'});
            $Response = $RestClient->client()->post($RequestUrl, $hHeaders, $EncodedRequest);
        }elsif($HttpMethod eq HTTP_PUT){
            my $EncodedRequest = $self->_encodeContent($hContent, $hHeaders->{'Content-type'});
            $Response = $RestClient->client()->put($RequestUrl, $hHeaders, $EncodedRequest);
        }elsif($HttpMethod eq HTTP_DELETE){
            $RequestUrl->query_form($hContent) if defined $hContent;
            $Response = $RestClient->client()->delete($RequestUrl, $hHeaders);
        }
    };
    if ( ExistsError() ) {
        my $Error = GetError();
        $self->_handleErrors($Error);
    }
    $Logger->logCommunication($MethodName.' (HTTP Response)', $Response);
    return $Response;
}

#========================================================================================
# function     _handleErrors
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_handleErrors($Error);
#----------------------------------------------------------------------------------------
# description  handle the errors which can happen
#----------------------------------------------------------------------------------------
# throws       JsonParserError, WebserviceError, NativeDie and all other Errors
#----------------------------------------------------------------------------------------
# input        $Error | epages error object | DE_EPAGES::Core::API::Error
#========================================================================================
sub _handleErrors {
    my $self = shift;
    my ($Error) = @_;
    my $MethodName = (caller(0))[3];

    if ($Error->code() eq 'NativeDie') {
        my $Message = $Error->message();
        my $ErrorMessage_for_encodeRequest = 'hash- or arrayref expected';
        my $ErrorMessage_for_decodeResponse = 'malformed JSON string';
        if ($Message =~ /$ErrorMessage_for_encodeRequest|$ErrorMessage_for_decodeResponse/g) {
            Error('JsonParserError', { 'Message' => $Message });
        }
        else {
            Error($Error); #forward all other NativeDies
        }
    }
    elsif ($Error->code() eq 'REQUEST_FAILED') {
        my $ErrorVars = $Error->vars();
        my $Response = HTTP::Response->new($ErrorVars->{'Status'});
        $Response->content($ErrorVars->{'Response'});

        my $Logger = $self->_getLogger();
        $Logger->logCommunication($MethodName, $ErrorVars);
        Error('WebserviceError', { 'StatusCode' => $ErrorVars->{'StatusLine'}, 'Status' => $ErrorVars->{'Status'}, 'Response' => $Response->decoded_content });
    }
    else {
        Error($Error); #forward all other Errors
    }

    return;
}

#========================================================================================
sub _getRestClient {
    my $self = shift;
    my ($hSettings) = @_;
    $hSettings //= {};

    $self->{'RestClient'} //= DE_EPAGES::EPagesJ::API::Object::RestClient->create($self->getEnvironment(), $hSettings);
    # Update user-agent
    my $UserAgent = $self->{'RestClient'}->client()->userAgent();
    $UserAgent->agent('ePages/6 ' . $UserAgent->agent()) unless $UserAgent->agent() =~ m/ePages/i;

    return $self->{'RestClient'};
}

#========================================================================================
sub _getLogger {
    my $self = shift;

    $self->{'Logger'} //= DE_EPAGES::Zettle::API::Logger->new();
    return $self->{'Logger'};
}

#========================================================================================
# function     _addPartnerHeader
# state        private
#========================================================================================
# syntax       $self->_addPartnerHeader($hHeaders);
#----------------------------------------------------------------------------------------
# description  Add X-iZettle-Application-Id header.
#----------------------------------------------------------------------------------------
# input        $hHeaders | Request headers | ref.hash
#========================================================================================
sub _addPartnerHeader {
    my $self = shift;
    my ($hHeaders) = @_;
    
    my $ZettlePartnerId = $self->_getZettlePartnerId();
    $hHeaders->{'X-iZettle-Application-Id'} = $ZettlePartnerId;

    return;
}

#========================================================================================
# function     _addAuthorization
# state        private
#========================================================================================
# syntax       $self->_addAuthorization($hHeaders);
#----------------------------------------------------------------------------------------
# description  Add valid authorization header: Bearer token.
#----------------------------------------------------------------------------------------
# input        $hHeaders | Request headers | ref.hash
#========================================================================================
sub _addAuthorization {
    my $self = shift;
    my ($hHeaders) = @_;    

    # Get access token (refresh if expired).
    my $ValidToken = $self->_getValidToken();

    $hHeaders->{'Authorization'} = "Bearer $ValidToken";

    return;
}

#========================================================================================
# function     _getValidToken
# state        private
#========================================================================================
# syntax       $AccessToken = $self->_getValidToken();
#----------------------------------------------------------------------------------------
# description  Retrieve a valid access token to be used in a Zettle API call, refreshing tokens if access token is expired.
#----------------------------------------------------------------------------------------
# return       $AccessToken | valid access token | string
#========================================================================================
sub _getValidToken {
    my $self = shift;

    my $Environment = $self->getEnvironment();
    my $TokenHelper = DE_EPAGES::Zettle::API::TokenHelper->new($Environment);
    if($TokenHelper->isTokenExpired()){
        my $RefreshToken = $Environment->refreshToken();
        my $AuthService = DE_EPAGES::Zettle::API::WebService::AuthService->new($Environment);
        #TODO: handle errors
        my $hTokenInfo = $AuthService->refreshTokenData($RefreshToken);
        $TokenHelper->updateTokenData($hTokenInfo);
    }

    return $Environment->accessToken();
}

#========================================================================================
sub _getZettlePartnerId {
    my $self = shift;

    #TODO: Zettle Partner ID shall be retrieved upon auth setup using following request: GET https://oauth.izettle.com/users/me
    # PartnerId attribute needs to be created in environment too: organization partner ID (UUID v1)
    # return $self->getEnvironment()->get('PartnerId');
    return '1a52e2a8-824d-11eb-acc8-aafa2648b59f';
}

1;