#========================================================================================
# package      DE_EPAGES::Zettle::API::WebService::VariationOptionHelper
# state        public
#----------------------------------------------------------------------------------------
# description  Helper functions to generate variation attribute related information for Zettle API.
#========================================================================================
package DE_EPAGES::Zettle::API::WebService::VariationOptionHelper;
use base Exporter;

use strict;

our @EXPORT_OK = qw (
    BuildOptionDefinitions
    BuildVariantOptions
);

#========================================================================================
# function     BuildOptionDefinitions
# state        public
#----------------------------------------------------------------------------------------
# syntax       BuildOptionDefinitions( $Product );
#----------------------------------------------------------------------------------------
# description  Parse $Product VariationTypeValues and generate data according to Zettle API variantOptionDefinitions attribute
#----------------------------------------------------------------------------------------
# input        $Product | a super product | DE_EPAGES::Product::API::Object::Product
#----------------------------------------------------------------------------------------
# return       $ahOptions | Superproduct "variant option definitions" as expected by Zettle API | ref.array.hash
# example
# [
#     {
#         "name": "Colour",
#         "properties": [
#             {
#                 "value": "red",
#                 "imageUrl": null
#             },
#             {
#                 "value": "green",
#                 "imageUrl": null
#             }
#         ]
#     },
#     ...
# ]
#========================================================================================
sub BuildOptionDefinitions {
    my ($Product) = @_;

    my $Shop = $Product->getSite();
    my $LanguageID = $Shop->get('LanguageID');
  
    my $aVariationTypeValues = $Product->get('VariationTypeValues');
    if (scalar @$aVariationTypeValues == 0){
        return; # this is not a master product
    }

    my @OptionDefinitions;
    foreach my $VariationAttributeValue (@{$aVariationTypeValues}){
        my $Attribute = $VariationAttributeValue->{'Attribute'};

        my @Properties;
        my $aAttributeValues = $VariationAttributeValue->{'Values'};
        foreach my $AttributeValue (@{$aAttributeValues}){
            my $Value = $AttributeValue->get('ValueOrAlias', $LanguageID);
            push(@Properties, {
                'value' => $Value,
                'imageUrl' => undef,
            });
        }

        my $AttributeAlias = $Attribute->get('NameOrAlias', $LanguageID);
        push(@OptionDefinitions, {
            'name' => $AttributeAlias,
            'properties' => \@Properties,
        });
    }

    return \@OptionDefinitions;
}

#========================================================================================
# function     BuildVariantOptions
# state        public
#----------------------------------------------------------------------------------------
# syntax       BuildVariantOptions( $SubProduct );
#----------------------------------------------------------------------------------------
# description  Build subproduct variant options (from variation attributes).
#----------------------------------------------------------------------------------------
# input        $SubProduct | a sub product | DE_EPAGES::Product::API::Object::Product
#----------------------------------------------------------------------------------------
# return       $ahVariations | Array of variant options as expected by Zettle API | ref.array.hash
# example
# [
#     { 'name' => 'Colour', 'value' => 'Blue' },
#     { 'name' => 'Size', 'value' => 'M' },
#     { 'name' => 'Material', 'value' => 'Cotton' },
# ]
#========================================================================================
sub BuildVariantOptions {
    my ($SubProduct) = @_;

    my @Options = ();
    my $SuperProduct = $SubProduct->get('SuperProduct');
    return \@Options unless defined $SuperProduct; # Test it's a SubProduct
    
    my $OptionDefinitions = BuildOptionDefinitions($SuperProduct);
    my @aOptionNames = map {$_->{name} } @{$OptionDefinitions};
    
    #TODO: why does NonVariationUserDefinedAttributeValues match?
    my $aUserDefinedAttributeValues = $SubProduct->get('UserDefinedAttributeValues');
    foreach my $AttributeValue (@{$aUserDefinedAttributeValues}){
        my $Alias = $AttributeValue->{Attribute}->{Attributes}->{Alias};
        my $Value = $AttributeValue->{Value}->{Attributes}->{Alias};
        push(@Options, {
            'name' => $Alias,
            'value' => $Value,
        }) if grep { /^$Alias$/ } @aOptionNames;
    }

    return \@Options;
}

1;
