#========================================================================================
# package      DE_EPAGES::Zettle::API::ZettleService
# state        public
#----------------------------------------------------------------------------------------
# description  object for calling functions on the ZettleService which serves as a Zettle facade
#========================================================================================
package DE_EPAGES::Zettle::API::ZettleService;

use strict;

use Data::Compare;

use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::Core::API::TestParameter  qw( TestObject );
use DE_EPAGES::Object::API::Factory  qw( ExistsObjectByGUID LoadObjectByGUID );
use DE_EPAGES::Zettle::API::Constants qw( %INITIAL_EXPORT_STATUS );
use DE_EPAGES::Zettle::API::TokenHelper;
use DE_EPAGES::Zettle::API::WebService::AuthService;
use DE_EPAGES::Zettle::API::WebService::MerchantService;
use DE_EPAGES::Zettle::API::WebService::ProductService;
use DE_EPAGES::Zettle::API::WebService::VariationOptionHelper qw ( BuildVariantOptions );


#ERRORDEFINITION
sub ErrorDefinition { return {}
}
#ERRORDEFINITION END

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::Zettle::API::ZettleService->new($Environment);
#----------------------------------------------------------------------------------------
# description  creates a new ZettleService object
#----------------------------------------------------------------------------------------
# input        $Environment | the Zettle environment object | DE_EPAGES::Zettle::API::Object:ZettleEnvironment
#----------------------------------------------------------------------------------------
# return       $self | self | DE_EPAGES::Zettle::API::ZettleService
#========================================================================================
sub new {
    my $class = shift;
    my ($Environment)  = @_;
    TestObject('ZettleEnvironment', $Environment, 'DE_EPAGES::Zettle::API::Object::ZettleEnvironment');
    
    my $self  = bless {
        'Environment' => $Environment
    }, $class;

    return $self;
}

#========================================================================================
# function     getEnvironment
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Environment = $self->getEnvironment();
#----------------------------------------------------------------------------------------
# description  Returns the Zettle Environment object.
#----------------------------------------------------------------------------------------
# return       $Environment | Zettle Environment | Object
#========================================================================================
sub getEnvironment {
    return shift->{'Environment'};
}

#========================================================================================
# function     connect
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->connect($AuthCode);
#----------------------------------------------------------------------------------------
# description  Use given $AuthCode to retrieve tokens from Zettle OAuth API and create/update zettle environment storing token data.
#               See also <function DE_EPAGES::Zettle::API::WebService::AuthService::getTokensOnAuthCode>.
#========================================================================================
sub connect {
    my $self = shift;
    my ($AuthCode) = @_;

    my $Environment = $self->getEnvironment();

    # Use OAuth service to exchange AuthCode for Access and Refresh tokens
    my $AuthService = DE_EPAGES::Zettle::API::WebService::AuthService->new($Environment);
    my $hTokens = $AuthService->getTokensOnAuthCode($AuthCode);
    
    my $TokenHelper = $self->_createTokenHelper($Environment);
    $TokenHelper->updateTokenData($hTokens);

    my $MerchantService = DE_EPAGES::Zettle::API::WebService::MerchantService->new($Environment);
    my $AccountInfo = $MerchantService->getAccountInfo();
    
    my $ContactEmail = $AccountInfo->{'contactEmail'};
    my $ReceiptEmail = $AccountInfo->{'receiptEmail'};
    my %MerchantInfo = (
        'uuid'     => $AccountInfo->{'uuid'},
        'email'    => $ContactEmail // $ReceiptEmail,
        'country'  => $AccountInfo->{'country'},
        'language' => $AccountInfo->{'language'},
        'currency' => $AccountInfo->{'currency'},
    );
    $Environment->setMerchantInfo(%MerchantInfo);
}

#========================================================================================
# function     disconnect
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->disconnect();
#----------------------------------------------------------------------------------------
# description  Disconnect from Zettle API and remove token data from zettle environment
#========================================================================================
sub disconnect {
    my $self = shift;

    my $Environment = $self->getEnvironment();
    my $AuthService = DE_EPAGES::Zettle::API::WebService::AuthService->new($Environment);
    eval {
        $AuthService->disconnect();
    };
    # Ignore errors like token expired or token already deleted
    if ( ExistsError() ) {
        my $Error = GetError();
        LogError($Error);
    }

    my $TokenHelper = $self->_getTokenHelper();
    $TokenHelper->deleteTokenData();

    # Configure ZettleEnvironment so that batch job proceeds with deletion afterwards
    $Environment->markForDeletion(1);
}

#========================================================================================
# function     isTokenAcquired
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->isTokenAcquired();
#----------------------------------------------------------------------------------------
# description  Check if tokens were retrieved from Zettle
#               See <function DE_EPAGES::Zettle::API::TokenHelper::isTokenAcquired>
#----------------------------------------------------------------------------------------
# return        TRUE if tokens present in ZettleEnvironment, false otherwise
#========================================================================================
sub isTokenAcquired {
    my $self = shift;    
    
    my $TokenHelper = $self->_getTokenHelper();
    return $TokenHelper->isTokenAcquired();
}

#========================================================================================
# function     initialExport
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->initialExport();
#----------------------------------------------------------------------------------------
# description  Run an initial product export which will send to Zettle all products that match criteria, and update InitialExportStatus in environment.
#               See <function DE_EPAGES::Zettle::API::WebService::ProductService::exportProducts>.
#               See <function DE_EPAGES::Zettle::API::WebService::DTO::Requirements::IsProductExportable>.
#----------------------------------------------------------------------------------------
# input       $ReplaceZettleLibrary | Determines whether all products will be removed from Zettle before export | boolean
# input       $IncludePrices | Determines if prices will be included in product export | boolean
#----------------------------------------------------------------------------------------
# return        Number of items to be processed by Zettle.
#========================================================================================
sub initialExport {
    my $self = shift;
    my ($ReplaceZettleLibrary, $IncludePrices) = @_;

    my $Environment = $self->getEnvironment();
    my $ProductService = DE_EPAGES::Zettle::API::WebService::ProductService->new($Environment);

    if ($ReplaceZettleLibrary){
        $self->_deleteLibrary();
    }else{
        # Use product info retrieved from Zettle to fill in product ZettleUUID attribute for previously exported products/variations
        my $aProductResponseDTO = $ProductService->getAllProducts();
        $self->_updateShopProductsUUID($aProductResponseDTO);
    }

    my $Shop = $Environment->getSite();
    my $ShopProducts = $Shop->get('Products');
    my $hResponse = $ProductService->exportProducts($ShopProducts, $IncludePrices);
    
    my $State = $hResponse->{'state'};
    my $Status = $self->_getStatusFromState($State);
    $Environment->initialExportStatus($Status);

    return $hResponse->{items};
}

#========================================================================================
# function     isInitialExportComplete
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->isInitialExportComplete();
#----------------------------------------------------------------------------------------
# description  Check if initial product export has been completed
#----------------------------------------------------------------------------------------
# return        TRUE if completed, false otherwise
#========================================================================================
sub isInitialExportComplete {
    my $self = shift;
    my $InitialExportStatus = $self->getInitialExportStatus();
    return ($InitialExportStatus == $INITIAL_EXPORT_STATUS{COMPLETED}) ? 1 : 0;
}

#========================================================================================
# function     getInitialExportStatus
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->getInitialExportStatus();
#----------------------------------------------------------------------------------------
# description  Retrieve updated initial export status querying Zettle if necessary (if export was RUNNING), updating environment export status is status changed.
#----------------------------------------------------------------------------------------
# return       $ExportStatus | See %INITIAL_EXPORT_STATUS values in <DE_EPAGES::Zettle::API::Constants> | integer
#========================================================================================
sub getInitialExportStatus {
    my $self = shift;
    my $Environment = $self->getEnvironment();
    my $InitialExportStatus = $Environment->initialExportStatus();

    # If last known status was 'running' we query Zettle for updated status
    if($InitialExportStatus == $INITIAL_EXPORT_STATUS{RUNNING}){
        my $ProductService = DE_EPAGES::Zettle::API::WebService::ProductService->new($Environment);
        my $ExportUUID = $Environment->initialExportUUID();
        my $State = $ProductService->getInitialExportState($ExportUUID);
        $InitialExportStatus = $self->_getStatusFromState($State);
        # Update export status in environment if status changed
        $Environment->initialExportStatus($InitialExportStatus) unless $InitialExportStatus == $INITIAL_EXPORT_STATUS{RUNNING};
    }

    return $InitialExportStatus;
}


#========================================================================================


#========================================================================================
# function     _updateShopProductsUUID
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_updateShopProductsUUID($aProductResponseDTO);
#----------------------------------------------------------------------------------------
# input        $aProductResponseDTO | Array of ProductResponseDTO (see DE_EPAGES::Zettle::API::WebService::DTO::ProductResponseDTO) | ref.array.object
#----------------------------------------------------------------------------------------
# description  Use product info retrievd from Zettle to fill in product ZettleUUID attribute for products previously exported to Zettle.
#               The goal is to avoid generating product duplicates in Zettle given that UUID information is the unique identifier used in Zettle API requests.
#               This let's us reuse previously generated UUIDs instead of generating new ones, which would end up with duplicates in Zettle.
#----------------------------------------------------------------------------------------
# return       $hUpdatedProducts | hash of updated products where key is product/variation ID and value is updated UUID | ref.hash
#========================================================================================
sub _updateShopProductsUUID {
    my $self = shift;
    my ($aProductResponseDTO) = @_;

    # Hash of product UUID updates
    my $hUpdatedProducts = {};
    foreach my $ProductResponseDTO (@{$aProductResponseDTO}){
        my $ZettleProductUuid = $ProductResponseDTO->uuid();
        # If product was exported from the shop, externalReference will contain product GUID
        my $ZettleExternalReference = $ProductResponseDTO->externalReference();
        if(defined $ZettleExternalReference && ExistsObjectByGUID($ZettleExternalReference)){
            my $Product = LoadObjectByGUID($ZettleExternalReference);
            #TODO: TestObject('Product', $Product, 'DE_EPAGES::Product:API::Object:Product'); # Ensure object is a product
            $Product->set({'ZettleUUID' => $ZettleProductUuid});
            $hUpdatedProducts->{$Product->id} = $ZettleProductUuid;
            
            # Also update UUID for product variations
            #TODO: create VariationResponseDTO object and use it here ($ahVariationDTOs)
            my $ahRawVariations = $ProductResponseDTO->variations();
            foreach my $hRawVariation (@{$ahRawVariations}){                
                # my $ZettleVariationUuid = $VariationDTO->uuid();
                my $ZettleVariationUuid = $hRawVariation->{'uuid'};
                my $ZettleVariationOptions = $hRawVariation->{'options'};
                
                # NOTE: Zettle variations don't have externalReference field so the best we can do is match by variation attribute options.
                my $SubProductMatch = $self->_matchSubProductByOptions($Product, $ZettleVariationOptions);
                if(defined $SubProductMatch){
                    $SubProductMatch->set({'ZettleUUID' => $ZettleVariationUuid});
                    $hUpdatedProducts->{$SubProductMatch->id} = $ZettleVariationUuid;
                }
            }
        }
    }

    return $hUpdatedProducts;
}


#========================================================================================
# function     _matchSubProductByOptions
# state        private
#----------------------------------------------------------------------------------------
# syntax       $SubProduct = $self->_matchSubProductByOptions($Product, $ZettleVariationOptions);
#----------------------------------------------------------------------------------------
# input        $Product | a super product | DE_EPAGES::Product::API::Object::Product
# input        $ZettleVariationOptions | attribute values for a zettle variation | ref.array.hash
#               [
#                   { 'name' => 'Colour', 'value' => 'Blue' },
#                   { 'name' => 'Size', 'value' => 'M' },
#                   { 'name' => 'Material', 'value' => 'Cotton' },
#               ]
#----------------------------------------------------------------------------------------
# description  Search SubProducts of given $Product and return the one whose variation attribute options match $ZettleVariationOptions.
#----------------------------------------------------------------------------------------
# return       $SubProduct | a SubProduct that matches given options or undef | DE_EPAGES::Product::API::Object::Product
#========================================================================================
sub _matchSubProductByOptions {
    my $self = shift;
    my ($Product, $ZettleVariationOptions) = @_;

    foreach my $SubProduct (@{$Product->get('SubProducts')}){
        my $SubProductOptions = BuildVariantOptions($SubProduct);
        # We need to specify full Compare package to avoid perl critic complaining about PerlTools not 'use'd
        return $SubProduct if Data::Compare::Compare($SubProductOptions, $ZettleVariationOptions);
    }

    return;
}

#========================================================================================
# function     _deleteLibrary
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_deleteLibrary();
#----------------------------------------------------------------------------------------
# description  Delete all products from Zettle library.
#========================================================================================
sub _deleteLibrary {
    my $self = shift;

    my $Environment = $self->getEnvironment();
    my $ProductService = DE_EPAGES::Zettle::API::WebService::ProductService->new($Environment);
    
    # Retrieve existing products
    my $aProductResponseDTO = $ProductService->getAllProducts();
    my @aProductUUIDs;
    foreach my $ProductResponseDTO (@{$aProductResponseDTO}){
        my $ProductUUID = $ProductResponseDTO->uuid;
        push(@aProductUUIDs, $ProductUUID);
    }
    # Delete products retrieved
    $ProductService->deleteProducts(\@aProductUUIDs) if scalar(@aProductUUIDs);
}

#========================================================================================
sub _getTokenHelper {
    my $self = shift;
    my ($Environment) = $self->getEnvironment();

    $self->{'TokenHelper'} //= $self->_createTokenHelper($Environment);
    return $self->{'TokenHelper'};
}

#========================================================================================
sub _createTokenHelper {
    my $self = shift;
    my ($Environment) = @_;
    $self->{'TokenHelper'} //= DE_EPAGES::Zettle::API::TokenHelper->new($Environment);
    return $self->{'TokenHelper'};
}

#========================================================================================
sub _getStatusFromState {
    my $self = shift;
    my ($State) = @_;

    my $ExportStatus;
    if($State eq 'IMPORTING'){
        $ExportStatus = $INITIAL_EXPORT_STATUS{RUNNING};
    }elsif($State eq 'FINISHED_SUCCESS'){
        $ExportStatus = $INITIAL_EXPORT_STATUS{COMPLETED};
    }elsif($State eq 'FINISHED_FAILED'){
        $ExportStatus = $INITIAL_EXPORT_STATUS{FAILED};
    }

    return $ExportStatus;
}

1;
