#========================================================================================
# package      DE_EPAGES::Zettle::API::ZettleUrlBuilder
# state        public
#----------------------------------------------------------------------------------------
# description  Build return URL for Backlink
#========================================================================================
package DE_EPAGES::Zettle::API::ZettleUrlBuilder;

use strict;

use URI;
use URI::URL;

use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Core::API::Url qw ( ParseQueryString );
use DE_EPAGES::Object::API::Factory qw( LoadRootObject );
use DE_EPAGES::Shop::API::Url qw ( BuildShopUrl );
use DE_EPAGES::Zettle::API::Constants qw ( APP_SCOPES );

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::Zettle::API::ZettleUrlBuilder->new();
#----------------------------------------------------------------------------------------
# description  constructor
#----------------------------------------------------------------------------------------
# return       $self | self | DE_EPAGES::Zettle::API::ZettleUrlBuilder
#========================================================================================
sub new {
    my $class = shift;
    my $self  = bless {}, $class;
    return $self;
}

#========================================================================================
# function     buildAdminUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->buildAdminUrl();
#----------------------------------------------------------------------------------------
# description  Retrieve the administration URL to access Zettle merchant account.
#----------------------------------------------------------------------------------------
# return       Zettle administration URL | string
#========================================================================================
sub buildAdminUrl {

    return shift->_loadSystemAttribute('Zettle_AdminURL');
}

#========================================================================================
# function     buildConnectUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->buildConnectUrl($Servlet, $Shop);
#----------------------------------------------------------------------------------------
# description  Build the connect URL used for on-boarding to Zettle
#----------------------------------------------------------------------------------------
# input        $Servlet | a Servlet object  | DE_EPAGES::Presentation::Servlet::Permission
# input        $Shop | a Shop object  | DE_EPAGES::Shop::API::Object::Shop
#----------------------------------------------------------------------------------------
# return       $URL->as_string() | Zettle connect url | string
#========================================================================================
sub buildConnectUrl {
    my $self = shift;
    my ($Servlet, $Shop) = @_;

    my $StateURL = $self->_buildStateURL($Servlet, $Shop);

    my $hQueryParameters = $self->_buildQueryParameters($StateURL);

    my $url = $self->_buildEncodedOnboardingUrl($hQueryParameters);

    return $url;
}

#========================================================================================
# function     buildRedirectToMboUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->buildRedirectToMboUrl( $hQueryParameters );
#----------------------------------------------------------------------------------------
# description  Build redirect to MBO URL using returned data from Zettle
#----------------------------------------------------------------------------------------
# input        $hQueryParameters | Parameters as returned from Zettle with keys
#               <ul>
#                   <li>code
#                   <li>state
#               </ul> | ref.hash
#----------------------------------------------------------------------------------------
# throws       NoBacklinkFound   If $hQueryParameters does not contain 'state' parameter
#----------------------------------------------------------------------------------------
# return       $URL->as_string() | BacklinkUrl with AuthorizationCode or Error | string
#========================================================================================
sub buildRedirectToMboUrl {
    my $self = shift;
    my ($hQueryParameters) = @_;
    my $aBacklinkUrl = $hQueryParameters->{'state'};

    if ( (!isValid($aBacklinkUrl)) or (!isValid($aBacklinkUrl->[0])) ) {
        Error('NoURLFound');
    }

    return $self->_buildRedirectUrl($hQueryParameters);
}

#========================================================================================
# function     buildBacklinkRedirectUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->buildBacklinkRedirectUrl();
#----------------------------------------------------------------------------------------
# description  Build backlink from 'Zettle_AppRedirectURL' system attribute (if present) or build from current domain (fallback)
# example      https://<DOMAIN>/epages/Site.admin?ChangeAction=BacklinkZettleOnboarding)
#----------------------------------------------------------------------------------------
# return       Retrieve backlink used by Zettle to return information to the shop (URL to be sent in 'state' parameter during OAuth) | string
#========================================================================================
sub buildBacklinkRedirectUrl {
    my $self = shift;

    # 1) Load redirect URL from system attribute (if present)
    my $RedirectUrl = $self->_loadSystemAttribute('Zettle_AppRedirectURL');
    return $RedirectUrl if isValid($RedirectUrl);
    
    # 2) Fallback: build redirect URL from current domain
    $RedirectUrl = $self->_loadSystemAttribute('ProtocolAndServerSSL');
    $RedirectUrl .= '/epages/Site.admin?ChangeAction=BacklinkZettleOnboarding';

    return $RedirectUrl;
}

#========================================================================================

# Retrieve MBO URL for backlink 'state' parameter, including ViewAction and ObjectID
# Sample: https://<SERVER>/epages/<SHOPNAME>.admin?ObjectID=<OBJECT_ID>&ViewAction=<VIEW_ACTION>
sub _buildStateURL {
    my $self = shift;
    my ($Servlet, $Shop) = @_;

    my $hParams = {
        # Action that will handle backlink and process auth code for OAuth flow
        'ChangeAction' => 'UnityMBO-ZettleOAuthCallback',
        'ObjectID'   => $Shop->id,
    };
    my $hOptions = {
        'Type' => 'admin',
        'UseSSL' => 1,
        'AddToken' => 1,
    };

    return BuildShopUrl($Shop, $hParams, $hOptions, $Servlet);
}

# Retrieve a hash with all query parameters required for backlink
sub _buildQueryParameters {
    my $self = shift;
    my ($stateURL) = @_;

    my $ClientID = $self->_loadSystemAttribute('Zettle_AppClientID');
    my $redirectUrl = $self->buildBacklinkRedirectUrl();

    return {
        'response_type'   => 'code',
        'client_id'       => $ClientID,
        'scope'           => join(' ', APP_SCOPES),
        'redirect_uri'    => $redirectUrl,
        'state'           => $stateURL,
    };
}

# Encode URL with parameters
sub _buildEncodedOnboardingUrl {
    my $self = shift;
    my ($hQueryParameters) = @_;

    my $ZettleOAuthURL = $self->_loadSystemAttribute('Zettle_AuthEndpoint'); #'https://oauth.izettle.com/authorize';

    my $Url = URI::URL->new($ZettleOAuthURL);
    $Url->query_form($hQueryParameters);
    my $hBacklinkUrlParameters = ParseQueryString($Url->equery(), 'utf-8');
    $Url->query_form( $hBacklinkUrlParameters );

    return $Url;
}

# Build redirect URL from data returned by Zettle Onboarding process
sub _buildRedirectUrl {
    my $self = shift;
    my ($hQueryParameters) = @_;

    # Original Change action must be removed
    delete($hQueryParameters->{'ChangeAction'});

    # State contains the URL which is backlink to MBO. No need to send it with QueryString again
    my $BackLinkUrl = delete($hQueryParameters->{'state'});
    my $URL = URI->new($BackLinkUrl->[0]);
    my $hBacklinkUrlParameters = ParseQueryString($URL->query(), 'utf-8');
    
    my $hQueryParam = {%{$hBacklinkUrlParameters}, %{$hQueryParameters}};
    $URL->query_form($hQueryParam);

    return $URL->as_string();
}

# Return the value of the given system attribute
sub _loadSystemAttribute {
    my $self = shift;
    my ($AttributeName) = @_;

    my $System = LoadRootObject();
    my $AttributeValue = $System->get($AttributeName);

    return $AttributeValue;
}

1;