#==============================================================================
# package      DE_EPAGES::Zettle::UI::ZettleEnvironment
# state        private
#==============================================================================
package DE_EPAGES::Zettle::UI::ZettleEnvironment;
use base DE_EPAGES::Presentation::UI::Object;

use strict;

use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::Zettle::API::Constants qw ( ALLOWED_EXPORT_INTERVALS %INITIAL_EXPORT_STATUS );
use DE_EPAGES::Zettle::API::ZettleService;

sub ViewZettleGeneral {
    my $self = shift;
    my ($Servlet) = @_;

    my $Environment = $Servlet->object();

    my $ZettleService = DE_EPAGES::Zettle::API::ZettleService->new($Environment);
    my $InitialExportStatus = $ZettleService->getInitialExportStatus();
    # See API/Constants::%INITIAL_EXPORT_STATUS
    $Servlet->vars('IsInitialExportCompleted', $InitialExportStatus == $INITIAL_EXPORT_STATUS{COMPLETED});
    
    #TODO: check WS state
    $Servlet->vars('WebServiceError', 0);

    $self->SUPER::View($Servlet);

    return;
}

sub ViewZettleSettings {
    my $self = shift;
    my ($Servlet) = @_;

    my $Environment = $Servlet->object();

    my $UrlBuilder = DE_EPAGES::Zettle::API::ZettleUrlBuilder->new();
    my $Shop = $Environment->getSite();
    my $ConnectUrl = $UrlBuilder->buildConnectUrl($Servlet, $Shop);
    $Servlet->vars('ConnectUrl', $ConnectUrl);
        
    my $AdminUrl = $UrlBuilder->buildAdminUrl();
    $Servlet->vars('ZettleAdminUrl', $AdminUrl);

    my $ZettleService = DE_EPAGES::Zettle::API::ZettleService->new($Environment);
    # See API/Constants::%INITIAL_EXPORT_STATUS
    my $InitialExportStatus = $ZettleService->getInitialExportStatus();
    $Servlet->vars('IsInitialExportCompleted', $InitialExportStatus == $INITIAL_EXPORT_STATUS{COMPLETED});
    $Servlet->vars('IsInitialExportRunning', $InitialExportStatus == $INITIAL_EXPORT_STATUS{RUNNING});

    #TODO: test WS state
    $Servlet->vars('WebServiceError', 0);

    #TODO: check if some initial export errors were registered
    $Servlet->vars('ZettleExportErrors', 0);

    $self->SUPER::View($Servlet);

    return;
}

sub ZettleUpdateConfig {
    my $self = shift;
    my ($Servlet) = @_;
    
    my $Environment = $Servlet->object();
    my $Form = $Servlet->form();

    # Verify export checkboxes
    my $ProductExport = $Form->formValue('ProductExport');
    my $FormPriceExport = $Form->formValue('PriceExport');
    my $ExportProductPrices = 0;
    $ExportProductPrices = 1 if defined $FormPriceExport && $FormPriceExport eq 'on';

    # Retrieve ProductExportInterval when ProductScheduledExport is enabled
    my $ExportScheduleInterval = 0;
    if($ProductExport == 1){
        $ExportScheduleInterval = $Form->formValue('ExportScheduleInterval');
        return unless $ExportScheduleInterval ~~ ALLOWED_EXPORT_INTERVALS;
    }

    $Environment->set({
        'ExportScheduleInterval' => $ExportScheduleInterval,
        'ExportProductPrices'   => $ExportProductPrices,
    });

    return;
}

#========================================================================================
# function     ZettleDisconnect
# state        public
#----------------------------------------------------------------------------------------
# syntax       ZettleDisconnect($Servlet);
#----------------------------------------------------------------------------------------
# description  Trigger disconnect for Zettle
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::Permission
#========================================================================================
sub ZettleDisconnect {
    my $self = shift;
    my ($Servlet) = @_;

    my $Environment = $Servlet->object;

    my $ZettleService = DE_EPAGES::Zettle::API::ZettleService->new($Environment);
    $ZettleService->disconnect();

    $Servlet->vars( 'ContentType', 'text/x-json' );
    my $hData = {
        result => 'OK'
    };
    $Servlet->writeContentOutput($hData);
    $Servlet->vars( 'ExitAfterEvent', 1 );
    $Servlet->vars( 'ResponseContentProcessed', 1 );

    return;
}

sub ZettleInitialExport {
    my $self = shift;
    my ($Servlet) = @_;

    my $Environment = $Servlet->object;
    my $Form = $Servlet->form;

    # Read form settings
    my $ExportSettings;
    foreach my $ZettleAttribute (qw(ReplaceZettleLibrary ExportProductPrices)) {
        $ExportSettings->{$ZettleAttribute} = $Form->value($ZettleAttribute) ? 1 : 0;
    }

    # Update initial export settings in Zettle environment
    $Environment->set($ExportSettings);
    
    # Run initial product export (asynchronous process)
    eval {
        my $ZettleService = DE_EPAGES::Zettle::API::ZettleService->new($Environment);
        my $ReplaceZettleLibrary = $ExportSettings->{'ReplaceZettleLibrary'};
        my $ExportPrices = $ExportSettings->{'ExportProductPrices'};
        $ZettleService->initialExport($ReplaceZettleLibrary, $ExportPrices);
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogError($Error);
        $Servlet->vars('ZettleExportErrors', 1);
    }else{
        $Servlet->vars('IsInitialExportCompleted', 0);
        $Servlet->vars('IsInitialExportRunning', 1);
    }

    return;
}

1;
